//#############################################################################
//                                                                           //
// M_CURVE.CPP                                                               //
// Subsystem: Utilities - Datenbank (Diffraktometrie/ Reflektometrie)        //
// Benutzung durch andere Subsysteme erforderlich: JA                        //
// Deklaration: DATAVISA.H                                                   //
//                                                                           //
//#############################################################################

#include "internls\evrythng.h" // GermanVersion
#include "datavisa\datavisa.h"

//#############################################################################
// sprachspezische Konstanten
//#############################################################################

#ifdef GermanVersion
  static const char _FreeingDataBase[] = "Datenbasis wird freigegeben, bitte haben Sie etwas Geduld...";
  static const char _NoMemory[]        = "Fehler, nicht genuegend Speicher!";
#else
  static const char _FreeingDataBase[] = "freeing database, please stand by...";
  static const char _NoMemory[]        = "Failure, not enough memory!";
#endif

//##############################################################################
// TCurve
//##############################################################################

// -----------------------------------------------------------------------------
// hier werden die Messwerte eines Scans abgelegt
// x-Wert = Motorwinkel (standard/continuous)
//        = Omegamotorwinkel (Omega2Theta)
// y-wert = gemessene Intensitaet des Detektors in dieser Position
// z-wert = 0 (standard/continuous) kein Monitor ausgewaehlt
//        = Monitorintensitaet (standard) Monitor ausgewaehlt
//        = Thetamotorwinkel (Omega2Theta)
// -----------------------------------------------------------------------------
// Konstruktor
TCurve::TCurve( void ) : m_PointList(0, 512), m_MinList(3, 1000000.0, 3), m_MaxList(3, -1000000.0, 3)
{
	// m_bStreamOpen = FALSE;
	m_ActualIdx   = -1;
	m_bEnd = FALSE;
};

// -----------------------------------------------------------------------------
// Destruktor
TCurve::~TCurve( void )
{
	New();
};

// -----------------------------------------------------------------------------
// Kurve auf Initialwerte zuruecksetzen
BOOL TCurve::New( void )
{
	m_ActualIdx   = -1;
	// m_bEnd = FALSE;
	m_MaxList.SetAt(0, -1000000.0);
	m_MaxList.SetAt(1, -1000000.0);
	m_MaxList.SetAt(2, -1000000.0);

	m_MinList.SetAt(0, 1000000.0);
	m_MinList.SetAt(1, 1000000.0);
	m_MinList.SetAt(2, 1000000.0);

	m_PointList.Clear();
	return TRUE;
};

// -----------------------------------------------------------------------------
// Index des aktuellen Punktes (Tripels) um 1 zuruecksetzen und 'Kurvenstatus' auf nicht fertig setzen
void TCurve::BackStep( void )
{
	m_bEnd = FALSE;
	if ( m_ActualIdx > 0 ) --m_ActualIdx;
};

// -----------------------------------------------------------------------------
// Zuweisungsoperator fuer Kurven
TCurve& TCurve::operator=( TCurve& crv )
{
	// wenn Kurve sich selbst zugewiesen dann Pointer auf Kurve zurueckgeben
	if ( this == &crv ) return *this;

	// alle Parameter zuruecksetzen
	m_bEnd = FALSE;
	m_ActualIdx = -1;
	
	m_MinList = crv.m_MinList;
	m_MaxList = crv.m_MaxList;
	m_PointList = crv.m_PointList;

	return *this;
};

// -----------------------------------------------------------------------------
// 06.05.2004 redundant zu PGet, deshalb ueberall durch PGet ersetzt
// liest ein Wertetripel aus Kurve aus (am aktuellen Index)
// Funktion verzichtet auf allgem. Aufruf von FastOpen und FastClose und
// ermglicht so schnelle Abfolge von Punktauslesungen
/* BOOL TCurve::FastPGet( float &x, float &y, float &z )
{
	// Abbruch wenn Kurvenende erreicht oder Kurve keine Punkte hat oder
	// wenn Kurve nicht offen und auch nicht zu oeffnen
	if ( (m_bEnd) || ( m_PointList.GetCount() == 0 ) || ((!m_bStreamOpen) && (!FastOpen())) ) return FALSE;
	
	x = y = z = 0.0; // bei Fehler wenigstens 0 zurueckliefern 

	// Abbruch wenn Kurvenende erreicht oder Kurve keine Punkte hat oder
	// wenn Kurve nicht offen und auch nicht zu oeffnen
	if ( (m_bEnd) || ( m_PointList.GetCount() == 0 ) || ((!m_bStreamOpen) && (!FastOpen())) ) return FALSE;

	if ( !(m_PointList.GetAt(m_ActualIdx).Valid) ) return FALSE;
	x = m_PointList.GetAt(m_ActualIdx).Pt.GetAt(0);
	y = m_PointList.GetAt(m_ActualIdx).Pt.GetAt(1);
	z = m_PointList.GetAt(m_ActualIdx).Pt.GetAt(2); 

	// ist Kurvenende bei naechstem Aufruf erreicht
	m_bEnd = ( m_ActualIdx == ((m_PointList.GetCount())-1) );
	if ( !m_bEnd ) m_ActualIdx++;
}; */

// -----------------------------------------------------------------------------
// gibt ein Wertetripel aus der Kurve zurueck 
BOOL TCurve::PGet( float &x, float &y, float &z )
{
	// merkt sich ob kurve bei aufruf geoeffnet war
	// BOOL bWasOpen = m_bStreamOpen;
	x = y = z = 0.0; // bei Fehler wenigstens 0 zurueckliefern 

	// Abbruch wenn Kurvenende erreicht oder Kurve keine Punkte hat oder
	// wenn Kurve nicht offen und auch nicht zu oeffnen
	if ( ( m_PointList.GetCount() == 0 ) || m_bEnd )  
		 /* || ((!m_bStreamOpen) && (!FastOpen()))*/ return FALSE;

	if ( !(m_PointList.GetAt(m_ActualIdx).Valid) ) return FALSE;
	x = m_PointList.GetAt(m_ActualIdx).GetAt(0);
	y = m_PointList.GetAt(m_ActualIdx).GetAt(1);
	z = m_PointList.GetAt(m_ActualIdx).GetAt(2); 

	
	m_bEnd = ( m_ActualIdx == (m_PointList.GetCount()-1) );
	if ( !m_bEnd ) m_ActualIdx++;

	// kurve wieder schliessen wenn sie am anfang geoeffnet war
	// if ( !bWasOpen ) FastClose();
	return TRUE;
};

// -----------------------------------------------------------------------------
// 06.05.2004 nicht mehr benoetigt, da m_bStreamOpen aukommentiert 
// 'oeffnet' Kurve, damit Auslesen oder Punkt anhaengen moeglich ist
/* BOOL TCurve::FastOpen( void )
{
	// test ob Kurve schon geoeffnet wenn ja schliessen
	// if ( m_bStreamOpen ) FastClose();

	// m_bStreamOpen = TRUE;
	return TRUE;
}; */

// -----------------------------------------------------------------------------
// 06.05.2004 nicht mehr benoetigt, da m_bStreamOpen aukommentiert 
// 'schliesst' Kurve und Zugriffe auf Kurve nicht moeglich
/* BOOL TCurve::FastClose( void )
{
	// m_bStreamOpen = FALSE;
	return TRUE;
}; */

// -----------------------------------------------------------------------------
// 06.05.2004 redundant zu PAdd, deshalb ueberall durch PAdd ersetzt
// anhaengen eines Punktes(tripel) x,y,z an die Kurve
// diese Funktion verzichtet auf unbed. aufruf von FastOpen und FastClose und
// ist damit fuer eine schnelle Abfolge von Punkthinzufuegungen geeignet
/* BOOL TCurve::FastPAdd( float &x, float &y, float &z )
{
	// wenn Punktanzahl der Kurve langsam die max. moegl. Punktanz. erreicht oder
	// wenn Kurve nicht geoeffnet dann Versuch FastOpen() zu rufen
	// Abbruch wenn dies nicht moeglich
	// if ( ((m_MaxPointIdx > (m_MaxMemIdx - 5)) 
	// if ((!m_bStreamOpen) && (!FastOpen()) ) return FALSE;
	
	m_bEnd = FALSE;

	TPoint pkt;
	pkt.Pt.SetAt(0, x);
	pkt.Pt.SetAt(1, y);
	pkt.Pt.SetAt(2, z);
	pkt.Valid= TRUE;
	if (!m_PointList.Append(pkt)) {
		MessageBox( GetFocus(), _NoMemory, "TCurve", MBSTOP );
		return FALSE;
	}
	m_ActualIdx++;
	
    // testen ob x,y,z wert des neuen Punktes ein m_Min bzw. m_Max sind wenn ja
	// diese als neues min. bzw. max. uebernehmen

	if ( m_MaxList.GetAt(0) < m_PointList.GetAt(m_ActualIdx).Pt.GetAt(0) )
		 m_MaxList.SetAt(0, m_PointList.GetAt(m_ActualIdx).Pt.GetAt(0) );
	if ( m_MinList.GetAt(0) > m_PointList.GetAt(m_ActualIdx).Pt.GetAt(0) )
	 	 m_MinList.SetAt(0, m_PointList.GetAt(m_ActualIdx).Pt.GetAt(0) );
	if ( m_MaxList.GetAt(1) < m_PointList.GetAt(m_ActualIdx).Pt.GetAt(1) )
		 m_MaxList.SetAt(1, m_PointList.GetAt(m_ActualIdx).Pt.GetAt(1) );
	if ( m_MinList.GetAt(1) > m_PointList.GetAt(m_ActualIdx).Pt.GetAt(1) )
		 m_MinList.SetAt(1, m_PointList.GetAt(m_ActualIdx).Pt.GetAt(1) );
	if ( m_MaxList.GetAt(2) < m_PointList.GetAt(m_ActualIdx).Pt.GetAt(2) )
		 m_MaxList.SetAt(2, m_PointList.GetAt(m_ActualIdx).Pt.GetAt(2) );
	if ( m_MinList.GetAt(2) > m_PointList.GetAt(m_ActualIdx).Pt.GetAt(2) )
		 m_MinList.SetAt(2, m_PointList.GetAt(m_ActualIdx).Pt.GetAt(2) );

	return TRUE;
}; */

// -----------------------------------------------------------------------------
// haengt ein Punkttripel an die Kurve an 
BOOL TCurve::PAdd( float &x, float &y, float &z )
{
	// kann Kurve geoeffnet werden, ggf. wird Kurve verlngert
	// if ( !FastOpen() ) return FALSE;
	
	m_bEnd = FALSE;

	TPoint pkt;
	pkt.SetAt(0, x);
	pkt.SetAt(1, y);
	pkt.SetAt(2, z);
	pkt.Valid= TRUE;
	if (!m_PointList.Append(pkt)) {
		MessageBox( GetFocus(), _NoMemory, "TCurve", MBSTOP );
		return FALSE;
	}
	m_ActualIdx++;
	
    // testen ob x,y,z wert des neuen Punktes ein m_Min bzw. m_Max sind wenn ja
	// diese als neues min. bzw. max. uebernehmen

	if ( m_MaxList.GetAt(0) < m_PointList.GetAt(m_ActualIdx).GetAt(0) )
		 m_MaxList.SetAt(0, m_PointList.GetAt(m_ActualIdx).GetAt(0) );
	if ( m_MinList.GetAt(0) > m_PointList.GetAt(m_ActualIdx).GetAt(0) )
	 	 m_MinList.SetAt(0, m_PointList.GetAt(m_ActualIdx).GetAt(0) );
	if ( m_MaxList.GetAt(1) < m_PointList.GetAt(m_ActualIdx).GetAt(1) )
		 m_MaxList.SetAt(1, m_PointList.GetAt(m_ActualIdx).GetAt(1) );
	if ( m_MinList.GetAt(1) > m_PointList.GetAt(m_ActualIdx).GetAt(1) )
		 m_MinList.SetAt(1, m_PointList.GetAt(m_ActualIdx).GetAt(1) );
	if ( m_MaxList.GetAt(2) < m_PointList.GetAt(m_ActualIdx).GetAt(2) )
		 m_MaxList.SetAt(2, m_PointList.GetAt(m_ActualIdx).GetAt(2) );
	if ( m_MinList.GetAt(2) > m_PointList.GetAt(m_ActualIdx).GetAt(2) )
		 m_MinList.SetAt(2, m_PointList.GetAt(m_ActualIdx).GetAt(2) );

	// FastClose(); // Kurve schliessen
	return TRUE;
};

// -----------------------------------------------------------------------------
// liefert den zu search naechsten X-Wert (den Indexwert) als Rueckgabewert
// und den zu X zugehoerigen Y-Wert sowie den Abstand zwischen X-wert und search
// 1.param (search): in Kurve soll nach einem X-punkt gesucht werden,
//                   der zu search den geringsten Abstand besitzt
// 2.Param (delta) : enthaelt nach Aufruf den Abstand von search zum X-wert
//                   
// 3.Param (value) : enthaelt den zum X-Wert gehoerigen Y-wert (Intensitaet)
// Rueckgabewert   : ist der Index X[Index] des naehsten X-wert zu search
//					 wenn search im inneren des x-bereichs der Kurve ist,
//                   sonst 0
// Voraussetzung ist, das die X-werte der Kurve aufsteigend sortiert sind
int TCurve::GetValueByValue( float search, float &delta, float &y )
{
	int Sign, Idx, NewIdx;

	delta = y = 0.0; // bei Fehler 0 
		
	if ( (m_PointList.GetCount() == 0) ||  // hat Kurve ueberhaupt Punkte
	     ((search > m_MaxList.GetAt(0)) || ( search < m_MinList.GetAt(0) )) /* || // ist search im X-Bereich
		 (!FastOpen()) */ ) return 0; // laesst Kurve sich oeffnen
		
	// binaere Suche in X-werten der kurve
	Idx = NewIdx = m_PointList.GetCount() / 2;	
	while ( NewIdx )
	{
		NewIdx /= 2;
		Sign = ( search > m_PointList.GetAt(Idx).GetAt(0) ) ? 0 : 1; 
		if ( Sign ) Idx -= NewIdx;
		else Idx += NewIdx;
	};
	// wenn search zwischen zwei x-werten dann naeheren nehmen
	if ( Sign )
		if ( ( search - m_PointList.GetAt(Idx).GetAt(0) ) > ( m_PointList.GetAt(Idx+1).GetAt(0) - search ) )
			Idx++;		
		else if ( ( search - m_PointList.GetAt(Idx-1).GetAt(0) ) > ( m_PointList.GetAt(Idx).GetAt(0) - search ) )
			Idx++; 
	// ausgabeparameter belegen + kurve schliessen
	y = m_PointList.GetAt(Idx).GetAt(1);
	delta = search - m_PointList.GetAt(Idx).GetAt(0); 
	// FastClose();
	return Idx;
};

// -----------------------------------------------------------------------------
// setzt den Index des aktuellen Tripels auf den angegebenen Wert
BOOL TCurve::SetPP( int aIdx )
{
	// test ob Kurve punkte hat
	if ( m_PointList.GetCount() == 0 ) return FALSE;
	
	m_ActualIdx = min( m_PointList.GetCount()-1, max( aIdx, 0 ) );
	m_bEnd = FALSE;
	return TRUE;
};

BOOL TCurve::SetPPLast() 
{
	return SetPP( GetPNumber() - 1 );
}

// -----------------------------------------------------------------------------
// berechnet eine durch den letzen Parameter bestimmte Fkt
// entweder maximale Intensitaet oder Schwerpunkt
BOOL TCurve::GetGravityCenter( float &x, float &y, float &z )
{
	int IdxMax;

	if ( (m_PointList.GetCount()-1) < 2 ) return FALSE; // wenn zu wenig Punkte, Abbruch

	 // tripel mit max y-Werte bestimmen
	IdxMax = 0;
	for ( int Idx = 0; Idx < m_PointList.GetCount(); Idx++ ) 
	{
		if ( m_PointList.GetAt(Idx).GetAt(1) > m_PointList.GetAt(IdxMax).GetAt(1) ) IdxMax = Idx;
	};
	// uebergeben des max. y-werte und der zugehoerigen x,z werte
	x = m_PointList.GetAt(IdxMax).GetAt(0);
	y = m_PointList.GetAt(IdxMax).GetAt(1);
	z = m_PointList.GetAt(IdxMax).GetAt(2); 

	// 13.04.2004 TOTER CODE 
	// Funktion hatte 4.Parameter (func) fuer ein switch
	// mit func=2  x,y,z tripel ermitteln, mit y maximal
    // mit func<>2 dann x,y,z Schwerpunkt der Kurve ermitteln
	// die Schwerpunktberechnung wird aber nie benutzt
	/*	
		default: // berechnet Schwerpunkt
			integral = weigth = 0.0;
			dx = m_lpPX[1].Pt[0] - m_lpPX[0].Pt[0];
			for ( Idx = 0; Idx <= m_MaxPointIdx; Idx++ )
			{
				weigth += m_lpPX[Idx].Pt[1] * dx;
				integral += m_lpPX[Idx].Pt[1] * (Idx * dx) * dx;
			}
			if (weigth) x = ( integral / weigth );

			for ( idx = 0; idx <= m_MaxPointIdx; idx++ )
			{
				if ( m_lpPX[ idx ].Pt[0] >= x )
				{
					y = m_lpPX[ idx ].Pt[1];
					z = m_lpPX[ idx ].Pt[2];
					break;
				}
			}
			break;
		*/
	return TRUE;
};

// -----------------------------------------------------------------------------
// markiert alle Tripel der Kurve als ungueltig, deren Y-Wert(Intensitaet)
// kleiner als Level * Intensitaetsbereich (max.Y- min.Y) ist
// 1.Param: nur Werte 0 <= aLevel <= 1 sind sinnvoll

BOOL TCurve::DeleteUnderGround( float aLevel )
{
	if ( m_PointList.GetCount() == 0 ) return FALSE; // hat die Kurve Punkte

	// Schwellwert den Y ueberschreiten muss, muss gueltig sein
	float PeakMin = ( m_MaxList.GetAt(1) - m_MinList.GetAt(1) ) * aLevel;

	// Kurve durchlaufen und alle Tripel als ungueltig kennzeichnet 
	// deren Y-Werte den Schwellwert nicht ueberschreiten
	for ( int Idx = 0; Idx < m_PointList.GetCount(); Idx++ ) 
	{
		if ( ( m_PointList.GetAt(Idx).GetAt(1) - m_MinList.GetAt(1) ) <= PeakMin ) 
		{
			m_PointList.GetAt(Idx).Valid = FALSE;
		}
	};
	return TRUE;
};
// -----------------------------------------------------------------------------
// ANFANG TOTER CODE 21.4.04
// -----------------------------------------------------------------------------
// speichert Kurve in angegebener Datei unter angegebener Option
// 1.Param Name der Datei 
// 2.Param type soll mehrere moeglichkeiten der art und weise wie und was von
//         der der Kurve abgelegt werden soll spezifizieren
//         (bisher nur type=100 implementiert=>legt nur die Y-werte der kurve
//         in der Datei ab)
/*
BOOL TCurve::Save( LPCSTR filename, int type, int )
{
	HANDLE hFile;
	float x, y, z;
	char buf[ MaxString ];
	LPDWORD lpWritten;

	lpWritten = 0;
	//  erstellen einer Datei mit angeg. namen
	// 10.05.2004 hFile = OpenFile( filename, &of, OF_CREATE );
	hFile = CreateFile( filename, GENERIC_WRITE|GENERIC_READ, 0, 0, CREATE_ALWAYS, 0, 0);
	if ( hFile == INVALID_HANDLE_VALUE ) return FALSE;

	// internen Index auf 1.Tripel der Kurve setzen + filepointer auf ende der
	// Datei + Kurve oeffnen 
	SetPP();

	// 10.05.2004 _llseek( hFile, 0, SEEK_END );
	SetFilePointer( hFile, 0, 0, FILE_END);
	FastOpen();
	// abhaengig von gewaehlter Option (hier nur eine implem.)
	if ( type==100 ) {
		// ein tripel aus Kurve holen(incl. index erhoehen) solange bis fehler
		// (Ende erreicht)
		while ( FastPGet( x, y, z ) )
		{
			// einen Y-wert pro zeile in datei schreiben
			sprintf( buf, "%.5f\n", y );
		
			// 10.05.2004 _lwrite( hFile, ( LPSTR ) buf, strlen( buf ) );
			WriteFile( hFile, (LPSTR)buf, strlen(buf), lpWritten, 0 );
		}
	}
	// datei schliessen + kurve schliessen
	CloseHandle( hFile );
    // _lclose( hFile );
	FastClose();
	return TRUE;
};
*/
// -----------------------------------------------------------------------------
// 07.04.2004 TOTER CODE da nicht benutzt
//! addiert den wert val zum Y-wert des Tripels x,y,z auf das der aktuelle
//! Index zeigt
//! 1.param val wird auf Y-wert aufaddiert
//! Rueckkehrcode false wenn Fehler sonst true
/*
BOOL TCurve::ValueAdd( float &val )
{
	// Addiert val jeweils zu [1] des aktuellen Punktes
	//! test ob kurve punkte hat wenn nicht abbruch
	if ( m_MaxPointIdx == -1 )
		return FALSE;
	//! test ob am Kurven ende wenn ja dann abbruch
	if ( m_bEnd )
		return FALSE;
	//! setzen ob im naechsten aufruf kurvenende erreicht
	m_bEnd = ( m_ActualIdx == m_MaxPointIdx );
	//! fixieren des speicherbereichs der y-werte (verschieben verhindern)
	//! und wenn dies nicht moeglich kurvenparam. auf init. werte zuruecksetzen +
	//! abbruch
	m_lpPX = (CPoint*)m_hMemX;
	if ( !m_lpPX )
	{
		m_ActualIdx = -1;
		m_MaxPointIdx = -1;
		return FALSE;
	}
	//! aufaddieren von val auf den akt.y-wert
	m_lpPX[ m_ActualIdx ].Pt.SetAt(1, m_lpPX[ m_ActualIdx ].Pt[1]+val ); // vormals m_lpPX[ m_ActualIdx ].Pt[1]+= val
	//! wenn noch nicht am ende der kurve angekommen dann akt.Index erhoehen
	if ( !m_bEnd )
		m_ActualIdx++;
	//! verschieben von y-werte speicherbereich wieder zulassen
	return TRUE;
};
*/

// -----------------------------------------------------------------------------
// 07.04.2004 TOTER CODE da nicht benutzt
//! legt alle x,y,z werte einer Kurve in einer Datei ab
//! 1.Param filename ist der name der Datei in der die Kurve abgelegt werden
//! soll
//! Rueckkehrcode false wenn Fehler(Datei laesst sich nicht anlegen) sonst true
/*
BOOL TCurve::Save( LPCSTR filename )
{
	OFSTRUCT of;
	int hFile;
	float x, y, z;
	char buf[ MaxString ];

	//!  erstellen einer Datei mit angeg. namen
	hFile = OpenFile( filename, &of, OF_CREATE );
	//! wenn fehler dann abbruch
	if ( hFile==HFILE_ERROR )
		return FALSE;
	//! internen Index auf 1.Tripel der Kurve setzen + filepointer auf ende der
	//! Datei + Kurve oeffnen
	SetPP();
	_llseek( hFile, 0, SEEK_END );
	FastOpen();
	//! ein tripel aus Kurve holen(incl. index erhoehen) solange bis fehler(ende
	//! erreicht)
	while ( FastPGet( x, y, z ) )
	{
		//! ein tripel (x,y,z) als einer Zeile in datei schreiben
		sprintf( buf, "%.3f %.3f %.3f\n", x, y, z );
		_lwrite( hFile, ( LPSTR ) buf, strlen( buf ) );
	}
	//! datei schliessen + kurve schliessen
	_lclose( hFile );
	FastClose();
	return TRUE;
};
*/

// -----------------------------------------------------------------------------
// 07.04.2004 TOTER CODE da nicht benutzt
/*
BOOL TCurve::DeleteFlanks()
{
	int MaxValIdx;
	int Idx = 0;

	// Flanken der Peaks beseitigen
	// Es wird nur y bearbeitet
	// Die gueltigen Punkte werden nach vorn geschoben + der Rest wird geloescht
	if ( -1 == m_MaxPointIdx )
		return FALSE;
	m_lpPX = (CPoint*)m_hMemX;
	if ( !m_lpPX )
		return FALSE;
	while ( Idx <= m_MaxPointIdx )
	{
		// Bis zum naechsten Peak
		while ( !m_lpPX[ Idx ].Valid )
			if ( Idx < m_MaxPointIdx )
				Idx++;
			else
				break;   // Ende erreicht
		// Groessten Wert suchen
		MaxValIdx = Idx;
		if ( Idx < m_MaxPointIdx )
			Idx++;
		else
			break;   // Ende erreicht
		while ( m_lpPX[ Idx ].Valid )
		{
			if ( m_lpPX[ Idx ].Pt[1] > m_lpPX[ MaxValIdx ].Pt[1] )
			{
				m_lpPX[ MaxValIdx ].Valid = FALSE;
				MaxValIdx = Idx;
			}
			else
				m_lpPX[ Idx ].Valid = FALSE;
			if ( Idx < m_MaxPointIdx )
				Idx++;
			else
				break;   // Ende erreicht
		}
		// Peak nach vorn kopieren

		//lpPnts[Idx] = lpPoints[MaxValIdx];

		m_lpPX[ Idx ].Valid = TRUE;
		m_lpPX[ MaxValIdx ].Valid = FALSE;
		if ( Idx < m_MaxPointIdx )
			Idx++;
	}
	// es wurden peaks gefunden
	//! nicht von diffr.gruppe auskommentiert
	//! Exit:
	return TRUE;
};
*/

// -----------------------------------------------------------------------------
//! nicht von diffr.gruppe auskommentiert
//! BOOL TCurve::GetReflection(int /*order*/,LPCPoint /*Peak*/)
/*
		  {
		  //! static BOOL   bSorted = FALSE;    // aendern
		  //! BOOL          bChange = FALSE; //wird nur im auskomm. Teil benutzt
		  BOOL          Ok = FALSE;
		  // int           Idx, MaxValIdx;
		  // CPoint        merker;
 
		  // Alle Peak-Maxima werden der Groesse nach sortiert
		  // Der groesste Peak hat die Ordnung 0 usw.
		  // Es sind mit weniger als 20 Peaks zu rechnen
		  if(-1 == m_MaxPointIdx)
					 return FALSE;
		  m_lpPX = (float*)GlobalLock(m_hMemX);
		  lpPY = (float*)GlobalLock(hMemY);
		  lpPZ = (float*)GlobalLock(hMemZ);
		  lpPV = (BOOL*)GlobalLock(hMemV);
		  if(!m_lpPX || !lpPY || !lpPZ || !lpPV)
					 goto Exit;
//Kommentarbeginn
		  if(!lpPV[0])
					 goto Exit;
		  if(0 < m_MaxPointIdx)
					 Idx = 1;
					 else
					 {
					 bSorted = TRUE;
					 //*Peak = lpPoints[0];
					 if(order == 0)
								Ok = TRUE;
								else
								Ok = FALSE;
					 goto Exit;
					 }
		  if(!bSorted)
					 {
					 // Sortieren  (Bubblesort)
					 Idx = 0;
					 // naechsten groessten Wert suchen
					 while(Idx < m_MaxPointIdx)
								{
								while(Idx <= m_MaxPointIdx)
										  {
										  if(lpPY[Idx] > lpPY[m_ActualIdx])
													 {
													 MaxValIdx = Idx;
													 bChange = TRUE;
													 }
										  Idx++;
										  }
								if(bChange)
										  {
										  // Mit ersten Punkt austauschen
										  merker = lpPoints[m_ActualIdx];
										  lpPoints[m_ActualIdx] = lpPoints[MaxValIdx];
										  lpPoints[MaxValIdx] = merker;
										  bChange = FALSE;
										  // point geht weiter als ThisPoint
										  if(m_ActualIdx < m_MaxPointIdx)
													 m_ActualIdx++;
										  Idx = m_ActualIdx+1;
										  }
								}
					 bSorted = TRUE;
					 }
		  m_ActualIdx = 0;
		  while(order--)
					 if(m_ActualIdx < m_MaxPointIdx)
								m_ActualIdx++;
								else
								// Peak der angegebenen Ordnung ist nicht vorhanden
								Ok = FALSE;
		  // Werte kopieren
		  (*Peak) = lpPoints[m_ActualIdx];
//Kommentarende
Exit:
		  GlobalUnlock(m_hMemX);
		  GlobalUnlock(hMemY);
		  GlobalUnlock(hMemZ);
		  GlobalUnlock(hMemV);
		  if(Ok)
					 return TRUE;
					 else
					 return FALSE;
		  };
*/

// -----------------------------------------------------------------------------
// 07.04.2004 TOTER CODE da nicht benutzt
//! liefert die Breite des 1.Bereiches, in dem alle y-werte des Tripels die
//! Haelfte des Intensitaetbereichs(Y(max)-Y(min)) uebersteigen, sowie aus
//! diesem Bereich x-wert,y-wert des Tripels fuer das Y-wert(Intensitaet)
//! maximal ist
//! 1. bereich bedeutet der am weitesten links liegende Bereich der obige
//! Eigenschaften erfuellt
//! 1.Param(ausgabe) hw enthaelt nach aufruf breite des Bereichs
//! 2.Param(ausgabe) pos enthaelt nach aufruf X-wert an der max. y-wert
//!                      (Intensitaet) in diesem Bereich
//! 3.Param(ausgabe) intens enthaelt nach aufruf max.y-wert(intensitaet) in
//!                         diesem Bereich
//! Rueckkehrcode false wenn fehler sonst true
//! anmerkung: dieser Bereich darf nicht bis ans ende der Kurve reichen
//! d.h. kurve darf nicht monoton wachsend sein (sonst return false)
/*
BOOL TCurve::GetPeakProperties( float &hw, float &pos, float &intens )
{
	int LeftPIdx, RightPIdx, MaxPIdx, Idx;
	BOOL retval = FALSE;

	//! test ob kurve punkte hat wenn nicht abbruch
	if ( -1 == m_MaxPointIdx )
		return FALSE;
	//! ermitteln der Bereiche, die obige bedingung erfuellen und bei fehler
	//! Abbruch
	if ( !DeleteUnderGround( 0.5 ) )
		return FALSE;
	// 000001111111100000
	// Bis zum naechsten Peak
	Idx = 0;
	//! speicherbereiche fixieren + test ob erfolgreich wenn nicht dann abbruch
	m_lpPX = (CPoint*)m_hMemX;
	if ( !m_lpPX )
		goto Exit;
	//! ueberspringen eines event. vorhandenen ungueltigen Bereichs gleich am
	//! anfang der kurve + abbruch falls die gesamte kurve ungueltig ist
	while ( !m_lpPX[ Idx ].Valid )
		if ( Idx < m_MaxPointIdx )
			Idx++;
		else
			goto Exit;  // Ende erreicht
	// linke Flanke
	LeftPIdx = MaxPIdx = Idx;
	//! ermitteln des max. y-wertes in diesem Bereich + abbruch falls auf diesen
	//! gueltigen bereich nicht ein ungueltiger folgt
	while ( m_lpPX[ Idx ].Valid )
	{
		if ( m_lpPX[ Idx ].Pt[1] > m_lpPX[ MaxPIdx ].Pt[1] )
			MaxPIdx = Idx;
		if ( Idx < m_MaxPointIdx )
			Idx++;
		else
			goto Exit;  // Ende erreicht
	}
	// rechte Flanke

	RightPIdx = Idx - 1;
	//! berechnen und uebergeben der ausgabeparameter
	hw = fabs( m_lpPX[ RightPIdx ].Pt[0] - m_lpPX[ LeftPIdx ].Pt[0] );
	pos = m_lpPX[ MaxPIdx ].Pt[0];
	intens = m_lpPX[ MaxPIdx ].Pt[1];
	//! funktion erfolgreich
	retval = TRUE;
Exit:
	//! verschieben der speicherbereiche wieder zulassen
	return retval;
};
*/

//##############################################################################
// TDataBase
//##############################################################################

// -----------------------------------------------------------------------------
// Konstruktor der Datenbasis
TDataBase::TDataBase( void ) : TBaseList(true, false, true, false, cDefaultSize)
{	
	m_ActualIdx = 0;
	Append(new TCurve);
};

// -----------------------------------------------------------------------------
// Destruktor
TDataBase::~TDataBase( void )
{
	// loeschen aller Kurven der Datenbasis
	Clear(); // nicht TBaseList::~TIndList() verwenden, sonst wird TBaseList::FreeData() statt TDataBase::FreeData() verwendet
};

// -----------------------------------------------------------------------------
// alle Kurven der Datenbasis loeschen und alle Indizes auf ini.werte
BOOL TDataBase::New( void )
{
	SetInfo( _FreeingDataBase );
	// Kurven beginnend von der Letzten bis zur Ersten loeschen
	if ( GetCount()>1 ) CutList(1);

	// 1 neue.Kurve erzeugen
	m_ActualIdx = 0;
	if ( GetCount()==0 && !Append(new TCurve) ) return FALSE;

	( (TCurve*)GetAt(0) )->New(); //17.11.2002
	// Scanreportkurve auf ini.werte zuruecksetzen
	m_ScanReport.New(); //17.11.2002
	SetInfo( " " );
	return TRUE;
};

// -----------------------------------------------------------------------------
// fuegt eine Kurve der angegebenen Groesse(1.Param.) zur Datenbasis hinzu
// true, wenn erfolgreich
BOOL TDataBase::AddCurve( void )
{
	// wenn erfolgreich Abbruch sonst Fehler setzen
	if ( !Append(new TCurve) ) return FALSE;	

	// m_eFault = NoMemory;  // 24.03.2004 
	return TRUE;
};

// -----------------------------------------------------------------------------
// entfernt die letze Kurve (dem Index nach) in der Datenbasis
// true, wenn erfolgreich
BOOL TDataBase::DelCurve( void )
{
	// immer eine Kurve brig behalten
	if ( GetCount() <= 1 ) return FALSE;

	if (m_ActualIdx == GetCount()-1) m_ActualIdx--;
	Remove( GetCount()-1 );
	return TRUE;
}

// -----------------------------------------------------------------------------
// setzt Kurvenindex auf angegebenen Wert (1.Param.) wenn mglich, dann true 
// ansonsten auf den entsprechenden Grenzwert
BOOL TDataBase::SetCurveIdx( int aIndex )
{
	m_ActualIdx = max( 0, min( GetCount()-1, aIndex ) );
	if ( m_ActualIdx != aIndex ) return FALSE;
	return TRUE;
};

// -----------------------------------------------------------------------------
// liefert Kurve mit angegebenen Index (1.Param.) zurueck
// wenn moeglich ansonsten naechstgelegene gueltige Kurve
TCurve *TDataBase::GetCurve( int aIndex )
{
	aIndex = max( 0, min( GetCount()-1, aIndex ) );
	return (TCurve*)GetAt( aIndex );
};

TCurve *TDataBase::GetLastCurve()
{
	return GetCurve( GetCount()-1 );
};

void TDataBase::FreeData(void *aData) const {
	TCurve *cast= (TCurve*)aData;
	_FREEOBJ(cast); // sonst wird TCurve::~TCurve() nicht gerufen	
}

// __LastLine__




