/*
$Header: /vol/baal-vol3/projekt98/quellen/XCTL_32/DETECUSE/DETECGUI.CPP,v 1.40 2004/07/30 04:21:55 kullmann Exp $

Projekt   : XCTL
Subsystem : Detektoren
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
          Institut fuer Informatik,
          Humboldt-Universitaet Berlin
Inhalt    : Code fuer die Oberflaechenklassen des Detektoren-Subsystems
*/

#include "internls\evrythng.h" // GermanVersion
#include "winresrc\rc_def.h"   // Ressourcen-IDs
#pragma hdrstop

#include "detecuse\detecgui.h"
#include "motrstrg\motrstrg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

extern HINSTANCE hModuleInstance;

//##############################################################################
// TCommonDevParamDlg
//##############################################################################

TCommonDevParamDlg::TCommonDevParamDlg( TDetector* detectorParam ) : TModalDlg( "COMMONDEVPARAM", hModuleInstance )
{
	if (detectorParam == 0)
		Detector= TDetectorManager::DetectorManager().GetDetector();
	else
		Detector= detectorParam;
}
//-----------------------------------------------------------------------------

BOOL TCommonDevParamDlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM lParam )
{
	// Initialisieren des Detektorauswahl-Listenfeldes
	hChooseDetector= GetDlgItem( hwnd, id_ChooseDetector );
	TDetectorManager::DetectorManager().FillDetectorComboBox(hChooseDetector);

	SetFocus( GetDlgItem( hwnd, id_Level ) );
	FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
	return TRUE;
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl, UINT codeNotify )
{
	switch ( id )
	{
		case cm_ParamSet:
			OnParamSet();
			break;

		case id_ShowCounter:
			OnShowCounter(hwnd);
			break;

		case cm_ActivateChanges:
			OnActivateChanges();
			break;

		case id_SoundSwitch:
			OnSoundSwitch();
			break;

		case cm_SpecificParameters:
			OnSpecificParameters();
			break;

		case id_ChooseDetector:
			OnChooseDetector(hwnd, codeNotify);
			break;

		default:
			TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
	}
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::OnParamSet()
{
	char buf[ MaxString ];

	ComboBox_SelectString( hChooseDetector, 0, Detector->GetCharacteristic() );

	CheckDlgButton( GetHandle( ), id_ShowCounter, ( Detector->GetCounterWnd() != 0 ) );

	// Nur 0-dimensionale Detektoren
	if (Detector->GetDimension() == 0)
	{
		TZeroDimDetector* ZD= dynamic_cast<TZeroDimDetector*>(Detector);
		EnableWindow( GetDlgItem( GetHandle( ), id_SoundSwitch ), TRUE );
		CheckDlgButton( GetHandle( ), id_SoundSwitch, ZD->GetSound() );
	}
	else
		EnableWindow( GetDlgItem( GetHandle( ), id_SoundSwitch ), FALSE );

	sprintf( buf, "%.2f", Detector->GetExposureSettings().GetExposureTime() );

	SetDlgItemText( GetHandle( ), id_ExposureTime, ( LPSTR ) buf );

	sprintf( buf, "%lu", Detector->GetExposureSettings().GetExposureCounts() );

	SetDlgItemText( GetHandle( ), id_ExposureCounts, ( LPSTR ) buf );

	CheckDlgButton( GetHandle( ), id_ShowCounter, ( Detector->GetCounterWnd() != 0 ) );

	EnableWindow( GetDlgItem( GetHandle( ), cm_SpecificParameters ),
				  Detector->HasSpecificParametersDlg() );
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::OnShowCounter(HWND hwnd)
{
	if ( ! CanClose( ) )
	{
		PostMessage( GetHandle( ), WM_COMMAND, cm_ParamSet, 0 );
		return;
	}

	if ( Detector->GetCounterWnd() == 0 )
		FORWARD_WM_COMMAND( GetFrameHandle( ), cm_CallExecuteCounter, 0, Detector->GetId( ), SendMessage );
	else
		FORWARD_WM_MDIDESTROY( GetClientHandle( ), Detector->GetCounterWnd(), SendMessage );

	FORWARD_WM_COMMAND( hwnd, cm_ParamSet, 0, 0, SendMessage );
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::OnActivateChanges()
{
	if (Detector->GetDimension() == 0)
	{
		BOOL bIsSound= IsDlgButtonChecked( GetHandle(), id_SoundSwitch );
		(dynamic_cast<TZeroDimDetector*>(Detector))->SetSound( bIsSound );
	}

	Detector->SetParameters();
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::OnSoundSwitch()
{
	OnActivateChanges();
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::OnSpecificParameters()
{
	Detector->RunSpecificParametersDlg( GetFrameHandle() );
	FORWARD_WM_COMMAND( GetHandle( ), cm_ActivateChanges, 0, 0, PostMessage );
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::OnChooseDetector(HWND hwnd, UINT codeNotify)
{
	if ( codeNotify != CBN_SELCHANGE )
		return;

	int idx= ComboBox_GetCurSel( hChooseDetector );

	if ( idx == CB_ERR )
		return;

	TDetector* newDetector= TDetectorManager::DetectorManager().GetDetector(idx);

	if ( Detector != newDetector )
	{
		// Neuer Detektor ausgewaehlt

		// Wenn CounterWindow offen, dann schliessen
		BOOL bCounterWindowWasOpen= ( Detector->GetCounterWnd() != 0 );
		if ( bCounterWindowWasOpen == TRUE )
			OnShowCounter(hwnd);

		OnActivateChanges();
		Detector= newDetector;
		TDetectorManager::DetectorManager().SetDetector(idx);

		OnParamSet();

		if ( bCounterWindowWasOpen == TRUE )
			OnShowCounter(hwnd);
	}
}
//-----------------------------------------------------------------------------

BOOL TCommonDevParamDlg::CanClose( void )
{
	char buf[ MaxString ];

	if (Detector->GetDimension() == 0)
	{
		BOOL bIsSound= IsDlgButtonChecked( GetHandle(), id_SoundSwitch );
		(dynamic_cast<TZeroDimDetector*>(Detector))->SetSound( bIsSound );
	}

	GetDlgItemText( GetHandle( ), id_ExposureCounts, ( LPSTR ) buf, 10 );
	DWORD dwNewExposureCounts= atol(buf);

	GetDlgItemText( GetHandle( ), id_ExposureTime, ( LPSTR ) buf, 10 );
	float fNewExposureTime= atof(buf);

	Detector->SetExposureSettings( TExposureSettings( fNewExposureTime, dwNewExposureCounts) );

	Detector->PushSettings( );
	return TRUE;
}
//-----------------------------------------------------------------------------

void TCommonDevParamDlg::LeaveDialog( void )
{
	Detector->SetParameters( );
}

//##############################################################################
// TPsdParametersDlg
//##############################################################################

TPsdParametersDlg::TPsdParametersDlg( TModelessDlg **aRef ) : TModelessDlg("PsdParameter", aRef, hModuleInstance), m_Parent(0)
{
} 
//-----------------------------------------------------------------------------

BOOL TPsdParametersDlg::Initialize ( HINSTANCE aInstance, HWND aParentWnd ) {
	m_Parent= aParentWnd;
	return TModelessDlg::Initialize( aInstance, aParentWnd );
}

BOOL TPsdParametersDlg::Dlg_OnInit(HWND hwnd, HWND hwndCtl, LPARAM lParam)
{
	if (! TDetectorManager::DetectorManager().DimAvailable(1))
	{
		MessageBox("Kein Psd-Detektor gefunden!", "Fehler", MBFAILURE);
		FORWARD_WM_COMMAND(hwnd, IDCANCEL, hwndCtl, 0, PostMessage);
		return FALSE;
	}

	Detector= dynamic_cast<TOneDimDetector*>(
				   TDetectorManager::DetectorManager().GetDetector(0, 1));

	// Initialisieren der Parameterfenster
	CtrlSetEnabled( id_StepWidth, FALSE );
	CtrlSetText( id_StepWidthUnit, Detector->GetUnitName() );	
	CtrlSetEnabled( id_AngleRange, FALSE );
	CtrlSetText( id_AngleRangeUnit, Detector->GetUnitName() );	

	PSD2Screen( TRUE, TRUE, TRUE ); // alle Eingabefelder
	SetFocus(hwnd);
	return TRUE;
}
//-----------------------------------------------------------------------------

void TPsdParametersDlg::Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch (id)
	{
		case cm_InitializePsd:
			OnInitializePsd(hwnd);
			break;

		case cm_SetEnergyRange:
			Screen2PSD( TRUE, FALSE, FALSE );
			break;

		case cm_SetPositionRange:
			Screen2PSD( FALSE, TRUE, FALSE );
			break;	

		case IDOK: // 'OK' und [ENTER]
			Screen2PSD( TRUE, TRUE, TRUE ); // alle Eingabefelder
			TModelessDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify); // Dialog verlassen
			break;

		default:
			TModelessDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify);
	}
}
//-----------------------------------------------------------------------------

void TPsdParametersDlg::PSD2Screen(BOOL aEnergyRange, BOOL aPositionRange, BOOL aRest) {
	if ( aEnergyRange || aPositionRange )  
	{
		CtrlSetLong( id_PositionMin, Detector->GetFirstChannel() );
		CtrlSetLong( id_PositionMax, Detector->GetLastChannel() );

		CtrlSetDouble( id_StepWidth, CalcValueInUnit(Detector->GetUnit(), Detector->GetAngleStep()), Detector->GetAngleStepDigits(), _DECIMAL);
		CtrlSetDouble( id_AngleRange, CalcValueInUnit(Detector->GetUnit(), Detector->GetAngleRange()), NKAUTO, _DECIMAL);
	}
	
	if ( aEnergyRange ) 
	{
		UINT her, ler;
		Detector->GetEnergyRange(ler, her);
		CtrlSetLong( id_UpperLevel, her );
		CtrlSetLong( id_LowerLevel, ler );

		CtrlSetFormat( id_EnergScale1, "max %d", Detector->GetScaledMaxChannels(0) );
		CtrlSetFormat( id_EnergScale2, "max %d", Detector->GetScaledMaxChannels(1) );
		CtrlSetFormat( id_EnergScale3, "max %d", Detector->GetScaledMaxChannels(2) );
		CtrlSetFormat( id_EnergScale4, "max %d", Detector->GetScaledMaxChannels(3) );

		CtrlSetChecked( id_EnergScale1, Detector->GetEnergyScale()!=1 && Detector->GetEnergyScale()!=2 && Detector->GetEnergyScale()!=3 );
		CtrlSetChecked( id_EnergScale2, Detector->GetEnergyScale()==1 );
		CtrlSetChecked( id_EnergScale3, Detector->GetEnergyScale()==2 );
		CtrlSetChecked( id_EnergScale4, Detector->GetEnergyScale()==3 );
	}
	if ( aPositionRange ) 
	{
		CtrlSetFormat( id_PosScale1, "max %d", Detector->GetScaledMaxChannels(0) );
		CtrlSetFormat( id_PosScale2, "max %d", Detector->GetScaledMaxChannels(1) );
		CtrlSetFormat( id_PosScale3, "max %d", Detector->GetScaledMaxChannels(2) );
		CtrlSetFormat( id_PosScale4, "max %d", Detector->GetScaledMaxChannels(3) );

		CtrlSetChecked( id_PosScale1, Detector->GetPositionScale()!=1 && Detector->GetPositionScale()!=2 && Detector->GetPositionScale()!=3 );
		CtrlSetChecked( id_PosScale2, Detector->GetPositionScale()==1 );
		CtrlSetChecked( id_PosScale3, Detector->GetPositionScale()==2 );
		CtrlSetChecked( id_PosScale4, Detector->GetPositionScale()==3 );
	}
	if ( aRest ) {
		CtrlSetLong( id_AddedChannels, Detector->GetAddedChannels() );
		CtrlSetChecked( id_EnergyControl, Detector->GetHVControl() );
		CtrlSetChecked( id_LeftConnectedZero, Detector->GetReadLeftFirst() );
	}
}
//-----------------------------------------------------------------------------

bool TPsdParametersDlg::Screen2PSD(BOOL aEnergyRange, BOOL aPositionRange, BOOL aRest) {
	BOOL aValid= FALSE;
	if ( aEnergyRange ) {
		UINT scale= 0;
		if ( CtrlIsChecked(id_EnergScale2) ) scale= 1;
		if ( CtrlIsChecked(id_EnergScale3) ) scale= 2;
		if ( CtrlIsChecked(id_EnergScale4) ) scale= 3;
		Detector->SetEnergyScale(scale);

		int ler= CtrlGetLong(id_LowerLevel, aValid);
		if ( !aValid ) return false; // Eingabe ungltig
		int her= CtrlGetLong(id_UpperLevel, aValid);
		if ( !aValid ) return false; // Eingabe ungltig
		Detector->SetEnergyRange(ler, her);
	}

	if ( aPositionRange ) {
		UINT scale= 0;
		if ( CtrlIsChecked(id_PosScale2) ) scale= 1;
		if ( CtrlIsChecked(id_PosScale3) ) scale= 2;
		if ( CtrlIsChecked(id_PosScale4) ) scale= 3;
		Detector->SetPositionScale(scale);

		int lpr= CtrlGetLong(id_PositionMin, aValid);
		if ( !aValid ) return false; // Eingabe ungltig
		int hpr= CtrlGetLong(id_PositionMax, aValid);
		if ( !aValid ) return false; // Eingabe ungltig
		Detector->SetChannelRange(lpr, hpr);
	}
	if ( aEnergyRange || aPositionRange )
		if ( m_Parent ) SendMessage(m_Parent, cm_UpdateRanges, 0, 0);

	if ( aRest ) {
		int chan= CtrlGetLong(id_AddedChannels, aValid);
		if ( !aValid ) return false; // Eingabe ungltig
		Detector->SetAddedChannels( chan );
		Detector->SetHVControl( CtrlIsChecked(id_EnergyControl) );
		SetStatus(" "); // Letzte HV-Regelung - Ausgabe lschen.
		Detector->SetReadLeftFirst( CtrlIsChecked(id_LeftConnectedZero) );		
	}

	// wirklich gesetzte Werte anzeigen (nur die genderten Werte aktualisieren)
	PSD2Screen( aEnergyRange, aPositionRange, aRest );
	return true;
}
//-----------------------------------------------------------------------------

void TPsdParametersDlg::OnInitializePsd(HWND hwnd)
{
	if ( Screen2PSD(TRUE, TRUE, TRUE) ) 
		if ( Detector->PsdInit() != R_OK )
			SetInfo("Init Psd failed.");

	PSD2Screen( TRUE, TRUE, TRUE );
}

//##############################################################################
// TScsParametersDlg
//##############################################################################

TScsParametersDlg::TScsParametersDlg( TDetector* detectorParam ) : TModalDlg( "SCSPARAMETER", hModuleInstance )
{
	Radicon= static_cast<TRadicon*>(detectorParam);
}
//-----------------------------------------------------------------------------

BOOL TScsParametersDlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM lParam )
{
	FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
	SetFocus( GetDlgItem( GetHandle( ), id_AngleRange ) );
	return TRUE;
}
//-----------------------------------------------------------------------------

void TScsParametersDlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl, UINT codeNotify )
{
	if ( id == cm_ParamSet )
		OnParamSet();
	else
		TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
}
//-----------------------------------------------------------------------------

void TScsParametersDlg::OnParamSet()
{
	char buf[ MaxString ];

	sprintf( buf, "%d", Radicon->GetDacThresh().GetMin() );
	SetDlgItemText( GetHandle( ), id_LowerLevel, ( LPSTR ) buf );

	sprintf( buf, "%d", Radicon->GetDacThresh().GetMax() );
	SetDlgItemText( GetHandle( ), id_UpperLevel, ( LPSTR ) buf );

	sprintf( buf, "%d", Radicon->GetHighVoltage() );
	SetDlgItemText( GetHandle( ), id_HighVoltage, ( LPSTR ) buf );
}
//-----------------------------------------------------------------------------

BOOL TScsParametersDlg::CanClose( void )
{
	char buf[ MaxString ];

	GetDlgItemText( GetHandle( ), id_UpperLevel, ( LPSTR ) buf, 10 );
	WORD wUpperThresh= atoi(buf);

	GetDlgItemText( GetHandle( ), id_LowerLevel, ( LPSTR ) buf, 10 );
	WORD wLowerThresh= atoi(buf);

	Radicon->SetDacThresh( wLowerThresh, wUpperThresh );

	GetDlgItemText( GetHandle( ), id_HighVoltage, ( LPSTR ) buf, 10 );
	Radicon->SetHighVoltage(atoi(buf));

	return TRUE;
}

//##############################################################################
// TCalibratePsdDlg
//##############################################################################

TCalibratePsdDlg::TCalibratePsdDlg( TCurve &curve ) : TModalDlg( "CALIBRATEPSD", hModuleInstance ), m_lastfocused(NULL), m_OldControlWnd(0)
{
	m_lnkTimer= new TInterfaceTimer( this, 80 ); //Observer, d.h. Timer benachrichtigt uns

	Curve= &curve; // MainCurve vom Daten-Fenster
	//! gibt an ob motor ueber groessere distanz faehrt
	bLongMove= FALSE;

	LeftPeak.Valid= FALSE;

	RightPeak.Valid= FALSE;
	//! gibt an ob rechter bzw. linker Peak schon erreicht ist
	bGetRightPeak= FALSE;
	bGetLeftPeak= FALSE;
	// 21.04.2004 bCalibrationOk= FALSE;

	//! Thetamotor setzen
	nMotor= mlGetIdByName( Theta );
	mlSetAxis( nMotor );

	//! alte Schrittweite merken + neue auf 1800 winkelsekunden setzen
	dOldMotorWidth= mGetValue( Width );
	mSetValue( Width, CalcValueInUnit(mGetUnit(), 1800.0) ); //! FIX Fehler 48
	mActivateDrive();
}
//-----------------------------------------------------------------------------

//! setzt Defaultwerte der Dialogbox bei Aufruf der Box
//! Rueckkehrcode immer true
BOOL TCalibratePsdDlg::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM lParam )
{
	if ( !mlIsAxisValid( Theta ) || !TDetectorManager::DetectorManager().DimAvailable(1))
	{
		MessageBox("Kein Psd-Detektor gefunden bzw. Theta-Achse nicht verfgbar!", "Fehler", MBFAILURE);
		FORWARD_WM_COMMAND(GetHandle(), IDCANCEL, hwndCtl, 0, PostMessage);
		return FALSE;
	}

	hBar= GetDlgItem( GetHandle(), id_Bar );

	Psd= dynamic_cast<TOneDimDetector*>( TDetectorManager::DetectorManager().GetDetector(0, 1) );
	//! Psd einstellen
	Psd->MeasureStop();
	// 12.06.2004 Psd->SetAddedChannels( 2 );
	m_OldControlWnd= Psd->GetControlWnd();
	Psd->SetControlWnd( GetHandle() );
	Psd->SetExposureSettings( TExposureSettings(2.0, 10000) );
	Psd->MeasureStart();

	dStepWidth= Psd->GetAngleStep();
	OnParamSet(); // bentigt dStepWidth
	BOOL bValid;
	double dDistance= DoRefreshPosition(bValid);
	CtrlSetDouble(id_NewAngle, dDistance, mGetDigits(), _DECIMAL);
	CtrlSetText( id_Unit, mGetUnitName() );	

	SetFocus( hBar );
	Delay( 5000 );
	return TRUE;
}
//-----------------------------------------------------------------------------

//! behandelt Timersignale zur aktualisierung der Winkelwerte in Dialogbox
void TCalibratePsdDlg::OnTimer ( TBasicTimer *const )
{
	BOOL bValid;
	//! Wait-mauszeiger setzen
	SetDefaultCursor( IDC_WAIT );

	//! test ob motor an Ziel angekommen
	if ( mIsMoveFinish() ) // nicht mehr in Bewegung
	{
		//! Motorpos.bestimmung + test ob fehler
		double dDistance= DoRefreshPosition(bValid);
		if ( !bValid ) return;

		CtrlSetDouble(id_NewAngle, dDistance, mGetDigits(), _DECIMAL);
		SetDefaultCursor( IDC_ARROW );
		m_lnkTimer->StopTimer();

	} else 
		if ( bLongMove ) // Motor noch in Bewegung
		//! bei bewegung ueber groessere distanz den akt. Winkel in Dialogbox + //! scrollbar aktualisieren
			DoRefreshPosition(bValid);
}
//-----------------------------------------------------------------------------

//! behandelt Kommandos in der Dialogbox
void TCalibratePsdDlg::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl,
									  UINT codeNotify )
{
	switch ( id )
	{
		case cm_CounterSet://! wenn Psdmessung fertig
			OnCounterSet();
			break;

		case id_NewAngle:
			m_lastfocused= hwndCtl;
			break;
			
		case cm_RightPositionReached://! rechte Position erreicht-Button gedrueckt
			m_lastfocused= hwndCtl;
			OnRightPositionReached();
			break;
			
		case cm_LeftPositionReached://! linke Position erreicht-Button gedrueckt
			m_lastfocused= hwndCtl;
			OnLeftPositionReached();
			break;
	
		case ID_CANCEL: // //! behandelt Abbrechen-Button 
			m_lastfocused= hwndCtl;
			OnCancel();
			break; 

		case IDOK: // Enter
			OnIDOK();
			break;
	
		default: //! beinhandelt auch Beenden-Button und Schlieen-"x" (IDCANCEL) 
			TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
	}
}
//-----------------------------------------------------------------------------

//! behandelt alle Scrollbarereignisse
void TCalibratePsdDlg::Dlg_OnHScrollBar( HWND hwnd, HWND hwndCtl, UINT code, int pos )
{
	m_lastfocused= hBar;
	switch ( code )
	{	
		case SB_LINEDOWN: //! Scrollbar um eine einheit nach rechts
		case SB_LINEUP: //! Scrollbar um eine einheit nach links
			//! wenn Motor in Bewegung: abbrechen
			if ( !mIsMoveFinish() ) return;
			bLongMove= FALSE;
			if ( code==SB_LINEUP ) mMoveByDistance( -fabs(mGetValue(Width)) );
			else mMoveByDistance( fabs(mGetValue(Width)) ); // SB_LINEDOWN
			m_lnkTimer->StartTimer();
			break;		
	}
}
//-----------------------------------------------------------------------------

//! wird bei Verlassen der Dialogbox aufgerufen, Rueckkehrcode immer true
void TCalibratePsdDlg::LeaveDialog( void )
{
	char buf[ MaxString ];
	char format[ MaxString ];
	char ident[ MaxString ];
	//! FIX Fehler 48
	Psd->SetAngleStep( dStepWidth );

	//! speichert Kanal pro Winkel im Ini-File des Psd
	sprintf( ident, "Device%d", Psd->GetId() );
	sprintf(format, "%s%d%s", "%.", Psd->GetAngleStepDigits(), "f"); // Anzahl der Nachkommastellen beachten
	sprintf( buf, format, Psd->GetAngleStep() );
	if ( Psd->GetAngleStep() != 0.0 ) WritePrivateProfileString( ident, "AngleStep", buf, GetHWFile() );

	//! loescht alle TimerIdInformation + stoppen der Messung
	//! FIX Fehler 49
	mStopDrive( FALSE );
	mSetValue( Width, dOldMotorWidth );
	m_lnkTimer->StopTimer();

	Psd->MeasureStop();
	Psd->SetControlWnd( m_OldControlWnd );
}
//-----------------------------------------------------------------------------

void TCalibratePsdDlg::OnIDOK()
{
	// [ENTER] nicht im 'Neuer Winkel': Abbruch
	if ( m_lastfocused!=Ctrl(id_NewAngle) ) return;

	// Der Nutzer hat mit neue Position eingegeben und [ENTER] gedrckt
	BOOL bValid= FALSE;
	double dDistance= CtrlGetDouble(id_NewAngle, mGetDigits(), _DECIMAL, bValid);
	CtrlSetDouble(id_NewAngle, dDistance, mGetDigits(), _DECIMAL);
	if ( bValid ) {
		mStopDrive(TRUE);
		bLongMove= TRUE;
		mMoveToDistance( dDistance );
		m_lnkTimer->StartTimer();
	}
	SetFocus( hBar );
}
//-----------------------------------------------------------------------------

void TCalibratePsdDlg::OnCancel() {
	mStopDrive( FALSE );
}
//-----------------------------------------------------------------------------

double TCalibratePsdDlg::DoRefreshPosition(BOOL &aValid) {
	aValid= FALSE;
	double dDistance;
	if ( !mGetDistance( dDistance ) ) return 0;

	//! Winkel 2Thetawerte in Dialogbox schreiben + scrollbar aktualisieren
	CtrlSetDouble(id_Angle, dDistance, mGetDigits(), _DECIMAL);

	SCROLLINFO scrInfo;
	scrInfo.cbSize= sizeof(SCROLLINFO);
	scrInfo.fMask= SIF_POS;
	scrInfo.nPos= GetBarPos();
	SetScrollInfo(hBar,SB_CTL,&scrInfo,TRUE);
	aValid= TRUE;
	return dDistance;
}
//-----------------------------------------------------------------------------

void TCalibratePsdDlg::OnParamSet()
{
	// akt. Winkel je Kanal setzen und Scrollbar-Bereich aktualisieren	
	CtrlSetDouble(id_StepWidth, dStepWidth, Psd->GetAngleStepDigits(), _DECIMAL); //! FIX Fehler 48
	
	// Initialisieren des Motor-Scroll-Bar's
	SCROLLINFO scrInfo;
	scrInfo.cbSize= sizeof(SCROLLINFO);
	scrInfo.fMask= SIF_RANGE;
	scrInfo.nMin= GetBarEgde(TRUE);
	scrInfo.nMax= GetBarEgde(FALSE);
	SetScrollInfo(hBar, SB_CTL, &scrInfo, TRUE);
}
//-----------------------------------------------------------------------------

void TCalibratePsdDlg::OnCounterSet()
{
	float val_f, val_f1, channel, dd;

	Psd->UpdateViews(TRUE);
	if ( m_OldControlWnd ) PostMessage( m_OldControlWnd, WM_COMMAND, cm_CounterSet, 0 ); // AreaScan-Fenster auch neu zeichnen

	//! test ob rechte Peakpos. schon eingestellt wurde
	if ( bGetRightPeak )
	{
		//! ermitteln + speichern des Kanals der max. Intensitaet
		Curve->GetGravityCenter( dd, dd, channel );
		RightPeak.Pt[0]= channel;
		RightPeak.Valid= TRUE;
		bGetRightPeak= FALSE;
		//! wenn linke Peakpos. schon ermittelt

		if ( LeftPeak.Valid )
		{
			//! winkeldifferenz zwischen linken und rechten peak ermitteln
			val_f= ( LeftPeak.Pt[ 1 ] - RightPeak.Pt[ 1 ] );
			//! wenn linker peak == rechter peak dann rechte Peakpos. verwerfen
			if ( val_f != 0.0 )
			{
				//! wenn gleicher kanal fuer linken und rechten peak dann
				//! Fehlermeldung
				val_f1= ( RightPeak.Pt[ 0 ] - LeftPeak.Pt[ 0 ] );
				if ( val_f1 != 0 )
				{
					//! Winkel je kanal ausrechnen und an Psd uebergeben
					dStepWidth= CalcValueFromUnit(mGetUnit(), val_f / val_f1); //! FIX Fehler 48
				} else {
					MessageBox( "Vorgang bitte wiederholen !", "Fehler", MBINFO );
					//! FIX Fehler 58
					//! reset aller wichtigen Werte
					LeftPeak.Valid= FALSE;
					RightPeak.Valid= FALSE;
					bGetRightPeak= FALSE;
					bGetLeftPeak= FALSE;
					CtrlSetEnabled( cm_LeftPositionReached, TRUE);
					CtrlSetEnabled( cm_RightPositionReached, TRUE);
					return;
				}
			} else {
				RightPeak.Valid= FALSE;
				bGetRightPeak= TRUE;
			}
		}

		//! sperren bzw. freigeben des rechte-Position-erreicht-Button
		CtrlSetEnabled(cm_RightPositionReached, bGetRightPeak );
	}

	//! test ob linke Peakpos. schon eingestellt wurde
	if ( bGetLeftPeak )
	{
		//! ermitteln + speichern des Kanals der max. Intensitaet
		Curve->GetGravityCenter( dd, dd, channel );
		LeftPeak.Pt[0]= channel;
		LeftPeak.Valid= TRUE;
		bGetLeftPeak= FALSE;
		//! wenn rechte Peakpos. schon ermittelt

		if ( RightPeak.Valid )
		{
			//! winkeldifferenz zwischen linken und rechten peak ermitteln
			val_f= ( LeftPeak.Pt[ 1 ] - RightPeak.Pt[ 1 ] );
			//! wenn linker peak == rechter peak dann rechte Peakpos. verwerfen

			if ( val_f != 0.0 )
			{
				//! wenn gleicher kanal fuer linken und rechten peak dann
				//! Fehlermeldung
				val_f1= ( RightPeak.Pt[ 0 ] - LeftPeak.Pt[ 0 ] );
				if ( val_f1 != 0 )
				{
					//! Winkel je kanal ausrechnen und an Psd uebergeben
					dStepWidth= CalcValueFromUnit(mGetUnit(), -val_f / val_f1); //! FIX Fehler 48
				} else {
					MessageBox( "Vorgang bitte wiederholen !", "Fehler", MBINFO );
					//! FIX Fehler 58
					//! reset aller wichtigen Werte
					LeftPeak.Valid= FALSE;
					RightPeak.Valid= FALSE;
					bGetRightPeak= FALSE;
					bGetLeftPeak= FALSE;
					CtrlSetEnabled( cm_LeftPositionReached, TRUE );
					CtrlSetEnabled( cm_RightPositionReached, TRUE );
					return;
				}
			} else {
				LeftPeak.Valid= FALSE;
				bGetLeftPeak= TRUE;
			}
		}

		//! sperren bzw. freigeben des linke-Position-erreicht-Button
		CtrlSetEnabled( cm_LeftPositionReached, bGetLeftPeak );
	}

	OnParamSet(); //! Winkel je Kanal-Wert in Dialogbox	

	// 12.06.2004 Psd->SetAddedChannels( 2 );

	Psd->MeasureStart();
}
//-----------------------------------------------------------------------------

void TCalibratePsdDlg::OnRightPositionReached()
{
	//! wenn Motor in bewegung abbrechen
	if ( !mIsMoveFinish() ) return;

	Psd->MeasureStop();
	bGetRightPeak= TRUE;
	bGetLeftPeak= FALSE;
	//! 2Theta winkel speichern
	RightPeak.Pt[1]= mGetValue( Distance );
	// 12.06.2004 Psd->SetAddedChannels( 1 );
	Psd->MeasureStart();
	SetFocus( hBar );
}
//-----------------------------------------------------------------------------

void TCalibratePsdDlg::OnLeftPositionReached()
{
	//! wenn Motor in bewegung abbrechen
	if ( !mIsMoveFinish() ) return;

	Psd->MeasureStop();
	bGetRightPeak= FALSE;
	bGetLeftPeak= TRUE;
	//! 2Theta winkel speichern
	LeftPeak.Pt[1]= mGetValue( Distance );
	// 12.06.2004 Psd->SetAddedChannels( 1 );
	Psd->MeasureStart();
	SetFocus( hBar );
}
//-----------------------------------------------------------------------------

double TCalibratePsdDlg::GetBarScale ( void ) const 
{
	double angleMin= min( mGetAngleRelativeMin(), mGetAngleRelativeMax() );
	double angleMax= max( mGetAngleRelativeMin(), mGetAngleRelativeMax() );
	double diff= angleMax - angleMin;
	try 
	{
		return 32000 / diff;
	} 
	catch (...) 
	{
		return 0; // sonst div 0
	}		
}
//-----------------------------------------------------------------------------

//! liefert die Position des Scrollbar anhand der akt. Motorposition im
//!Rueckkehrcode
int TCalibratePsdDlg::GetBarPos() const
{
	double angle;
	mGetDistance(angle);
	double angleMin= min( mGetAngleRelativeMin(), mGetAngleRelativeMax() );
	return (int) ( (angle-angleMin)*GetBarScale());
}
//-----------------------------------------------------------------------------

//! ermittelt Anzahl der Scrollbarschritte links bzw. rechts von der abs. Null
int TCalibratePsdDlg::GetBarEgde( BOOL aLeft ) const
{
	if ( !aLeft )
	{
		double angleMin= min( mGetAngleRelativeMin(), mGetAngleRelativeMax() );
		double angleMax= max( mGetAngleRelativeMin(), mGetAngleRelativeMax() );
		return (int) ( (angleMax-angleMin)*GetBarScale() );
	}
	else
		return 0;
}
