/*
$Header: /vol/baal-vol3/projekt98/quellen/XCTL_32/DETECUSE/detecuse.cpp,v 1.39 2004/07/30 04:21:56 kullmann Exp $

Projekt   : XCTL
Subsystem : Detektoren
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
          Institut fuer Informatik,
          Humboldt-Universitaet Berlin
Inhalt    : Code fuer das Interface des Detektoren-Subsystems
*/

#include "utils\u_utils.h"
#include "winresrc\rc_def.h" // Ressourcen-IDs
#include <string.h>

// TCurve
//#include "datavisa\datavisa.h"

#include "detecuse\detecuse.h"
#include "detecuse\detecmes.h"
#include "detecuse\detecgui.h"

#include "utils\CheckMemory.h"	//DUMPMEM

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

// #############################################################################
// Implementationsdetails fuer TDetectorManager
//
// Da BC45 den STL-Datentyp vector<TDetector*> nicht unterstuetzt,
// hier eine Implementation als verkettete Liste.
// Diese Details sind absichtlich ausserhalb der Klassendeklaration von
// TDetectorManager realisiert, damit diese Implementation nach Umstieg auf VC++
// geaendert werden kann.
// #############################################################################

//! Verkettete Liste von Zeigern auf TDetector-Objekte
class TDListNode
{
public:
	TDListNode();
	~TDListNode();

	//! fuegt einen TDetector-Zeiger zum Container hinzu
	void push_back(TDetector*);

	//! gibt den im entsprechenden Knoten gespeicherten TDetector-Zeiger zurueck
	TDetector* node()
	{
		return m_detector;
	};

	//! gibt den naechsten Knoten der verketteten Liste zurueck
	TDListNode* next()
	{
		return m_next;
	};

	//! gibt die Laenge der Liste zurueck
	int size()
	{
		return m_size;
	};

private:

	//! der in diesem Knoten gespeicherte Detektor
	TDetector* m_detector;

	//! Zeiger auf den naechsten Knoten der Liste
	TDListNode* m_next;

	//! Laenge der Liste
	int m_size;
};

TDListNode::TDListNode()
{
	m_next= 0;
	m_detector= 0;
	m_size= 0;
}

TDListNode::~TDListNode()
{
	// rekursive Destruktion
	_FREEOBJ(m_next);

	_FREEOBJ(m_detector);
}

void TDListNode::push_back(TDetector* param)
{
	if ( m_detector == 0 )
		m_detector= param;
	else
	{
		if (m_next == 0)
			m_next= new TDListNode();

		m_next->push_back(param);
	}
	m_size++;
}

static TDListNode* DetectorContainer;

//******************************************************************************
//########### Methoden zum Verwalten der Detektoren ############################
//******************************************************************************

TDetectorManager::TDetectorManager()
{
	SetInfo("Erstellen der Detektoren-Liste");

	DetectorContainer= new TDListNode();

	int id= 0;
	char Ident[20];
	char buf[127];

	// Alle "deviceX"-Abschnitte der hardware.ini abarbeiten,
	// fuer jeden gueltigen Abschnitt wird ein TDetector-Objekt erstellt
	do
	{
		sprintf( Ident, "Device%d", id );
		GetPrivateProfileString( Ident, "Type", "", buf, MaxString, GetHWFile() );

		// Der kuerzeste Typ ist "PSD", jeder kuerzere Wert ist entweder falsch
		// oder gar nicht deklariert ( buf == "" )
		if ( strlen(buf) < 3)
			break;

		CreateDetector(buf, id);
		id++;
	}
	while (1);

	if (DetectorContainer->node() == 0)
	{
		MessageBox(0, "Es wurde kein nutzbarer Detektor gefunden.\n"
				   "Das Programm wird jetzt beendet",
				   "Kritischer Fehler", MBFAILURE);

		_FREEOBJ(DetectorContainer);	//hp
		exit(4);
	}

	ActiveDetector= DetectorContainer->node();
}

TDetectorManager::~TDetectorManager( void )
{
	_FREEOBJ(DetectorContainer);
};

TDetectorManager& TDetectorManager::DetectorManager()
{
	// die Singleton-Instanz
	static TDetectorManager theDetectorManager;

	return theDetectorManager;
}

void TDetectorManager::CreateDetector(const char* type, int id)
{
	TDetector* newDetector;

	char szDetectorType[MaxString];
	strcpy(szDetectorType, type);
	strupr(szDetectorType);

	if ( strcmp( szDetectorType, "GENERIC" ) == 0)
		newDetector= new TGenericDetector(id);
	else
		if ( strcmp( szDetectorType, "STOE-PSD" ) == 0)
			newDetector= new TStoePsd(id);
		else
			if ( strcmp( szDetectorType, "RADICON" ) == 0)
				newDetector= new TRadicon(id);
			else
				if ( strcmp( szDetectorType, "BRAUN-PSD" ) == 0)
					newDetector= new TBraunPsd(id);
				else
					if ( strcmp( szDetectorType, "PSD" ) == 0)
						newDetector= new TOneDimTestDetector(id);
					else
						if ( strcmp( szDetectorType, "TEST" ) == 0)
							newDetector= new Testdev(id);
						else
							newDetector= new TZeroDimSimpleTestDetector(id);

	char buf[127];
	sprintf( buf, "Initialisieren des Detektors %s",
			 newDetector->GetCharacteristic() );
	SetInfo( buf );

	// Versuch, den Detektor zu initalisieren. Gibt Initialize() FALSE
	// zurueck, ist der Detektor nicht nutzbar. Das Detektor-Objekt
	// wird zerstoert und nicht in den DetectorContainer eingefuegt.
	if (newDetector->Initialize())
		DetectorContainer->push_back(newDetector);
	else
	{
		sprintf( buf, "Detektor %s konnte nicht initialisiert werden.",
				 newDetector->GetCharacteristic() );
		MessageBox( GetFocus( ), buf, "Detector-Fehler", MBSTOP );

		_FREEOBJ(newDetector);
	}
}

BOOL TDetectorManager::DimAvailable(int dimension) const
{
	// Da sich die Liste der nutzbaren Detektoren nicht nach der Erstellung
	// nicht mehr aendert, kann das Ergebnis zwischengespeichert werden.
	static BOOL bDim[3]= { FALSE, FALSE, FALSE };
	static BOOL bIsCached= FALSE;

	if ( ( dimension < 0 ) || ( dimension > 2) )
		return FALSE;

	if ( bIsCached == FALSE )
	{
		TDListNode* DCIterator= DetectorContainer;

		// Iterieren ueber den Container und Ermitteln, ob von jeder Dimension
		// mindestens ein Detektor nutzbar ist.
		while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
		{
			bDim[DCIterator->node()->GetDimension()]= TRUE;
			DCIterator= DCIterator->next();
		}

		bIsCached= TRUE;
	}

	return bDim[dimension];
}

BOOL TDetectorManager::IsValidId(int n) const
{
	return ( ( n >= 0 ) && ( n <= DetectorContainer->size()) );
}

TDetector* TDetectorManager::GetDetector( void ) const
{
	return ActiveDetector;
}

TDetector* TDetectorManager::GetDetector( int n, int nFilterDimension ) const
{
	if (! IsValidId(n) )
		return ActiveDetector;

	TDListNode* DCIterator= DetectorContainer;
	TDetector* theDetector;
	int nCounter= 0;

	// nur wenn die Dimension gefiltert wird
	if ( nFilterDimension == 0 || nFilterDimension == 1 )
	{
		while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
		{
			theDetector= DCIterator->node();
			if ( theDetector->GetDimension() == nFilterDimension )
			{
				// nur wenn es der nte Detektor der gesuchten Dimension ist
				if ( nCounter == n )
					return theDetector;

				nCounter++;
			}
			DCIterator= DCIterator->next();
		}
		// nicht gefunden
		return ActiveDetector;
	}

	while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
	{
		// den nten Detektor zurueckgeben
		if ( nCounter == n )
			return DCIterator->node();
		nCounter++;

		DCIterator= DCIterator->next();
	}

	return ActiveDetector;
}

BOOL TDetectorManager::SetDetector( int n, int nFilterDimension )
{
	if (! IsValidId(n) )
		return FALSE;

	TDListNode* DCIterator= DetectorContainer;
	TDetector* theDetector;
	int nCounter= 0;

	// nur wenn die Dimension gefiltert wird
	if ( nFilterDimension == 0 || nFilterDimension == 1 )
	{
		while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
		{
			theDetector= DCIterator->node();
			if ( theDetector->GetDimension() == nFilterDimension )
			{
				// nur wenn es der nte Detektor der gesuchten Dimension ist
				if ( nCounter == n )
				{
					ActiveDetector= theDetector;
					return TRUE;
				}

				nCounter++;
			}
			DCIterator= DCIterator->next();
		}
		return FALSE;
	}

	while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
	{
		// den nten Detektor aktivieren
		if ( nCounter == n )
		{
			ActiveDetector= DCIterator->node();
			return TRUE;
		}
		nCounter++;
		DCIterator= DCIterator->next();
	}

	return FALSE;
}

BOOL TDetectorManager::SetDetector( const TDetector* newDetector )
{
	TDListNode* DCIterator= DetectorContainer;
	TDetector* theDetector;

	// nur wenn dieser Detektor in unserer Liste vorkommt
	// eigentlich eine Vorsichtsmassnahmen
	while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
	{
		theDetector= DCIterator->node();
		if ( theDetector == newDetector )
		{
			ActiveDetector= theDetector;
			return TRUE;
		}
		DCIterator= DCIterator->next();
	}
	return FALSE;
}

void TDetectorManager::FillDetectorComboBox( HWND comboBox, int nFilterDimension) const
{
	TDListNode* DCIterator= DetectorContainer;
	TDetector* theDetector;

	while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
	{
		theDetector= DCIterator->node();
		switch (nFilterDimension)
		{
			case 0:
			case 1:
			case 2:
				if ( theDetector->GetDimension() == nFilterDimension )
					ComboBox_AddString( comboBox, theDetector->GetCharacteristic() );
				break;

			default:
				ComboBox_AddString( comboBox, theDetector->GetCharacteristic() );
				break;
		}
		DCIterator= DCIterator->next();
	}
}

int TDetectorManager::GetIdByDescription( LPCSTR detectorname ) const
{
	int nCounter= 0;

	TDListNode* DCIterator= DetectorContainer;

	while ( ( DCIterator != 0 ) && ( DCIterator->node() != 0 ) )
	{
		if ( strcmp( DCIterator->node()->GetCharacteristic(), detectorname ) == 0 )
			return nCounter;

		nCounter++;
		DCIterator= DCIterator->next();
	}
	return -1;
}

//******************************************************************************
// TDetector
//******************************************************************************

TDetector::TDetector(int id)
{
	nId= id;
	exposure= TExposureSettings( 0.7f, 20000 );
	fIntensity= 1.0f;
	fSigma= 0.1f;
	bDataValid= FALSE;
	bActive= FALSE;
	hWndFrame= GetFrameHandle();
	hControlWnd= NULL;
	hCounterWnd= NULL;
	uAskTime= 300;

	LoadDetectorSettings();
}

TDetector::~TDetector( )
{
	SaveDetectorSettings();
}

void TDetector::LoadDetectorSettings()
{
	char buf[ MaxString ];

	GetPrivateProfileString( "Name", "Counter", szCharacteristic);

	GetPrivateProfileString( "Debug", "0", buf);
	bDebug= ( atoi( buf ) != 0 );

	GetPrivateProfileString( "ExposureTime", "4.0", buf);
	exposure.SetExposureTime( atof( buf ));

	GetPrivateProfileString( "ExposureCounts", "10000", buf);
	exposure.SetExposureCounts( atol( buf ));
}

void TDetector::SaveDetectorSettings() const
{
	char buf[ MaxString ];

	sprintf( buf, "%.2f", exposure.GetExposureTime() );
	WritePrivateProfileString( "ExposureTime", buf);
	sprintf( buf, "%ld", exposure.GetExposureCounts() );
	WritePrivateProfileString( "ExposureCounts", buf);
}

BOOL TDetector::SetExposureSettings (const TExposureSettings& newExposureSettings)
{
	exposure= newExposureSettings;

	return SetParameters();
}

BOOL TDetector::Initialize( void )
{
	PushSettings( );
	return TRUE;
}

void TDetector::UpdateViews(BOOL bDisplayOnly) const
{
	if ( GetCounterWnd() )
		// Display benachrichtigen
		PostMessage( GetCounterWnd(), WM_COMMAND, cm_CounterSet, nId );

	if ( bDisplayOnly )
		return;

	if ( GetControlWnd() )
		// Daten-Empfaenger benachrichtigen
		PostMessage( GetControlWnd(), WM_COMMAND, cm_CounterSet, nId );
}

BOOL TDetector::SetParameters( void )
{
	// Wenn keine Messung lief, einfach Parameter setzen
	if ( bActive == FALSE )
		return _SetParameters();

	// Sonst Messung erst beenden, Parameter setzen und Messung wieder starten
	MeasureStop();
	BOOL bRetCode= _SetParameters();
	MeasureStart();
	return bRetCode;
}

TDetector::PushSettings( void )
{
	Settings.exposure= exposure;
	return TRUE;
}

TDetector::PopSettings( void )
{
	exposure= Settings.exposure;

	SetParameters( );

	return TRUE;
}

//! Einfache Refaktorisierung (Methode extrahieren)
//! Laedt den Wert fuer einen angegebenen Schluessel aus der hardware.ini
//! und schreibt bei Bedarf einen Default-Wert zurueck.
DWORD TDetector::GetPrivateProfileString(LPCSTR lpKeyName, LPCSTR lpDefault, LPSTR lpReturnedString, UINT iMaxLen ) const
{
	char Ident[20];
	sprintf( Ident, "Device%d", GetId() );

	DWORD ret= ::GetPrivateProfileString(
					Ident, lpKeyName, lpDefault, lpReturnedString, iMaxLen, GetHWFile() );

	if ( CreateDefaults() )
		::WritePrivateProfileString( Ident, lpKeyName, lpReturnedString, GetHWFile());

	return ret;
}

DWORD
TDetector::WritePrivateProfileString( LPCSTR lpKeyName, LPCSTR lpString) const
{
	char Ident[20];
	sprintf( Ident, "Device%d", GetId() );

	return ::WritePrivateProfileString( Ident, lpKeyName, lpString, GetHWFile());
}

float TDetector::CalculateSigma(float intensity, float factor)
{
	if ( intensity == 0.0 )
		return 1.0;

	return factor * sqrt(intensity) / intensity;
}

double TDetector::MillisecondsToSeconds( DWORD milliseconds )
{
	double integerPart= ( (double)milliseconds ) / 1000.0;
	double fractionalPart= ( (double)(milliseconds % 1000) ) / 1000.0;
	return ( integerPart + fractionalPart );
}

//############################################################################
// TZeroDimDetector
//############################################################################

// time event handling
int nEventsToCall;
int nEvent;
int nEventsCalled;
float fEventIntensity;
HWND hEventControlWnd;
BOOL bEventDataValid;

TZeroDimDetector::TZeroDimDetector(int id) : TDetector(id)
{
	bSound= FALSE;
	LoadDetectorSettings();
}

TZeroDimDetector::~TZeroDimDetector()
{
	SaveDetectorSettings();
}

void TZeroDimDetector::SetSound(BOOL bSoundParam)
{
	bSound= bSoundParam;
}

void TZeroDimDetector::LoadDetectorSettings()
{
	char buf[ MaxString ];

	GetPrivateProfileString( "Sound", "1", buf);
	bSound= atoi( buf );
}

void TZeroDimDetector::SaveDetectorSettings() const
{
	char buf[ MaxString ];

	sprintf( buf, "%d", bSound );
	WritePrivateProfileString( "Sound", buf);
}

BOOL TZeroDimDetector::Initialize( void )
{
	return TDetector::Initialize();
}

int TZeroDimDetector::MeasureStart( void )
{
	if (bActive)
		MeasureStop( );

	if ( bDebug )
		SetInfo( "MStart" );

	if ( ! GetControlWnd() && ! GetCounterWnd() )
		return R_Failure;

	uAskTime= TRange<UINT>(500, 7000).ForceIntoRange(
				   GetExposureSettings().GetExposureTime() * 1100 );

	SetTimer( hWndFrame, DetectorTimerIdStart + GetId(), uAskTime, NULL );
	bDataValid= FALSE;
	bActive= TRUE;
	return R_MeasOk;
}

int TZeroDimDetector::MeasureStop( void )
{
	if ( !bActive )
		return R_MeasOk;

	KillTimer( hWndFrame, DetectorTimerIdStart + GetId() );

	if ( bDebug && bActive )
		SetInfo( "MStop" );

	bActive= FALSE;

	return R_MeasOk;
}

int TZeroDimDetector::GetIntegral( float &data )
{
	data= 0.0;

	if ( !bDataValid )
		return R_Failure;

	data= fIntensity;

	fSigma= CalculateSigma ( fIntensity );

	return R_OK;
}

//! Methode wird periodisch aufgerufen (Initialisierung in InitializeEvent)
//! realisiert Intensitaetsmessung waehrend dem zeitintervall zwischen zwei
//! Aufrufen der Methode (fuer kontinuierlichen Scan)
//! anschliessend wird das Scanfenster benachrichtigt
//! (cm_CounterSet waehrend messung; cm_SteeringReady bei letzter Messung)
//! hier nur Simulation
CALLBACK TZeroDimDetector::EventHandler( UINT, UINT, DWORD, DWORD, DWORD )
{
	double angx;

	nEventsCalled++;
	// mlGetDistance(angx);
	angx= ( nEventsCalled - nEventsToCall / 2 ) * 0.246;
	angx= angx * angx / 11.3;
	fEventIntensity= 32000 / ( 1 + 1.8 * angx * angx ) + ( rand( ) % 80 ) + 270;

	//! Messwert gueltig
	bEventDataValid= TRUE;
	PostMessage( hEventControlWnd, WM_COMMAND, cm_CounterSet, nEventsCalled );

	if ( nEventsCalled == nEventsToCall )
	{
		timeKillEvent( nEvent );
		PostMessage( hEventControlWnd, WM_COMMAND, cm_SteeringReady, 0l );
		//    bEventDataValid= FALSE;
	}

	return R_OK;
}

//! macht dem Detektor das Fenster bekannt, an das er innerhalb von EventHandler
//! seine Nachrichten schicken soll
//! (fuer kontinuierlichen Scan)
void TZeroDimDetector::SetEventHost( HWND hwnd)
{
	hEventControlWnd= hwnd;
}

//! loescht Detektor-Event
//! sorgt, dafuer dass Eventhandler nicht mehr aufgerufen wird
//! (fuer kontinuierlichen Scan)
void TZeroDimDetector::KillEvent(void)
{
	timeKillEvent( nEvent );
}

//! liefert die durch EventHandler gemessene Intensitaet im Parameter zurueck
//! Rueckkehrcode TRUE
//! nur einmal Auslesen eines Wertes erlaubt
//! bei wiederholtem Ausleseversuch 0 in Parameter und
//! Rueckkehrcode FALSE
//! (fuer kontinuierlichen Scan)
BOOL TZeroDimDetector::GetEventIntensity( float &data)
{
	data= 0;

	if (!bEventDataValid)
		return FALSE;

	data= fEventIntensity;

	bEventDataValid= FALSE;

	return TRUE;
}

//! initialisiert ein periodisches Aufrufen der Methode EventHandler
//! 2. Parameter gibt Anzahl der Aufrufe an
//! Aufrufintervall entspricht der Messzeit
//! Rueckkehrcode FALSE wenn Aufrufintervall < 0.1 sek.
//! sonst TRUE (wenn erfolgreich)
//! (fuer kontinuierlichen Scan)
BOOL TZeroDimDetector::InitializeEvent( HWND hwnd, int ec )
{
	int EventTime= 1000.0 * exposure.GetExposureTime();

	SetControlWnd(hwnd);
	nEventsCalled= 0;
	nEventsToCall= ec;
	// check calltime

	if ( EventTime <= 100 )
	{
		MessageBox( GetFocus( ), "Mezeit wurde zu klein gewhlt !", "InitializeEvent", MBINFO );
		return FALSE;
	}

	if ( NULL != GetControlWnd() )
		nEvent= timeSetEvent( EventTime, 1, ( LPTIMECALLBACK ) TZeroDimDetector::EventHandler, 0, TIME_PERIODIC );

	bDataValid= FALSE;

	return TRUE;
}

//############################################################################
// TOneDimDetector
//############################################################################
TOneDimDetector::TOneDimDetector(int id) : TDetector(id), hCountBuf(0)
{
	SetDataType( PsdPositionData );
	bSoftOverflow= bHardOverflow= bSignalGrowUp= bHVControl= FALSE;
	m_Axis= (EAxisType)0;

	strcpy(szUnit, "");
	LoadDetectorSettings();
}

TOneDimDetector::~TOneDimDetector( )
{
	//21.08.2004 MEMCORRECT if (hCountBuf) HeapFree(GetProcessHeap(),0,hCountBuf);  hCountBuf= 0;
	_FREELIST(hCountBuf);

	SaveDetectorSettings();
}

void TOneDimDetector::LoadDetectorSettings()
{
	char buf[ MaxString ];

	GetPrivateProfileString( "EnergyScale", "2", buf);
	SetEnergyScale( (UINT)atoi(buf) );

	GetPrivateProfileString( "PositionScale", "1", buf);
	SetPositionScale( (UINT)atoi(buf) );

	GetPrivateProfileString( "OverflowIntensity", "50000.0", buf);
	OverflowIntensity= atof( buf );

	GetPrivateProfileString( "SignalGrowUp", "0", buf);
	bSignalGrowUp= ( atoi( buf ) != 0 );

	GetPrivateProfileString( "HVRegelung", "0", buf);
	bHVControl= ( atoi( buf ) != 0 );

	GetPrivateProfileString( "ReadLeftFirst", "1", buf);
	bReadLeftFirst= ( atoi ( buf ) != 0 );

	GetPrivateProfileString( "Unit", "Sekunden", szUnit, _MAXLENUNIT);
	eUnit= UnitEnum( szUnit );

	GetPrivateProfileString( "AngleStep", "1.0", buf);
	fAngleStep= atof( buf ); // in Motoreinheit und fr Scale=0 lesen; d.h. nicht SetAngleStep benutzen - 

	GetPrivateProfileString( "Digits", "5", buf ); // Nachkommastellen fr die Kanalbreite in arcsec
    nDigits= atoi( buf );

	GetPrivateProfileString( "MaxChannels", "4096", buf);
	nMaxChannels= atoi( buf );

	// First- und LastChannel direkt einlesen, spter bei SetChannelRange ggf. korrigieren
	GetPrivateProfileString( "FirstChannel", "0", buf);
	int firstCh= atoi( buf );

	GetPrivateProfileString( "LastChannel", "4095", buf);
	int lastCh= atoi( buf );

	// bentigt <uPositionScale>, <nMaxChannels> und <eDataType>==PsdPositionData
	SetChannelRange( firstCh, lastCh );

	// bentigt First und LastChannel
	GetPrivateProfileString( "AddedChannels", "4", buf);
	SetAddedChannels( atoi( buf ) );

	GetPrivateProfileString( "SteeringAxis", "0", buf);
	m_Axis= mlParsingAxis(buf); // ggf. (TAxisType)0
}

void TOneDimDetector::SaveDetectorSettings() const
{
	char aBuf[ MaxString ];
	char aFormat[ MaxString ];

	sprintf( aBuf, "%u", uEnergyScale );
	WritePrivateProfileString( "EnergyScale", aBuf);

	sprintf( aBuf, "%u", uPositionScale );
	WritePrivateProfileString( "PositionScale", aBuf);

	sprintf( aBuf, "%d", ( bHVControl ? 1 : 0 ) );
	WritePrivateProfileString( "HVRegelung", aBuf);

	WritePrivateProfileString( "Unit", szUnit);

	sprintf( aBuf, "%d", nMaxChannels);
	WritePrivateProfileString( "MaxChannels", aBuf);	

	sprintf( aBuf, "%d", GetFirstChannel() ); // umrechnen auf Scale
	WritePrivateProfileString( "FirstChannel", aBuf);

	sprintf( aBuf, "%d", GetLastChannel() ); // umrechnen auf Scale
	WritePrivateProfileString( "LastChannel", aBuf );

	sprintf( aBuf, "%d", nAddedChannels );
	WritePrivateProfileString( "AddedChannels", aBuf);

	sprintf( aBuf, "%f", OverflowIntensity );
	WritePrivateProfileString( "OverflowIntensity", aBuf);

	sprintf( aBuf, "%d", ( bSignalGrowUp ? 1 : 0 ) );
	WritePrivateProfileString( "SignalGrowUp", aBuf);

	sprintf( aBuf, "%d", ( bReadLeftFirst ? 1 : 0 ) );
	WritePrivateProfileString( "ReadLeftFirst", aBuf);

	sprintf( aBuf, "%d", nDigits );
	WritePrivateProfileString( "Digits", aBuf); // Nachkommastellen der Kanalbreite in arcsec speichern
		
	sprintf(aFormat, "%s%d%s", "%.", nDigits, "f"); // Anzahl der Nachkommastellen beachten
	sprintf( aBuf, aFormat, fAngleStep ); // in Motoreinheit und fr Scale=0 speichern; d.h. nicht GetAngleStep benutzen
	WritePrivateProfileString( "AngleStep", aBuf);

	WritePrivateProfileString( "MJ_Channel", "0"); // 10.12.2002 Kulllmann, Reinecker Kanal-Offset lschen
}

BOOL TOneDimDetector::Initialize( void )
{
	TDetector::Initialize( );

	if ( GetWidth() == 0.0 )
	{
		MessageBox( GetFocus( ), "Psd ist nicht kalibriert !", "Meldung", MBSTOP );
		fAngleStep= 1.0;
		SetAddedChannels( 4 );
	}

	// allocate memory
	//21.08.2004 MEMCORRECT hCountBuf= (HGLOBAL)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,GetBufferSize() * sizeof(DWORD));
	hCountBuf= new char[ GetBufferSize() * sizeof(DWORD) ];

	//hp initial hardware check
	if (PsdInit() != R_OK)
		return(false);

	PushSettings( );

	return TRUE;
}

BOOL TOneDimDetector::RunSpecificParametersDlg( HWND aParentWnd )
{
	//! neu Kullmann+Reinecker: TheModeless durch PsdParamDlg ersetzt; extern entfernt
	static TModelessDlg *PsdParamDlg= 0;

	if ( (!PsdParamDlg) || (!PsdParamDlg->GetHandle()) )
	{ //! GetHandle() ist wichtig fr den Fall, dass sich das Dialogfenster selbst zerstrt hat !!!
		PsdParamDlg= new TPsdParametersDlg(&PsdParamDlg);
		if ( PsdParamDlg ) PsdParamDlg->Initialize( dlGetInstance(), aParentWnd );
		return TRUE;
	}
	else
	{
		_FREEOBJ(PsdParamDlg);
		return FALSE;
	}
}

void TOneDimDetector::SetAddedChannels(int nParam)
{
	nAddedChannels= TRange<int>(1, GetChannelNumber()).ForceIntoRange(nParam);
}

int TOneDimDetector::GetFirstChannel() const 
{
	if ( eDataType == PsdEnergyData )		
		return 0; // volle Kanalbreite benutzen
	else
		// FirstChannel auf Scale umrechnen
		return max( 0, min(GetScaledMaxChannels(uPositionScale)-1, nFirstChannel >> uPositionScale) );
};

int TOneDimDetector::GetLastChannel() const 
{
	if ( eDataType == PsdEnergyData )		
		return GetScaledMaxChannels(uEnergyScale)-1; // volle Kanalbreite benutzen
	else
		// LastChannel auf Scale umrechnen
		return max( 0, min(GetScaledMaxChannels(uPositionScale)-1, nLastChannel >> uPositionScale) );
};

void TOneDimDetector::SetChannelRange(int first, int last)
{
	if ( eDataType == PsdEnergyData ) return; // First- und LastChannel nicht verndern (sind bei PsdEnergyData allein abhngig von nMaxChannels und EnergyScale)

	// nochrechnen auf <MaxChannels>-Kanle - siehe PsdReadOut
	TRange<int> limit(0, nMaxChannels-1);
	first= limit.ForceIntoRange(first << uPositionScale);
	last= limit.ForceIntoRange(last << uPositionScale);

	nFirstChannel= min(first, last);
	nLastChannel= max(first, last);
}

UINT TOneDimDetector::GetMaximumChannel( void ) const
{
	if ( bReadLeftFirst )
		return uMaximumChannel;

	return ( GetChannelNumber() - uMaximumChannel );
}

// NEU: Bercksichtigt alle Messwerte und bertrgt diese in die Kurve
int TOneDimDetector::FillInCurve( TCurve &curve, EUnitType aMotorUnit )
{
	// Vorbedingungen
	if ( bHardOverflow )
		return R_HardOverflow;

	UINT addC= max(1, nAddedChannels);

	LPDWORD lpdwCountBuf= (LPDWORD)hCountBuf;

	curve.New( );
	// 07.05.2004 curve.FastOpen( );

	double dAngle= 0.0;
	double intens= 0.0;
	double dWidth= CalcValueInUnit( aMotorUnit, GetAngleStep() ); // GetAngleStep - unabhngig von <AddedChannels>
	for ( int idx= 0;idx<GetChannelNumber(); idx++ ) {		
		int index;
		if ( bReadLeftFirst )
			index= idx;
		else // Lesereihenfolge umdrehen (hinten anfangen und dann nach vorn)
			index= (GetChannelNumber()-1) - idx; 

		intens+= (float)lpdwCountBuf[index]; // <intens> zuerst inkrementieren
		if ( (idx+1) % addC==0 ) { // Kanalgruppe vollstndig
			// Zwischenspeichern in floats, da curve.PAdd Referenzen verlangt
			float x= dAngle;
			float y= intens;
			float z= idx;
			curve.PAdd( x, y, z ); // 07.05.2004 FastP...
			intens= 0; // neue Kanalgruppe beginnt
		}
		dAngle += dWidth; // <dAngle> als letztes inkrementieren
	}

	// 07.05.2004 curve.FastClose( );

	//! neu akk	
	fIntensity= dwIntegratedCounts; //fIntensity= dwIntegratedCounts / dRealTime;
	fSigma= CalculateSigma( fIntensity );

	// Ist ein "weicher" Overflow aufgetreten ?
	if ( fIntensity >= OverflowIntensity )
	{
		bSoftOverflow= TRUE;
		return R_SoftOverflow;
	}

	return R_OK;
}

/*10.07.2004 // ALT: Nicht alle Messwerte werden bercksichtigt, es fehlt IMMER der letzte Wert.
int TOneDimDetector::FillInCurve( TCurve &curve, EUnitType aMotorUnit )
{
	// Vorbedingungen
	if ( bHardOverflow )
		return R_HardOverflow;

	UINT addC= max(1, nAddedChannels);

	LPDWORD lpdwCountBuf= (LPDWORD)hCountBuf;

	curve.New( );
	// 07.05.2004 curve.FastOpen( );

	double dAngle= 0.0;
	double intens= 0.0;
	double dWidth= CalcValueInUnit( aMotorUnit, GetWidth()); // GetWidth - bercksichtigt <AddedChannels>
	for ( int idx= 0;idx < (GetChannelNumber() - addC); idx += addC )
	{
		intens= 0.0;

		for ( int i= 0; i < addC; i++ )
		{
			int index;
			// Summe pro Kanalgruppe ermitteln
			index= idx + i;
			if ( !bReadLeftFirst )
				index= ( GetChannelNumber() - 1 ) - index;

			intens += ( float ) lpdwCountBuf[index];
		}

		// Zwischenspeichern in floats, da curve.PAdd Referenzen verlangt
		float x= dAngle;
		float y= intens;
		float z= idx;
		curve.PAdd( x, y, z ); // 07.05.2004 FastP...

		dAngle += dWidth;
	}

	// 07.05.2004 curve.FastClose( );

	//! neu akk	
	fIntensity= dwIntegratedCounts; //fIntensity= dwIntegratedCounts / dRealTime;
	fSigma= CalculateSigma( fIntensity );

	// Ist ein "weicher" Overflow aufgetreten ?
	if ( fIntensity >= OverflowIntensity )
	{
		bSoftOverflow= TRUE;
		return R_SoftOverflow;
	}

	return R_OK;
}*/

int TOneDimDetector::GetIntegral( float &integral )
{
	integral= 0.0;

	if ( bHardOverflow )
		return R_HardOverflow;

	integral= fIntensity= dwIntegratedCounts;
	fSigma= CalculateSigma( fIntensity );

	// Ist ein "weicher" Overflow aufgetreten ?
	if ( fIntensity >= OverflowIntensity )
	{
		bSoftOverflow= TRUE;
		return R_SoftOverflow;
	}

	return R_OK;
}

int TOneDimDetector::MeasureStart( void )
{
	if (bActive)
		MeasureStop( );

	if ( !GetControlWnd() && !GetCounterWnd() )
		return R_Failure;

	bSoftOverflow= FALSE;
	bHardOverflow= FALSE;
	bFirstReadOut= TRUE;

	if ( R_Failure == PsdStart( ) )
		return R_Failure;

	uAskTime= TRange<UINT>(500, 2000).ForceIntoRange(
				   GetExposureSettings().GetExposureTime() * 1020 );

	// Start-Zeit merken
	dwStartTime= timeGetTime( );

	SetTimer( hWndFrame, DetectorTimerIdStart + GetId( ), uAskTime, NULL );

	bActive= TRUE;
	bDataValid= FALSE;

	return R_MeasOk;
}

int TOneDimDetector::MeasureStop( void )
{
	if ( !bActive )
		return R_MeasOk;

	KillTimer( hWndFrame, DetectorTimerIdStart + GetId( ) );

	bDataValid= FALSE;

	PsdStop( );

	bActive= FALSE;

	return R_MeasOk;
}

int TOneDimDetector::PollDetector( void )
{
	char buf[ MaxString ];
	int retval= R_MeasOk;

	KillTimer( hWndFrame, DetectorTimerIdStart + GetId( ) );

	if ( !bActive )
		return R_Failure;

	bActive= FALSE;

	dwElapsedTime= timeGetTime( ) - dwStartTime;

	if ( MillisecondsToSeconds( dwElapsedTime ) < GetExposureSettings().GetExposureTime() )
	{
		retval= R_MeasInProcess;
		bActive= TRUE;

		if ( !bSignalGrowUp )
		{
			//*** Kein stufenweiser Signalaufbau
			SetTimer( hWndFrame, DetectorTimerIdStart + GetId( ), uAskTime, NULL );
			return R_MeasInProcess;
		}

		//*** Stufenweiser Signalaufbau
		//FEHLER? Akkumuierter Signalaufbau von TStoePSD::PollDetector bernehmen!
		if ( PsdReadOut( IntermediateRead ) == R_Failure )
			return R_Failure;

		dRealTime= MillisecondsToSeconds( dwElapsedTime );

		bFirstReadOut= FALSE;

		long lWaitTime= GetExposureSettings().GetExposureTime() * 1020 - dwElapsedTime;

		uAskTime= TRange<UINT>(500, 2000).ForceIntoRange( lWaitTime );

		SetTimer( hWndFrame, DetectorTimerIdStart + GetId( ), uAskTime, NULL );

		goto SendNotify;
	}

	if ( PsdStop( ) == R_Failure )
		return R_Failure;

	dRealTime= MillisecondsToSeconds( dwElapsedTime );

	if ( PsdReadOut( FinalRead ) == R_Failure )
		return R_Failure;

	bDataValid= TRUE;

SendNotify:      //*** Benachrichtigen der entsprechenden Fenster ***

	if ( dRealTime == 0.0 )
	{
		SetInfo( "dRealTime == 0.0 Fehler Psd" );
		DelayTime( 300 );
		dRealTime= 1.0;
		PsdStop( );
		bDataValid= FALSE;
		return R_Failure;
	}

	if ( bHardOverflow )
	{
		strcpy( buf, "berlauf Psd ( Hochspannung ? )" );
		MessageBeep( 0 );
		retval= R_HardOverflow;
	}

	UpdateViews();

	return retval;
}

void TOneDimDetector::SetAngleRange( float )
{
	//JP!!!!
	return;
}

BOOL TOneDimDetector::PushSettings( void )
{
	TDetector::PushSettings( );
	Settings.nAddedChannels= nAddedChannels;
	return TRUE;
}

BOOL TOneDimDetector::PopSettings( void )
{
	TDetector::PopSettings( );
	nAddedChannels= Settings.nAddedChannels;
	return TRUE;
}

float TOneDimDetector::GetAngleStep() const
{
	// auf Scale umrechnen; ">>" geht nicht, da <fValParam> float ist
	return fAngleStep * pow(2, uPositionScale); // Kanalbreite nur bei Positionspektrum (Energiespektrum hat nur Kanaldarstellung)
};
void TOneDimDetector::SetAngleStep( float stp )
{
	// nochrechnen auf <MaxChannels>-Kanle: "<<" geht nicht, da <stp> float ist
	fAngleStep= stp / pow(2, uPositionScale); // Kanalbreite nur bei Positionspektrum (Energiespektrum hat nur Kanaldarstellung)
};

float TOneDimDetector::GetWidth() const
{
	return nAddedChannels * GetAngleStep(); 
}

float TOneDimDetector::GetAngleRange() const 
{
	return GetChannelNumber() * GetAngleStep();  
}

UINT TOneDimDetector::GetAngleStepDigits( void ) const
{
	return nDigits;
}

void TOneDimDetector::SetAngleStepDigits(UINT aDigits) 
{
	nDigits = aDigits;
}

void TOneDimDetector::CheckMaximumChannel(UINT channel, DWORD counts)
{
	// Kanal mit Maximum ermitteln
	if ( dwMaxCounts < counts )
	{
		uMaximumChannel= channel;
		dwMaxCounts= counts;
	}
}

UINT TOneDimDetector::SetEnergyScale(UINT aEnergyScale)
{
	UINT result= uEnergyScale;
	uEnergyScale= max( 0x0, min(0x3, aEnergyScale) );
	return result;
}

UINT TOneDimDetector::SetPositionScale(UINT aPositionScale) 
{
	UINT result= uPositionScale;
	uPositionScale= max( 0x0, min(0x3, aPositionScale) );
	return result;
};

//##############################################################################
// TExposureSettings
//##############################################################################

TExposureSettings::TExposureSettings(
	float exposureTime, DWORD exposureCounts)
		: CountBounds(10, 300000L), TimeBounds(0.1f, 500.0f)
{
	SetExposureTime( exposureTime );
	SetExposureCounts( exposureCounts );
}

void TExposureSettings::SetExposureTime(float exposureTime)
{
	fExposureTime= TimeBounds.ForceIntoRange( exposureTime );
}

void TExposureSettings::SetExposureCounts( DWORD exposureCounts )
{
	dwExposureCounts= CountBounds.ForceIntoRange( exposureCounts );
}

//##############################################################################
// TDetectorGUI
//##############################################################################

void TDetectorGUI::RunPsdCalibrateDialog(HWND hwnd, TCurve &curve)
{
	TModalDlg *dlg= new TCalibratePsdDlg( curve );
	if ( dlg ) dlg->ExecuteDialog( hwnd );
	_FREEOBJ(dlg);
}

void TDetectorGUI::RunCommonDevParamDialog( HWND hwnd )
{
	TModalDlg* dlg= new TCommonDevParamDlg();
	if ( dlg ) dlg->ExecuteDialog( hwnd );
	_FREEOBJ(dlg);
}
