//##############################################################################
//                                                                            //
// D_EXTERN.CPP                                                               //
//                                                                            //
// Subsystem: Diffraktometrie/ Reflektometrie                                 //
//            DIE MDI-Fensterbasisklasse                                      //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//                                                                            //
//##############################################################################
//! Kommentierung und Aenderungen B. Buss, 2000/2001, HUB IfI

#include <direct.h>

#include "internls\evrythng.h" // GermanVersion
#include "winresrc\rc_def.h"   // Ressourcen-IDs
#pragma hdrstop

#include "difrkmty\d_extern.h" // SCHNITTSTELLE fr diese Datei
#include "motrstrg\motrstrg.h" // fr Motor-C-Interface

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//! Kurvendatenbasis
__declspec(dllimport) LPDataBase lpDBase;

//#############################################################################
// globale Typen und Konstanten
//#############################################################################

//! Anzahl der max. unterschiedlichen Schrittweiten (dyn. Schrittweite)
const int MaxScaleCount= 10;

const UINT tiles_x_achse= 5; 
const UINT tiles_y_achse= 5; 

const UINT Mindxpos= 200; 
const UINT Maxdxpos= 700; 
const UINT Mindypos= 200; 
const UINT Maxdypos= 700; 

const UINT XBNorm= 85; 
const UINT YBNorm= 62; 

//#############################################################################
// sprachspezifische Konstanten
//#############################################################################

#ifdef GermanVersion
  static const char szMsgLine600[]= "Scanachse ???";
  static const char szMsgLine601[]= "ausserhalb des Bereichs";
  static const char szMsgLine604[]= "Vergleichs-Scan speichern...";
  static const char szMsgLine605[]= "Vergleichs-Scan (*.bk)|*.bk||";
  static const char szMsgLine606[]= "[Header]\nVergleichs-Scan \n";
  static const char szMsgLine607[]= "Linie von (%.*f, %.*f) bis (%.*f, %.*f)";
  static const char szMsgLine608[]= "Theta:%.*f (D:%.3f) Omega:%.*f (D:%.3f) Int.:%.*f Zeile:%d Spalte:%d";
  static const char szMsgLine609[]= "X:%.*f Y:%.*f Theta:%.3f Omega:%.3f Int.:%.*f Zeile:%d Spalte:%d";
  static const char strMessage14[]= "Datei wurde bereits gespeichert !";

  static const char szIntensLi[]=   "Intensitten\nlinear skaliert";
  static const char szIntensLiEZ[]= "linear skaliert";
  static const char szIntensLo[]=   "Intensitten\nlog10 skaliert";
  static const char szIntensLoEZ[]= "log10 skaliert";
#else
  static const char szMsgLine600[]= "Scanaxis ???";
  static const char szMsgLine601[]= "out of Range";
  static const char szMsgLine604[]= "Save Comparison-Scan...";
  static const char szMsgLine605[]= "Comparison-Scan (*.bk)|*.bk||";
  static const char szMsgLine606[]= "[Header]\nComparison-Scan \n";
  static const char szMsgLine607[]= "Line from (%.*f, %.*f) to (%.*f, %.*f)";
  static const char szMsgLine608[]= "Theta:%.*f (D:%.3f) Omega:%.*f (D:%.3f) Int.:%.*f Row:%d Column:%d";
  static const char szMsgLine609[]= "X:%.*f Y:%.*f Theta:%.3f Omega:%.3f Int.:%.*f Row:%d Column:%d";
  static const char strMessage14[]= "File was already saved !";

  static const char szIntensLi[]=   "Intensities\nlinear scaling";
  static const char szIntensLiEZ[]= "linear scaling";
  static const char szIntensLo[]=   "Intensities\nlog10 scaling";
  static const char szIntensLoEZ[]= "log10 scaling";
#endif

//##############################################################################
// TPlotWindow
//##############################################################################

//! Konstruktor TPlotWindow
TPlotWindow::TPlotWindow(HINSTANCE aInstance) : TFileWindow(aInstance), BSource(0), MainCurve(0), SecondCurve(0), hFont(0), DragDC(0)
{
	m_Dotted= FALSE;
	m_HasFilter= FALSE;
	m_FilterStrength= 0.0;
	bEnablePaint= FALSE; // siehe EnablePaint() und DisablePaint()
	WindowType= wtUnknown;
	bMeasurementActive= FALSE;
	bSecondaryCoor= FALSE;

	//! durch einen Aufruf von set_new_handler(0) wird das traditionelle
	//! new-Verhalten wiederhergestellt, bei dem keine Exception ausgeloest wird
	set_new_handler(0);
	//! xST/yST/zST darin als TScaleType (comhead.h) def.
	LowerBound= (float)1e-1; //! gibt beim ersten Aufruf MinIntensitaet vor
	UpperBound= (float)1e5; //! gibt beim ersten Aufruf MaxIntensitaet vor

	eOutputType= otCurve;

	CoorSys.xMin= 0.01;
	CoorSys.xMax= 100.0;
	CoorSys.xScal= sLinear;
	CoorSys.yMax= UpperBound;
	CoorSys.yMin= LowerBound;
	CoorSys.yScal= sLogarithmic;
	CoorSys.zMax= UpperBound;
	CoorSys.zMin= LowerBound;
	CoorSys.zScal= sLinear;
	//! CoorSys wird in AltCoorSys gesichert
	AltCoorSys= CoorSys;
	bAltCoorSystem= FALSE;

	bXY_Scaling= FALSE;
	bButtonDown= FALSE;

	//! Screen - Typ TScreen (m_data.h)
	Screen.x0= 0; 
	Screen.x1= stdx - 1;
	Screen.dx= stdx;
	Screen.y0= 0;
	Screen.y1= stdy - 1;
	Screen.dy= stdy;
	//! SecondCurve - Typ TCurve (m_curve.h)
	SecondCurve= (TCurve *) new TCurve;

	//! Font - Def. in windows.h
	LOGFONT lf;	
	// LogFont spezifizieren...
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight= -13;
	lf.lfWidth= 0; // wird automatisch entsprechend der Hoehe gesetzt
	lf.lfWeight= 400;
	lf.lfUnderline= FALSE;
	lstrcpy(lf.lfFaceName, "Times New Roman");
	//! the CreateFontIndirect function creates a logical font that has
	//! the characteristics specified in the specified structure
	hFont= CreateFontIndirect(&lf);

	HCURSOR hOldCursor= SetDefaultCursor( IDC_WAIT );
	switch ( eOutputType ) { // notify type TOutputType
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			new TBitmapSource(this); // registriert sich selbst fr dieses Fenster
			BSource->FormatDBaseToBitmapSource();
			break;

		default:
			//! setze neue Koordinatensystembereiche
			SetRanges();
	}
	SetCursor(hOldCursor); // restore cursor before waiting cursor

	//! setze Bool-Variable bEnablePaint auf TRUE (m_data.h)
	EnablePaint();
	//! noch keine alten Daten geladen...
	SetLoadFormat(ffUnknown);
	strcpy(abzisse, szMsgLine600);

	VisualDB.bPsd=FALSE; VisualDB.dThetaWindow= VisualDB.dMoveRelation= VisualDB.dOmegaMin= VisualDB.dOmegaMax= VisualDB.dThetaMinFirst= VisualDB.dThetaMaxFirst= VisualDB.dThetaMinLast= VisualDB.dThetaMaxLast= 0; VisualDB.fAngleRange= VisualDB.fAngleStep= 0;
	eLoadFormat= ffUnknown;
	xo= yo= 0;

	// Motorkoordinaten nur bei AreaScan umrechnen
	m_MotorUnit= utNone;
	m_DetectorUnit= utNone;

	// ehemalig extern-Verweise aus D_INTERN.CPP
	tilesx= tiles_x_achse; 
	tilesy= tiles_y_achse;
	scrolldyposmin= Mindypos;
	scrolldyposmax= Maxdypos;
	scrolldxposmin= Mindxpos;
	scrolldxposmax= Maxdxpos;
	xBorder= XBNorm;
	yBorder= YBNorm;
	RLdx= 400; 
	RLdy= 400; 
	bIncreasePP= FALSE;
	MinIntensitaet= (float)1e-5;
	MaxIntensitaet= (float)1e10;	
	mPoint1.x= mPoint1.y= mPoint2.x= mPoint2.y= 0; //! zum Aufzeichnen der Mausbewegungen beim Ziehen einer Linie mit der Maus
	DataaquisitionActive= false;
};
//*****************************************************************************

//! Destruktor TPlotWindow
TPlotWindow::~TPlotWindow()
{
	//! loesche SecondCurve - Typ TCurve (m_curve.h)
	_FREEOBJ(SecondCurve);
	_FREEOBJ(BSource);
	if ( hFont ) {
		DeleteObject(hFont);
		hFont= 0;
	}
};
//*****************************************************************************

BOOL TPlotWindow::New(void)
{
	//! wenn ReciprokeLatticeBitmap oder MatrixBitmap dann generiere
	//! gleich auch neue BSource - Typ TBitmapSource (m_data.h)
	if ( eOutputType==otReciprokeLatticeBitmap || eOutputType==otMatrixBitmap )
		BSource->New();

	bMeasurementActive= FALSE;
	return TFileWindow::New();
}
//*****************************************************************************

//! berechnet neue Groesse fuer Ausgabefenster, wenn mit Mouse geaendert
//! keine Groessenanpassung mehr vorgesehen...
/* Fehler 12
void TPlotWindow::Size(HWND hWnd,WPARAM,LPARAM)
//! berechne Groee des Screens
{
//! the GetClientRect function retrieves the coordinates
//! of a window's client area
//! hWnd - handle of window; &WndRect - address of Rect structure for
//! client coordinates
 GetClientRect(hWnd,&WndRect);
//! nicht hBitmap (Bitmap-windows.h) und
//! ReciprokeLatticeBitmap oder MatrixBitmap
 if(!::hBitmap && (eOutputType <= MatrixBitmap))
 {
//! berechne Groee des Screens und setze Struct-Variablen
//! woher kommen ominoese Multiplikatoren fuer dx und dy ???
 Screen.dx= (WndRect.right - WndRect.left) - xBorder;
 Screen.x0= 0;
 Screen.x1= Screen.dx;
 Screen.dx*= 0.8;
 Screen.dy= (WndRect.bottom - WndRect.top) - yBorder;
 Screen.y0= 0;
 Screen.y1= Screen.dy;
 Screen.dy*= 0.9;
 }
};
//******************************************************************************/

//! setze neue Koordinatensystembereiche
void TPlotWindow::SetRanges()
{
	//! nur RecipokeLatticeBitmap und MatrixBitmap haben x,y,z Koordinatensystem
	if ( eOutputType==otReciprokeLatticeBitmap || eOutputType==otMatrixBitmap )
	{
		CoorSys.xMin= 0.01;
		CoorSys.xMax= 100.0;
		CoorSys.yMin= 0.01;
		CoorSys.yMax= 100.0;
		CoorSys.zMax= UpperBound;
		CoorSys.zMin= LowerBound;
	} else {
		/*Kullmann, Reinecker: ATOS-Testfall MS.2 Fehlerbehebung
		CoorSys.xMin= 0.01;
		CoorSys.xMax= 100.0;*/
		CoorSys.yMax= UpperBound;
		CoorSys.yMin= LowerBound;
	}
};
//*****************************************************************************

//! behandelt Kommando: linke Maustaste wurde gedrueckt
//! gibt x und y Werte in der untersten Statuszeile aus, wenn linke
//! Maustaste kurzzeitig gedrueckt, in einem Koord.system
void TPlotWindow::OnLButtonDown(WPARAM, LPARAM lParam)
{
	char buf[MaxString];
	int nk1, nk2, nk3;
	TCoorSystem *pCoorSys;

	LPCurve Scan;
	LPCurve ScanReport;
	float x, y, z, offset, temp_xo, ODelta, TDelta;
	double Omega, Theta;
	int row, column;

	bButtonDown= TRUE;
	//! the GetDC function retrieves a handle of a display device context (DC)
	//! for the client area of the specified window
	//! DragDC - Typ HDC (m_data.h) - Handle auf ein DC (dc.h)
	DragDC= GetDC( GetHandle() );
	//! setzt den Mausfang auf das aktuelle Fenster
	SetCapture(GetHandle());
	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			pCoorSys= &AltCoorSys;
			break;

		default:
			pCoorSys= &CoorSys;
	}

	/*//! switch Anweisung folgt dem, wohin, mit der Maus geklickt wurde
	int xi, yi;
	//! bzw welche Optionen an waren...
	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			xi= LOWORD(lParam) - xBorder;
			break;

		default:
			xi= LOWORD(lParam) - xBorder;
	}
	xo= ((float)xi * pCoorSys->xSF) + pCoorSys->xMin;


	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			yi= Screen.y1 - HIWORD(lParam) - yBorder;
			break;

		default:
			yi= Screen.y1 - HIWORD(lParam);
	}
	switch (pCoorSys->yScal)
	{
		case sLogarithmic:
			y= yi * pCoorSys->ySF;
			if (y > 7.0) 
				y= yi * log10(pCoorSys->yMax / pCoorSys->yMin) / Screen.dx;
			yo= pow((float)10, y) * pCoorSys->yMin;
			break;

		case sLinear:
			yo= (yi * pCoorSys->ySF) + pCoorSys->yMin;
			break;
	}*/
	float valueZ= 0;
	Screen2Title(LOWORD(lParam) - xBorder, (eOutputType==otCurve) ? (Screen.y1 - HIWORD(lParam)) : (Screen.y1 - HIWORD(lParam) - yBorder), 
	             xo, yo, valueZ, false);

	ScanReport= lpDBase->GetScanReport();


	//! ThetaOffset muss in Koordinatenausgabe beruecksichtigt werden
	if ( eOutputType==otMatrixBitmap )
	{
		row= ScanReport->GetValueByValue(yo, x, y);
		offset= GetThetaOffset(row) - VisualDB.dThetaMinFirst;
		temp_xo= xo;

		if (VisualDB.bPsd)
		{
			xo= xo - pCoorSys->xMin;
			xo= xo - offset;
		}
	}
	else
		temp_xo= xo;

	mPoint1.x= temp_xo;
	mPoint1.y= yo;

	//! zeige X,Y,Intensitaet,Row,Column in Statuszeile
	//! - wenn in Koordinatensystem geklickt
	float xMin= pCoorSys->xMin;
	float xMax= pCoorSys->xMax;
	if ( eOutputType!=otMatrixBitmap && !bSecondaryCoor ) { //! im Falle von Winkelbeschriftung		
		xMin= CalcValueInUnit( m_DetectorUnit, CalcValueFromUnit(m_MotorUnit, xMin) ); 
		xMax= CalcValueInUnit( m_DetectorUnit, CalcValueFromUnit(m_MotorUnit, xMax) ); 
	}
	if ( yo>=pCoorSys->yMin && yo<=pCoorSys->yMax && 
		 temp_xo>=xMin && temp_xo<=xMax )
	{
		switch (eOutputType)
		{
			case otReciprokeLatticeBitmap: //! Zeile und Spalte im Bitmap berechnen				
				row= (yo - pCoorSys->yMin) / pCoorSys->ySF;
				column= (xo - pCoorSys->xMin) / pCoorSys->xSF;
				//! Omega und Theta aus dem "Daten"Feld holen
				if ( BSource->IsDatenInited() )
				{
					Omega= BSource->GetOmega(row, column);
					Theta= BSource->GetTheta(row, column);
				} else {
					Omega= 0;
					Theta= 0;
				}

				//! nur die Vorinitialisierung des "Daten"feldes abfangen
				if ( (Omega && Theta) != row*row - column*column )
				{
					//! Zeile und Spalte in der Areascan-Datenbasis holen und Pointer darauf setzen
					row= ScanReport->GetValueByValue(Omega, x, y);
					Scan= lpDBase->GetCurve(row);
					//** pick out the intensity
					column= Scan->GetValueByValue(Theta, x, y);
					Scan->SetPP(column);
					//! Intensitaet holen
					Scan->PGet(x, y, z); //07.05.2004 FastP...
					//! zur Ausgabe in der Statuszeile Thetaoffset aufaddieren
					//! diese Unterscheidung, da 0-dim. andere Datenbasisspeicherung als
					//! 1-dim Areascans (ThetaOffset mitgespeichert/nichtgespeichert)
					if (VisualDB.bPsd)
						Theta= Theta + GetThetaOffset(row);
				} else Omega= Theta= row= column= 0;
				break;

			case otMatrixBitmap:
				Omega= yo;
				Theta= xo;
				//! Zeile und Spalte in der Areascan-Datenbasis holen und Pointer darauf setzen
				row= ScanReport->GetValueByValue(Omega, ODelta, y);
				Scan= lpDBase->GetCurve(row);
				//** pick out the intensity
				column= Scan->GetValueByValue(Theta, TDelta, y);
				Scan->SetPP(column);
				//! Intensitaet holen
				Scan->PGet(x, y, z); //07.05.2004 FastP...
				xo= temp_xo;
				break;
		}
		//! hier Unterscheidung nach benoetigter Genauigkeit eingebaut - Fehler 6
		nk1= GetNachkommaStelle(pCoorSys, 1, 0);
		nk2= GetNachkommaStelle(pCoorSys, 0, 1);
		if (y >= 1000)
			nk3= 0;
		else if (y >= 100)
			nk3= 1;
		else if (y >= 10)
			nk3= 2;
		else
			nk3= 3;

		//! Standardausgabe 
		sprintf(buf, "X: %.*f Y: %.*f", nk1, temp_xo, nk2, yo);
		if (row && column) switch (eOutputType)
		{
			case otReciprokeLatticeBitmap:
				sprintf(buf, szMsgLine609,
						nk1, xo, nk2, yo, Theta, Omega, nk3, y, row, column);
				break;

			case otMatrixBitmap:
				sprintf(buf, szMsgLine608,
						nk1, xo, TDelta, nk2, yo, ODelta, nk3, y, row, column);
				break;
		}
		//! buf in Statuszeile anzeigen
		SetInfo(buf);
	} else SetInfo(szMsgLine601);
};
//*****************************************************************************

void TPlotWindow::OnMouseMove(WPARAM, LPARAM lParam)
//! diese Fkt. gibt in der Statuszeile dx und dy Werte aus
//! wenn mit gedrueckter linker Maustaste ueber ein Koord.system gefahren wird
{
	char buf[MaxString];
	int xi, yi, temp1, temp2;
	float x, y;
	TCoorSystem *pCoorSys;

	//! wenn Button nicht gedrueckt gehe zurueck...
	if ( !bButtonDown )
		return;

	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			xi= LOWORD(lParam) - xBorder;
			yi= Screen.y1 - HIWORD(lParam) - yBorder;
			pCoorSys= &AltCoorSys;
			break;

		default:
			xi= LOWORD(lParam) - xBorder;
			yi= Screen.y1 - HIWORD(lParam);
			pCoorSys= &CoorSys;
	}
	//! Abzisse immer linear skaliert
	x= (xi * pCoorSys->xSF) + pCoorSys->xMin;
	//! Ordinate linear oder log skaliert
	switch (pCoorSys->yScal)
	{
		case sLogarithmic:
			y= yi * pCoorSys->ySF;
			if (y > 7.0)
				y= yi * log10(pCoorSys->yMax / pCoorSys->yMin) / Screen.dx;
			y= pow((float)10, y) * pCoorSys->yMin;
			break;

		case sLinear:
			y= (yi * pCoorSys->ySF) + pCoorSys->yMin;
			break;

		default:
			y=0;
	}
	//! zeige dX,dY in Statuszeile - wenn in Koordinatensystem geklickt
	if ( mPoint1.y <= pCoorSys->yMax && mPoint1.y >= pCoorSys->yMin && 
		 mPoint1.x <= pCoorSys->xMax && mPoint1.x >= pCoorSys->xMin && 
		 y <= pCoorSys->yMax && y >= pCoorSys->yMin && 
		 x <= pCoorSys->xMax && x >= pCoorSys->xMin)
	{
		//! hier Unterscheidung nach benoetigter Genauigkeit eingebaut - Fehler 6
		temp1= GetNachkommaStelle(pCoorSys, 1, 0);
		temp2= GetNachkommaStelle(pCoorSys, 0, 1);		
		if ( DataaquisitionActive ) { //! wenn DataAquisition aktiv
			mPoint2.x= x;
			mPoint2.y= y;
			sprintf(buf, szMsgLine607, 
				    temp1, mPoint1.x, temp2, mPoint1.y, 
					temp1, mPoint2.x, temp2, mPoint2.y);
		} else sprintf(buf, "dX= %.*f dY= %.*f", temp1, x - xo, temp2, y - yo);
		//! in Statuszeile ausgeben
		SetInfo(buf);
	}
	else
		SetInfo(szMsgLine601);
};
//*****************************************************************************

void TPlotWindow::OnLButtonUp(WPARAM, LPARAM)
//! nur dazu da, um das Loslassen der linken Maustaste zu registrieren
{
	bButtonDown= FALSE;
	if ( DragDC ) {
		ReleaseDC(GetHandle(), DragDC);
		ReleaseCapture();
		DragDC= 0;
	}
};
//*****************************************************************************

//! fuelle TKSystem mit Standardwerten
BOOL TPlotWindow::SetMeasurementArea(TKSystem &ks)
{
	ks[0].x= 1.0;
	ks[0].y= 1.0;
	ks[0].Ord= 0;

	ks[1].x= 10.0;
	ks[1].y= 1.0;
	ks[1].Ord= 1;

	ks[2].x= 10.0;
	ks[2].y= 10.0;
	ks[2].Ord= 2;

	ks[3].x= 1.0;
	ks[3].y= 10.0;
	ks[3].Ord= 3;
	return TRUE;
};
//*****************************************************************************

void TPlotWindow::OnRenderFormat(WPARAM wParam, LPARAM)
//! the WM_RENDERFORMAT message is sent to the clipboard owner
//! when a particular format with delayed rendering needs to be rendered
//! HANDLE SetClipboardData(uint format, HANDLE handle);
//! bei erfolgreicher Ausfuehrung ist der Rueckgabewert ein Handle auf die Daten
{
	HANDLE hClip;

	if (eOutputType==otCurve)
		return;
	switch (wParam)
	{
		case CF_BITMAP:
			hClip= (HANDLE)BSource->CreateGDIObject(GetDC(GetHandle()));
			SetClipboardData(CF_BITMAP, hClip);
			break;

		case CF_PALETTE:
			hClip= (HANDLE)CreatePalette(BSource->GetPalette());
			SetClipboardData(CF_PALETTE, hClip);
			break;

		case CF_DIB:
			hClip= BSource->RenderDIB();
			SetClipboardData(CF_DIB, hClip);
			break;
	}
};
//*****************************************************************************

void TPlotWindow::OnRenderAllFormats(WPARAM, LPARAM)
//! the WM_RENDERALLFORMATS message is sent to the clipboard owner
//! when the owner application is being destroyed
{
	UINT uFormat= 0;

	OpenClipboard(GetHandle());
	// hier stand ein =, was soll das?
	while (NULL != (uFormat= EnumClipboardFormats(uFormat)))
		SendMessage(GetHandle(), WM_RENDERFORMAT, uFormat, 0L);
	CloseClipboard();
};
//*****************************************************************************

void TPlotWindow::DoCopy(void)
//! Kopieren mit Hilfe der Zwischeablage - Typen: CF_DIB, CF_PALETTE

{
	//! OpenClipboard(hWnd) - oeffnet die Zwischenablage und hindert
	//! andere Anwendungen daran, den Inhalt der Zwischenablage zu veraendern
	//! (Ausgabetyp ist nicht Matrix- oder RL Bitmap) oder Zwischenablage
	//! geht nicht auf ---> denn zurueck...
	if ( (eOutputType==otCurve) || !OpenClipboard(GetHandle()) )
		return;

	//! EmptyClipBoard() - Leert die Zwischenablage
	//! und gibt alle Handles auf die Daten der Zwischenablage frei
	EmptyClipboard();

	//! SetClipBoardData(...) - setzt ein Handle auf den Datenblock
	//! an der durch handle angezeigten Position
	SetClipboardData(CF_DIB, NULL);
	SetClipboardData(CF_PALETTE, NULL);

	//! schliet die Zwischenablage
	CloseClipboard();
};
//*****************************************************************************

void TPlotWindow::Screen2Title(float aScreenX, float aScreenY, float &resultX, float &resultY, float &resultZ, bool aChannel5Steps) {
	TCoorSystem *pCoorSys;
	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			pCoorSys= &AltCoorSys;
			break;

		default:
			pCoorSys= &CoorSys;
	}

	// x-Achse
	if ( eOutputType==otMatrixBitmap && bSecondaryCoor ) { // MatrixBitmap und Winkeldarstellung
		float xMin= MainCurve->GetMin( P_Z );
		float xMax= (MainCurve->GetMax( P_Z ) * pCoorSys->xMax) / (MainCurve->GetMax( P_X ) + GetThetaOffset(0));
		float xSF= (xMax - xMin) / Screen.dx;
		resultX= xMin + aScreenX*xSF;		
	} else {
		resultX= pCoorSys->xMin + aScreenX * pCoorSys->xSF;
		if ( bSecondaryCoor ) { //! im Falle von Kanalbeschriftung			
			if ( aChannel5Steps ) { // sonst treten an x-Achse zwei identische Beschriftungen auf
				// in 5er Schritten runden
				long chann= long(resultX);
				if ( chann % 5 == 0 ) /*perfekt*/;
				else if ( (chann-1) % 5 == 0 ) chann-= 1; // na gut, Verschiebung um -1 
				else if ( (chann+1) % 5 == 0 ) chann+= 1; // na gut, Verschiebung um +1 
				else if ( (chann-2) % 5 == 0 ) chann-= 2; // na gut, Verschiebung um -2
				else if ( (chann+2) % 5 == 0 ) chann+= 2; // na gut, Verschiebung um +2 
				resultX= chann;
			} 
		} else if ( m_DetectorUnit!=utNone && m_MotorUnit!=utNone ) resultX= CalcValueInUnit( m_DetectorUnit, CalcValueFromUnit(m_MotorUnit, resultX) ); //! im Falle von Winkelbeschriftung
	}

	// y-Achse
	switch (pCoorSys->yScal)
	{
		case sLinear:
			resultY= pCoorSys->yMin + aScreenY*pCoorSys->ySF;
			break;

		default:
			resultY= pCoorSys->yMin * pow((double)10, (double)(aScreenY)*pCoorSys->ySF);
	}

	// x-Achse (zweizeilige Beschriftung; nur DTN-File)
	if (eLoadFormat == ffDtnFile) resultZ= pCoorSys->zMin + aScreenX*pCoorSys->zSF;
	else resultZ= 0;
}

void TPlotWindow::GetTitleX(int i, int titlesx, char *buf, bool reduceLines) {
	TCoorSystem *pCoorSys;
	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			pCoorSys= &AltCoorSys;
			break;

		default:
			pCoorSys= &CoorSys;
	}

	// an der Achse darzustellende(r) Zahlenwert(e)
	float valueX, valueY, valueZ;
	float x= float(i) / float(tilesx)*Screen.dx;
	Screen2Title(x, 0/*y-Achse nicht berechnen*/, 
		         valueX, valueY, valueZ, fabs(pCoorSys->xMax-pCoorSys->xMin) >= tilesx*5);
	if ( eOutputType==otMatrixBitmap && bSecondaryCoor ) Double2String(buf, valueX, 0, TRUE, _DECIMAL, _THOUSAND); // MatrixBitmap und Winkeldarstellung
	else if ( bSecondaryCoor ) Double2String(buf, valueX, 0, TRUE, _DECIMAL, _THOUSAND); //! im Falle von Kanalbeschriftung			
	else Double2String(buf, valueX, GetNachkommaStelle(pCoorSys, 1, 0), TRUE, _DECIMAL, _THOUSAND); // sprintf(buf, "%.*f", GetNachkommaStelle(pCoorSys, 1, 0), fval); //! im Falle von Winkelbeschriftung		

	// dtn-Abszissenbeschr. (zweizeilige Zahlenwerte)
	if (eLoadFormat == ffDtnFile) {
		char buf2[MaxString];
		Double2String(buf2, valueZ, GetNachkommaStelle(pCoorSys, 1, 0), TRUE, _DECIMAL, _THOUSAND); // sprintf(buf, "%.*f", GetNachkommaStelle(pCoorSys, 1, 0), fval);
		strcat(buf, "\n");
		strcat(buf, buf2);
	}			
}

void TPlotWindow::GetTitleY(int i, int titlesy, char *buf, bool reduceLines) {
	TCoorSystem *pCoorSys;
	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			pCoorSys= &AltCoorSys;
			break;

		default:
			pCoorSys= &CoorSys;
	}

	// an der Achse darzustellende(r) Zahlenwert(e)
	float valueX= 0, valueY= 0, valueZ= 0;
	float y= float(i) / float(titlesy)*Screen.dy;
	Screen2Title(0/*x-Achse nicht berechnen*/, y, 
	             valueX, valueY, valueZ, false);
	if ( valueY==0 )
		Double2String(buf, 0, 2, TRUE, _DECIMAL, _THOUSAND); // sprintf(buf, "0.00");
	else
		Double2String(buf, valueY, GetNachkommaStelle(pCoorSys, 0, 1), TRUE, _DECIMAL, _THOUSAND); // sprintf(buf, "%.*f", GetNachkommaStelle(pCoorSys, 0, 1), fval);

	// letzte Beschriftung = Beschriftung der Ordinaten-Achse mit Skalierungstyp
	if ( i==tilesy ) 
		if ( eOutputType==otCurve ) {
			if ( reduceLines ) {
				if ( pCoorSys->yScal==sLinear ) sprintf(buf, szIntensLiEZ);
				else if ( pCoorSys->yScal==sLogarithmic ) sprintf(buf, szIntensLoEZ);
			} else {
				if ( pCoorSys->yScal==sLinear ) sprintf(buf, szIntensLi);
				else if ( pCoorSys->yScal==sLogarithmic ) sprintf(buf, szIntensLo);
			}
		}
}

BOOL TPlotWindow::GetTextExtentPointEx(HDC hdc, LPCTSTR lpString, int cbString, LPSIZE lpSize)
{
	BOOL result= TRUE; // falls Leerstring
	lpSize->cx= 0;
	lpSize->cy= 0;
	SIZE tagS;
	GetTextExtentPoint(hdc, "Qq", strlen("Qq"), &tagS);

	char *buf= new char[ strlen(lpString)+1 ];
	strcpy(buf, lpString);

	char *actual= strtok( buf, "\n\r" ); // erste "Zeile" auslesen
	while ( actual ) { // ausgeben der "Zeilen"
		SIZE tagT;
		if ( !GetTextExtentPoint(hdc, actual, strlen(actual), &tagT) ) result= FALSE;
		lpSize->cx= max(lpSize->cx, tagT.cx);
		lpSize->cy+= tagS.cy;
		actual= strtok( NULL, "\n\r" ); // nchste "Zeile"
	}	

	_FREELIST(buf);
	return result;
};


void TPlotWindow::TextOutEx(HDC hdc, int nXStart, int nYStart, LPCTSTR lpString, int cbString, EHAlign hAlign, EVAlign vAlign)
{
	char *buf= new char[ strlen(lpString)+1 ];
	strcpy(buf, lpString);
	SIZE tagO;
	GetViewportExtEx(hdc, &tagO);
	bool bTopDown= tagO.cy>=0; // bei Sichtweise von oben nach unten: Koordinaten werden grer 
	bool bLeftRight= tagO.cx>=0; // bei Sichtweise von links nach rechts: Koordinaten werden grer 
	SIZE tagS;
	GetTextExtentPoint(hdc, "Qq", strlen("Qq"), &tagS);
	SIZE tagG;
	GetTextExtentPointEx(hdc, lpString, strlen(lpString), &tagG);

	int y= nYStart;
	if ( vAlign==vaCenter ) {
		if ( bTopDown ) y= nYStart - tagG.cy/2;
		else y= nYStart + tagG.cy/2;
	} else if ( vAlign==vaBottom ) {
		if ( bTopDown ) y= nYStart - tagG.cy;
		else y= nYStart + tagG.cy;
	}
	char *actual= strtok( buf, "\n\r" ); // erste "Zeile" auslesen
	while ( actual ) { // ausgeben der "Zeilen"
		SIZE tagT;
		GetTextExtentPoint(hdc, actual, strlen(actual), &tagT);
		int x= nXStart;
		if ( hAlign==haCenter ) {
			if ( bLeftRight ) x= nXStart - tagT.cx/2;
			else x= nXStart + tagT.cx/2;
		} else if ( hAlign==haRight ) {
			if ( bLeftRight ) x= nXStart - tagT.cx;
			else x= nXStart + tagT.cx;
		}
		TextOut(hdc, x, y, actual, strlen(actual));

		if ( bTopDown ) y+= tagS.cy;
		else y-= tagS.cy;
		actual= strtok( NULL, "\n\r" ); // nchste "Zeile"
	}	

	_FREELIST(buf);
}

void TPlotWindow::DrawCoordinateSystem(HDC hdc)
//! zeichnet Koord.System - oder was soll der Name sonst :-)
{
	//! der Datentyp, der das Handle auf eine physikalische Schrift repraesentiert
	HFONT hOldFont;
	//! der Windows-Datentyp, der Handles fuer logische Stifte repraesentiert
	HPEN hOldPen, hPenDot, hPenBlack;
	//! the SIZE structure specifies the width and height of a rectangle
	SIZE tagS;
	//! the POINT structure defines the x- and y- coordinates of a point
	POINT egde[5];
	char buf[MaxString];
	float dist, offset;
	int i;

	CoorSys.xSF= (CoorSys.xMax - CoorSys.xMin) / Screen.dx; 
	CoorSys.ySF= (CoorSys.yMax - CoorSys.yMin) / Screen.dy; 

	// inquire scaling parameters
	CoorSys.xSF= (CoorSys.xMax - CoorSys.xMin) / Screen.dx;
	if ( eOutputType!=otCurve || CoorSys.yScal==sLinear ) CoorSys.ySF= (CoorSys.yMax - CoorSys.yMin) / Screen.dy; // Bitmaps sind nur linear skaliert
	else CoorSys.ySF= log10(CoorSys.yMax / CoorSys.yMin) / Screen.dy;

	hPenBlack= CreatePen(PS_SOLID, 2, RGB(0, 0, 0));
	hPenDot= CreatePen(PS_DOT, 1, RGB(0, 0, 0));
	hOldFont= (HFONT)SelectObject(hdc, hFont);
	hOldPen= (HPEN)SelectObject(hdc, hPenBlack);

	// wie viele Zeilen bentigt die hchste x-Achsenbeschriftung; ausfhrliche Beschreibung zulassen?
	GetTextExtentPoint(hdc, "Qq", strlen("Qq"), &tagS);			
	int rowHeight= tagS.cy;
	int rows= 1; // Beschriftung muss in so vielen Zeilen angezeigt werden, damit es keine berschneidungen gibt
	int heightY= 0; // so hoch sind die Beschriftungen insgesamt fr das aktuelle <rows>
	for (int reduceLinesX= 0; reduceLinesX<2; reduceLinesX++) {
		int rowStep= 1; // wie viele Zeilen belegt die hchste Bezeichnung
		for (i= 0; i<=tilesx; i++) {
			GetTitleX(i, tilesx, buf, reduceLinesX!=0);
			GetTextExtentPointEx(hdc, buf, strlen(buf), &tagS);			
			rowStep= max(rowStep, min((yBorder-2)/rowHeight, tagS.cy/rowHeight) ); // die Beschriftung darf nicht mehr als den verfgbaren Platz ver(sch)wenden
		}

		// In wie vielen Ebenen muss die X-Achse beschriftet werden, damit sich die Beschriftungen nicht berschneiden?
		rows= 1; 
		bool ueberschneid, lastLoop= false;
		do {
			ueberschneid= false;
			heightY= 0;
			int platz= float(Screen.dx) / float(tilesx) * float(rows); // max. Pixelanzahl zwischen zwei Beschriftungen auf gleicher Ebene (wird diese Anzahl berschritten, kommt es zu berschneidungen)
			for (int row=0; row<rows; row++) { // jede Zeile auf berschneidungen prfen
				int dy= 0;
				int lastWid= 0;
				for (i= 0; i<=tilesx; i++) {
					if ( (i%rows)!=row ) continue;
					
					// diese Beschriftung kommt in Zeile <row>
					GetTitleX(i, tilesx, buf, reduceLinesX!=0);
					GetTextExtentPointEx(hdc, buf, strlen(buf), &tagS);			
					int thisWid= tagS.cx;
					if ( i!=tilesx ) thisWid/= 2; // bis auf den letzten Abschnitt (rechtsbndig) ist alles zentriert; belegt nur die halbe Breite
					if ( lastWid+thisWid > platz-max(5, float(platz)*0.02) ) ueberschneid= true;
					lastWid= thisWid;
					dy= max(dy, tagS.cy);
				}
				heightY+= dy;
			}
			if ( lastLoop ) break;
			if ( ueberschneid ) {
				rows+= 1;
				if ( heightY > yBorder-max(5, float(yBorder)*0.02) ) { // reicht der Platz zwischen x-Achse und sichtbarem Bereich?
					rows= max(1, rows-1);
					lastLoop= true; // Schleife mit neuem <rows> wiederholen und <heightX> berechnen; dann soft beenden
				}
			}
		} while ( ueberschneid );
		if ( !ueberschneid ) break; // reduceLinesX muss nicht eingeschaltet werden
	}

	// ermitteln, ob y-Achsenbeschriftung ausfhrlich oder einzeilig
	bool reduceLinesY= false;
	int platz= float(Screen.dy-Screen.y0)/tilesy;
	int lastHei= 0;
	for (i= 0;i <= tilesy; i++) {
		GetTitleY(i, tilesy, buf, false);
		GetTextExtentPointEx(hdc, buf, strlen(buf), &tagS);
		int thisHei= tagS.cy;
		if ( i!=tilesy ) thisHei/= 2; // bis auf letzten Abschnitt (oben) alles mittig ausgerichtet
		if ( lastHei+thisHei > platz-max(5, platz*0.02) ) { // ausfhrliche Beschreibung ist zu hoch
			reduceLinesY= true; 
			break;
		}
		lastHei= thisHei;
	}

	// Rahmen zeichnen
	egde[0].x= egde[3].x= Screen.x0;
	egde[1].x= egde[2].x= Screen.x0 + Screen.dx;
	egde[0].y= egde[1].y= Screen.y0;
	egde[2].y= egde[3].y= Screen.y0 + Screen.dy;
	egde[4]= egde[0];
	Polyline(hdc, egde, 5);

//************ Absizze (x) *************************************************
	SelectObject(hdc, hPenBlack);
	SelectObject(hdc, hPenDot);

	TCoorSystem *pCoorSys;
	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap:
			pCoorSys= &AltCoorSys;
			break;

		default:
			pCoorSys= &CoorSys;
	}

	//! dient zur Unterscheidung, um ThetaOffset in RawMatrix bei
	//! Winkeldarstellung zu beruecksichtigen
	if ( eOutputType==otMatrixBitmap && bSecondaryCoor )
	{
		offset= VisualDB.dThetaMinLast - VisualDB.dThetaMinFirst; //GetThetaOffset(lpDBase->GetCNumber()-1)-GetThetaOffset(0);
		offset= offset / pCoorSys->xSF;
		i= 0;
	} else {
		offset= 0;
		i= 1;
	}

	for (i;i < tilesx;i++)
	{
		//! zeichnet Verbindungslinien ein...
		egde[0].x= Screen.x0 + i / tilesx * (Screen.dx);
		egde[1].x= Screen.x0 + offset + i / tilesx * (Screen.dx); 
		egde[0].y= Screen.y0;
		egde[1].y= Screen.y0 + Screen.dy;

		//! falls Hilfslinien ausserhalb des Screens fallen, berechne Anstieg der Geraden
		//! und zugehoerigen y-Wert
		if (egde[1].x > Screen.dx + Screen.x0)
		{
			egde[1].x= Screen.dx + Screen.x0;
			egde[1].y= (Screen.dx + Screen.x0 - egde[0].x) * ((Screen.dy - Screen.y0) / offset);
		}
		Polyline(hdc, egde, 2);
	}

	//! eingefuegt fuer dtn-File Abzissenbeschriftung
	if (eLoadFormat == ffDtnFile) 
		pCoorSys->zSF= (pCoorSys->zMax - pCoorSys->zMin) / Screen.dx; 

	//! zeichnet die Markierungen an der x-Achse
	for (i= 0; i<=tilesx; i++)
	{
		// Verbindungslinien und Markierungen 
		dist= i / tilesx * Screen.dx;
		egde[0].x= egde[1].x= Screen.x0 + dist + offset;
		egde[0].y= Screen.y0 + Screen.dy;
		egde[1].y= Screen.y0 + Screen.dy - 5;
		if (egde[1].x < Screen.dx + Screen.x0)
			Polyline(hdc, egde, 2);

		egde[0].x= egde[1].x= Screen.x0 + dist;
		egde[0].y= Screen.y0 + 5;
		egde[1].y= Screen.y0;
		Polyline(hdc, egde, 2);
	}

	// Beschriftungen
	int y= -yBorder/2 + heightY/2;
	for (int r= 0; r<rows; r++) 
	{
		int dy= 0;
		for (i= 0; i<=tilesx; i++) if ( i%rows==r ) 
		{
			GetTitleX(i, tilesx, buf, reduceLinesX!=0);
			GetTextExtentPointEx(hdc, buf, strlen(buf), &tagS);
			TextOutEx(hdc, Screen.x0 + float(i) / tilesx * Screen.dx, y - tagS.cy/2, buf, strlen(buf), (i==tilesx) ? haRight : haCenter, vaCenter);
			dy= max(dy, tagS.cy);
		}
		y-= dy;
	}

//************ Ordinate (y) **************************************************
	SelectObject(hdc, hPenDot);
	for (i= 1;i < tilesy;i++)
	{
		//! zeichnet Verbindungslinien ein...
		egde[0].y= egde[1].y= 1 + i / tilesy * Screen.dy + Screen.y0;
		egde[0].x= Screen.x0;
		egde[1].x= Screen.x0 + Screen.dx;
		Polyline(hdc, egde, 2);
	}

	SelectObject(hdc, hPenBlack);
	for (i= 0;i <= tilesy;i++)
	{
		// Markierungen 
		dist= i / tilesy * Screen.dy;
		egde[0].y= egde[1].y= Screen.x0 + dist;
		egde[0].x= Screen.x0 + Screen.dx;
		egde[1].x= Screen.x0 + Screen.dx - 5;
		Polyline(hdc, egde, 2);

		egde[0].x= Screen.x0 + 5;
		egde[1].x= Screen.x0;
		Polyline(hdc, egde, 2);

		// Beschriftung
		GetTitleY(i, tilesy, buf, reduceLinesY);
		SetTextAlign(hdc, TA_LEFT);
		TextOutEx(hdc, -0.5*xBorder, egde[0].y, (LPSTR)buf, strlen(buf), haCenter, (i==tilesy) ? vaTop : vaCenter);
	}

	//****************************************************************************
	//! loesche hPenBlack,hPenDot und krame hOldPen, hOldFont wieder hervor
	SelectObject(hdc, hOldPen);
	SelectObject(hdc, hOldFont);
	DeleteObject(hPenBlack);
	DeleteObject(hPenDot);
};
//*****************************************************************************

int TPlotWindow::GetNachkommaStelle(TCoorSystem* pCoorSys, int x, int y)
//! neue Funktion, die die benoetigte Nachkommastellengenauigkeit
//! berechnet und die noetigen Nachkommastellen als int zurueckliefert
//! wenn x=1 dann fuer x-Achse / wenn y=1 dann fuer y-Achse
//! gehoert zu Fehler 6
{
	float fval;

	if (y)
		fval= fabs(pCoorSys->yMax - pCoorSys->yMin);
	if (x)
		fval= fabs(pCoorSys->xMax - pCoorSys->xMin);

	if (fval < 0.25)
		return 5;
	else
		if (fval < 1)
			return 4;
		else
			if (fval < 3)
				return 3;
			else
				return 2;
};
//*****************************************************************************

LRESULT TPlotWindow::OnEvent ( HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	if ( message==WM_ERASEBKGND ) return 01; // Durch die Verwendung des hMemDC und der hBitmap in DoPaint braucht der Hintergrund nicht mehr gelscht werden, sonst flackert es.
	else return TFileWindow::OnEvent(hwnd, message, wParam, lParam);
}
//*****************************************************************************

LRESULT TPlotWindow::OnCommand(WPARAM wParam, LPARAM lParam)
{
	TModalDlg *dlg;
	WPARAM Cmd;
	Cmd= GET_WM_COMMAND_ID(wParam, lParam);
	switch ( Cmd ) {
		case cm_Open:   
			LoadOldData();
			SetRanges();
			return 0l;

		case cm_SaveSecondCurve:
			SaveSecondCurve();
			return 0l;

		case cm_KillSecondCurve:
			KillSecondCurve();
			return 0l;

		case cm_FreezeCurve:
			FreezeCurve();
			return 0l;

		case cm_ShowCurveParam:
			dlg= (TCurveShowParamDlg *)new TCurveShowParamDlg(this);
			if ( dlg ) dlg->ExecuteDialog(GetHandle());
			_FREEOBJ(dlg);
			UpdateWnd();
			return 0l;

		case cm_MoveScanReady:
			TCurve *crv= new TCurve();
			//07.05.2004 crv->FastOpen();
			float x, y, z;
			LPLONG data;
			data= mGetMoveScan();
			long yo;
			yo= data[0];
			int cnt;
			for (cnt= 0;cnt < mGetScanSize();cnt++)
			{
				x= z= cnt;
				y= (data[cnt] - yo);
				crv->PAdd(x, y, z); //07.05.2004 FastP...
			}
			//07.05.2004 crv->FastClose();
			TKSProperties ksp;
			ksp.bLogarithmicY= FALSE;
			ksp.fMinX= 0.0f;
			ksp.fMaxX= (float)mGetScanSize() * 1.1;
			ksp.fMinY= 0.01f;
			ksp.fMaxY= (float)(data[cnt - 1] - yo) * 1.3;
			SetKSProperties(&ksp);
			PickUpData((LPARAM)(TCurve*)crv);
			// SendMessage(Main.hScanWindow,WM_COMMAND,cm_PlotCurve,(LPARAM)(TCurve*)crv);
			char Msg[MaxString];
			sprintf(Msg, "MoveFinish: %ld >>> Letzter Wert: %ld", data[mGetMoveFinishIdx()] - yo, data[mGetScanSize() - 1] - yo);
			SetInfo(Msg);
			_FREEOBJ(crv);
			SendMessage(GetHandle(), WM_SETFOCUS, 0, 0l);
			return 0l;
	}
	return TFileWindow::OnCommand(wParam, lParam);
}
//*****************************************************************************

//! Paint, das von Basisklassen aufgerufen wird, wenn diese auf eine WM_PAINT-
//! Botschaft reagieren
void TPlotWindow::DoPaint(HDC hdc, PAINTSTRUCT *)
{
	BOOL PaintHole= TRUE;
	if ( bUserPaint ) {
		//! nur bei bEnablePaint zeichnen
		if (!bEnablePaint)
			return;
		
		//30.06.2004 bei Ausgabe auf MemDC geht nur noch PaintHole PaintHole= WholeWnd(); //! WholeWnd() gibt an, dass fr dieses Paint() nicht bPaintPoint gesetzt ist
		bUserPaint= false; // nchster Aufruf knnte WM_PAINT sein
	}

	// Bestimmen der Ausdehnung in Pixeln des Zeichen-Fensters
	RECT rect;
	GetClientRect(GetHandle(), (LPRECT)&rect);

	// Bitmap und DC im Speicher erzeugen, dann darauf zeichnen
	HBITMAP hBitmap= CreateCompatibleBitmap(hdc, rect.right-rect.left, rect.bottom-rect.top);
	HDC hMemDC= CreateCompatibleDC(hdc);	
	HGDIOBJ OldBitmap= SelectObject(hMemDC, hBitmap);

	// Hintergrund lschen (bevor neues Koordinatensystem gesetzt wird!)
	if ( PaintHole ) { 
		HBRUSH brush= CreateSolidBrush(RGB(255, 255, 255));
		FillRect(hMemDC, &rect, brush);
		DeleteObject(brush);
	}

	switch (eOutputType)
	{
		case otReciprokeLatticeBitmap:
		case otMatrixBitmap: {
			//! eigentliche Bitmap
			BSource->DrawBitmap(hMemDC, 0, -Screen.dy, Screen);			
			//! Legende (ColorTable)
			BSource->DrawBitmapFrame(hMemDC, Screen); 
			// Koordinatensystem (erst nach DrawBitmapFrame!)
			DrawCoordinateSystem(hMemDC); 

			// jetzt die Bitmap auf <hdc> zeichnen
			StretchBlt( hdc, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
						hMemDC, -xBorder, (rect.bottom-rect.top)-yBorder, rect.right-rect.left, -(rect.bottom-rect.top), SRCCOPY ); // BitBlt kann nicht verwendet werden, weil hMemDC eine andere Koordinatenskalierung besitzt
			break;
		}

		//! dies und viele folgende Sachen nur im Falle von Kurve zu tun...
		case otCurve:
			if (bAltCoorSystem)
			{
				CoorSys= AltCoorSys;
				bAltCoorSystem= FALSE;
			}
			else
				//! setze neue Koordinatensystembereiche
				SetRanges();

			// Daten-Bereich festlegen
			Screen.dx= (rect.right - rect.left) - xBorder; 
			Screen.x0= 0;
			Screen.x1= Screen.dx;
			Screen.dy= (rect.bottom - rect.top) - yBorder;
			Screen.y0= 0;
			Screen.y1= Screen.dy;

			//! Koordinatenursprung und -skalierung ndern, damit die stndigen Koordinatenumrechnungen entfallen		
			SIZE tagS;
			POINT tagP;
			SetMapMode(hMemDC, MM_ANISOTROPIC); // wir wollen die Ausdehnung der x- und y-Achse selbst bestimmen		
			SetWindowExtEx(hMemDC, Screen.dx + xBorder, Screen.dy + yBorder, &tagS); // setze neue x- und y-Ausdehnung		
			SetViewportExtEx(hMemDC, Screen.dx + xBorder, -(Screen.dy + yBorder), &tagS); // nur in diesem Bereich wollen wir zeichnen		
			SetWindowOrgEx(hMemDC, 0, 0, &tagP); // folgende Koordinaten als -ursprung des Fensters		
			SetViewportOrgEx(hMemDC, xBorder, Screen.dy, &tagP); // folgende Koordinaten als -ursprung des sichtbaren Bereiches

			if ( PaintHole ) // Koordinatensystem zeichnen (erst nachdem neues Koordinatensystem gesetzt und CoorSys ermittelt wurde!)
				DrawCoordinateSystem(hMemDC); // redraw coordinate system

			int cnt= 0;
			//! Windows-Datentyp, der Handles fuer logische Stifte repraesentiert
			HPEN hOldPen, hPen;
			//! 32 bit Zeiger
			//! the POINT structure defines the x- and y-coordinates of a point
			LPPOINT lpPoints;

			//! wenn zweite Kurve nicht leer: Kurve komplett zeichnen
			if ( SecondCurve->GetPNumber()>1 ) //! 1 < letzter Eintrag + 1
			{
				lpPoints= (LPPOINT)new POINT[ SecondCurve->GetPNumber() ];
				//! setzt ActualIdx auf pos oder auf MaxPointIdx, wenn pos>MaxPointidx
				//! hier ActualIdx=0
				SecondCurve->SetPP();
				//! fuellt die Struktur lpPoints mit den Werten fuer eine Kurve...
				FormatCurveToPLine(SecondCurve, lpPoints, cnt);
				//! ...um damit spaeter dann die Kurve darzustellen
				hPen= CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
				hOldPen= (HPEN)SelectObject(hMemDC, hPen);
				if ( m_HasFilter ) 
					FilterPLine(lpPoints, cnt);
				if ( m_Dotted ) {
					HBRUSH hBrush= CreateSolidBrush(RGB(0, 0, 255));
					HBRUSH hOldBrush= (HBRUSH)SelectObject(hMemDC, hBrush);
					Dottedline(hMemDC, lpPoints, cnt);
					SelectObject(hMemDC, hOldBrush);
					DeleteObject(hBrush);
				} else Polyline(hMemDC, lpPoints, cnt);
				SelectObject(hMemDC, hOldPen);
				DeleteObject(hPen);
				_FREELIST(lpPoints);
			}
			//! wenn erste Kurve nicht leer: diese [auch] zeichnen
			if ( MainCurve->GetPNumber()>1 ) //! 1 < letzter Eintrag + 1
			{
				if (PaintHole)
				{ // alle Punkte der Kurve zeichnen, dazu PP auf Index 0 setzen
					lpPoints= (LPPOINT)new POINT[ MainCurve->GetPNumber() ];
					//! letzter ActualIdx auf Anfang
					MainCurve->SetPP();
				}
				else
				{ // nur den letzten Punkt zeichnen, Index um eine Position zurcksetzen
					lpPoints= (LPPOINT)new POINT [2];
					//! setzt ActualIdx vor-letztes MaxPointIdx
					MainCurve->SetPPLast();
					MainCurve->BackStep();
				}
				//! fuellt die Struktur lpPoints mit den Werten fuer eine Kurve...
				FormatCurveToPLine(MainCurve, lpPoints, cnt);

				//! ...um damit spaeter dann die Kurve darzustellen
				hPen= CreatePen(PS_SOLID, 1, RGB(255, 0, 0));
				hOldPen= (HPEN)SelectObject(hMemDC, hPen);
				if ( m_HasFilter ) 
					FilterPLine(lpPoints, cnt);
				if ( m_Dotted ) {
					HBRUSH hBrush= CreateSolidBrush(RGB(255, 0, 0));
					HBRUSH hOldBrush= (HBRUSH)SelectObject(hMemDC, hBrush);
					Dottedline(hMemDC, lpPoints, cnt);
					SelectObject(hMemDC, hOldBrush);
					DeleteObject(hBrush);
				} else Polyline(hMemDC, lpPoints, cnt);
				SelectObject(hMemDC, hOldPen);
				DeleteObject(hPen);
				_FREELIST(lpPoints);
			}

			// jetzt die Bitmap auf <hdc> zeichnen und alles aufrumen
			StretchBlt( hdc, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
						hMemDC, -xBorder, Screen.dy, xBorder+Screen.dx, -(yBorder+Screen.dy), SRCCOPY ); // BitBlt kann nicht verwendet werden, weil hMemDC eine andere Koordinatenskalierung besitzt
			break;
	} // switch

	// alles aufrumen
	SelectObject(hMemDC, OldBitmap);
	DeleteObject(hBitmap);
	DeleteDC(hMemDC);
};
//*****************************************************************************

void TPlotWindow::Dottedline(HDC hdc, const POINT *lppt, int cPoints)
{
	for (int i= 0; i<cPoints; i++) {
		Ellipse(hdc, lppt[i].x-1, lppt[i].y-1, lppt[i].x+1, lppt[i].y+1);
	}
}
//*****************************************************************************

HICON TPlotWindow::GetIcon ( void ) const
{
	return (HICON)LoadImage( GetMainInstance(), MAKEINTRESOURCE(icon_ScanWindow),IMAGE_ICON,16,16,LR_DEFAULTCOLOR);
};
//*****************************************************************************

void TPlotWindow::SetLoadFormat(EFileFormat type)
{
	eLoadFormat= type;
};
//*****************************************************************************

BOOL TPlotWindow::SaveFile(EAskType ask) {
	char NewFilename[ _MAX_PATH ];
	char dir[_MAX_DIR], drv[_MAX_DRIVE], name[_MAX_FNAME], ext[_MAX_EXT];
	OFSTRUCT of;
	int nNumber= 0, hFile;
	HFILE ht;

	//! ht= _lopen("test",READ_WRITE);
	ht= _lopen("test", OF_READWRITE);
	_lclose(ht);
	_chdir(szDataPath);
	switch (ask)
	{
		case atNoAsk:
			// Test ob File beschrieben werden kann
			_splitpath(FileName, drv, dir, name, ext);
			_makepath(NewFilename, "", "", name, szExtension);
			hFile= OpenFile(NewFilename, &of, OF_CREATE);
			if (hFile==HFILE_ERROR)
				do {					
					sprintf(NewFilename, "%sscan%03d.%s", szDataPath, nNumber++, szExtension);
					hFile= OpenFile(NewFilename, &of, OF_EXIST);
				} while ( hFile!=HFILE_ERROR && nNumber<10000 );
			_lclose(hFile);
			break;

		case atNewName:
			sprintf(NewFilename, "*.%s", szExtension);
			if ( !FileSaveDialog("Sichern unter ...", szFilter, NewFilename, szDataPath) ) return FALSE; // 'Abbruch' geklickt
			if ( strlen(NewFilename)==0 || strcmp(NewFilename, "*")==0 ) 
				return FALSE;
			break;

		default:
			if (!bFileChanged)
			{
				SetInfo(strMessage14);
				return FALSE;
			}
			if (!bIsNewFile)
			{
				_splitpath(FileName, drv, dir, name, ext);
				_makepath(NewFilename, "", "", name, szExtension);
			}
			else
				sprintf(NewFilename, "*.%s", szExtension);
			if ( !FileSaveDialog("Sichern als ...", szFilter, NewFilename, szDataPath) ) return FALSE; // 'Abbruch' wurde geklickt
			if ( strlen(NewFilename)==0 || strcmp(NewFilename, "*")==0 ) 
				return FALSE;
	}
	_splitpath(NewFilename, drv, dir, name, ext);
	_makepath(FileName, drv, dir, name, szExtension);
	bIsNewFile= FALSE;
	//  bOldDataLoaded= FALSE;
	bFileChanged= FALSE;
	_chdir(szDataPath);
	SetTitle();
	return TRUE;
}
// ---------------------------------------------------------------------------
// 15.06.2004 Kullmann/Reinecker: neue Methode zur uebersichtlicheren Kurvendarstellung
void TPlotWindow::FilterPLine(LPPOINT lpPoints, int &idx)
{
	bool found = false;
	POINT FilterLevel;
	LPPOINT YAccumulation = (LPPOINT)new POINT[idx+1];
	
	m_FilterStrength = max(0.0, min(100.0, m_FilterStrength)); 

	for (int i = 0; i <= idx; i++) 
	{
		YAccumulation[i].x = -1; // die Hufigkeit des Auftretens
		YAccumulation[i].y = -1; // die gemessenen Intensitaet
	}
	FilterLevel.x = -1;
	FilterLevel.y = -1;
	for (i = 0; i <= idx; i++) 
	{
		found = false;
		// ob schon mal gezhlt
		for (int j = 0; j <= idx; j++) 
		{
			if ( lpPoints[i].y == YAccumulation[j].y )
			{
				(YAccumulation[j].x)++;
				found = true;
				break;
			}
		}
		// wenn neu dann aufnehmen
		if (!found) 
		{
			YAccumulation[i].y = lpPoints[i].y;
			YAccumulation[i].x = 1;
		}
	}
	// Niveau der groesste Haeufigkeit bestimmen
	for (i = 0; i <= idx; i++) 
	{
		if (FilterLevel.x < YAccumulation[i].x)
		{
			FilterLevel.x = YAccumulation[i].x;
			FilterLevel.y = YAccumulation[i].y;
		};
	};
	float MaxAcc = FilterLevel.x; // Maximale Hufigkeit merken
	// optimale Haeufigkeit fr Y-Wert oberhalb der groessten Haeufigkeit finden
	for (i = 0; i <= idx; i++) 
	{
		if ( (YAccumulation[i].y > FilterLevel.y) && ((MaxAcc-((MaxAcc/100)*m_FilterStrength)) < YAccumulation[i].x) ) // naechst kleinere Haufigkeit
		{
			FilterLevel.x = YAccumulation[i].x;
			FilterLevel.y = YAccumulation[i].y;
		};
	};
	// alle Y-Werte auf dieses Niveau anheben
	for (i = 0; i <= idx; i++) 
		if (lpPoints[i].y < FilterLevel.y) lpPoints[i].y = FilterLevel.y;
};

// ---------------------------------------------------------------------------
// 15.06.2004 Kullmann/Reinecker: verkuerzte und korrigierte Methode zur Skalierung der Y-Werte
// fuellt die Struktur lpPoints mit den Werten fuer eine Kurve
void TPlotWindow::FormatCurveToPLine(TCurve *crv, LPPOINT lpPoints, int &idx)
{
	idx = 0;
	float x; // x-Wert = Thetamotorwinkel  
	float y; // y-wert = gemessene Intensitaet des Detektors in dieser Position
	float z; // z-wert = Kanal
	float offset= GetThetaOffset(0); 

	// FastPGet(x,y,z) holt Werte der Kurve und setzt Kurvenindex weiter
	while (crv->PGet(x, y, z)) //07.05.2004 FastP...
	{
		// wenn X-Achse mit Theta-Winkel Beschriftung 
		if (!bSecondaryCoor) 
		{
			x += offset; // Theta-Offset beruecksichtigen
			lpPoints[idx].x= (x - CoorSys.xMin) / CoorSys.xSF;
		} else lpPoints[idx].x= (z - CoorSys.xMin) / CoorSys.xSF; // Beschriftung mit Kanaelen

		//if (fabs(z - x) == 0.0)	continue; // 15.06.2004 Fehler: was soll Kanaele - Winkel bringen ???
		
		if (y > CoorSys.yMin) // Ok, Wert ist oberhalb X-Achse
		{
			if (CoorSys.yScal == sLinear)
				lpPoints[idx].y= (y - CoorSys.yMin) / CoorSys.ySF;
			else
				lpPoints[idx].y= log10(y / CoorSys.yMin) / CoorSys.ySF;
		}
		else // sonst auf Niveau der X-Achse setzen
			lpPoints[idx].y= 0;	

		if ( idx != 0 ) {
			if ( !m_Dotted ) {
				// ehem. Fehler: Bei gleichem durch Skalierung entstandenem X-Wert 
				// wurde der Y-Wert des Vorgaengers einfach ueberschrieben.
				// so koennte z.B. das abs. Maximum verlorengehen!!!
				// Korrektur durch Abfrage 
				if ( (lpPoints[idx - 1].x == lpPoints[idx].x) ) {
					lpPoints[idx - 1].y = max( lpPoints[idx - 1].y, lpPoints[idx].y ); // bei gleichen x-Werten max. y-Wert nehmen
					continue;
				}
			} else 
				if ( (lpPoints[idx - 1].x == lpPoints[idx].x) && (lpPoints[idx - 1].y == lpPoints[idx].y) ) continue; // nie zwei genau gleiche Punkte in die Liste aufnehmen
		}
		idx++;
	};
};
// ---------------------------------------------------------------------------
//! sichert Koordinatensystem in AltCoorsys
//! m_main.cpp im Falle von cm_MoveScanReady
void TPlotWindow::SetKSProperties(TKSProperties* ksp)
{
	if (!ksp)
		return;

	AltCoorSys.yMin= ksp->fMinY;
	AltCoorSys.yMax= ksp->fMaxY;
	AltCoorSys.xMin= ksp->fMinX;
	AltCoorSys.xMax= ksp->fMaxX;
	if (ksp->bLogarithmicY)
		AltCoorSys.yScal= sLogarithmic;
	else
		AltCoorSys.yScal= sLinear;

	AltCoorSys.xScal= sLinear; // Logarithmic; geaendert, weil xST meiner Meinung nach nie Logarithmic
	bAltCoorSystem= TRUE;
};
//*****************************************************************************

void TPlotWindow::PickUpData(LPARAM data)
//! wird nur in m_main.cpp im Falle von cm_MoveScanReady benoetigt
//! sichert Daten in der SecondCurve
{
	*SecondCurve= *((TCurve *)data);
};
//*****************************************************************************

//! wird aufgerufen aus dem Scan-Fenster-Menue unter Scan -> Loeschen
//! bei gesendetem cm_KillSecondCurve
//! setzt SecondCurve auf Initialwerte zurck (loeschen aller Messtripel)
//! zeichnet Scan-Fenster neu
void TPlotWindow::KillSecondCurve(void)
//! SecondCurve wird mit Standardwerten ueberschrieben
{
	SecondCurve->New();
	//! Update des ClientWindows durch
	UpdateWnd();
};
//*****************************************************************************

//! wird aufgerufen aus dem Scan-Fenster-Menue unter Scan -> Fixieren
//! bei gesendetem Kommando cm_FreezeCurve
//! fixiert die aktuelle Hauptkurve -> setzt Pointer von SecondCurve auf MainCurve
void TPlotWindow::FreezeCurve(void)
//! speichert Objektzustand von MainCurve in SecondCurve
{
	if (1 > MainCurve->GetPNumber())
		return;
	*SecondCurve= *MainCurve;
};
//*****************************************************************************

//! wird aufgerufen aus dem Scan-Fenster-Men' unter Scan -> Speichern
//! bei gesendetem Kommando cm_SaveSecondCurve
//! speichert fixierte Kurve (SecondCurve) unter auszuwaehlendem Dateinamen
//! mit Extension *.bk
void TPlotWindow::SaveSecondCurve(void)
//! speichert die SecondCurve
{
	char NewFilename[ _MAX_PATH ];
	OFSTRUCT of;
	int hFile;
	float x, y, z;
	char buf[MaxString];

	// 'Datei Speichern'-Dialog anzeigen und ausgewhlte Datei auswerten
	char Extension[]= "bk";
	sprintf(NewFilename, "*.%s", Extension);
	char temp[_MAX_PATH];
	strcpy(temp, szMsgLine605); // szMsgLine605 nicht direkt angeben, weil const
	if ( !FileSaveDialog(szMsgLine604, temp, NewFilename, "") ) return; // Abbruch wurde geklickt
	if ( strlen(NewFilename)==0 || strcmp(NewFilename, "*")==0 ) return;
	char dir[_MAX_DIR], drv[_MAX_DRIVE], name[_MAX_FNAME], ext[_MAX_EXT];
	_splitpath(NewFilename, drv, dir, name, ext);
	_makepath(FileName, drv, dir, name, Extension);

	//! erstellen der Datei mit ausgewaehltem Namen + Test, ob dies moeglich
	//! wenn nicht, Fehlermeldung + Abbruch
	hFile= OpenFile(FileName, &of, OF_CREATE);
	if ( hFile==HFILE_ERROR )
	{
		MessageBox(sth_CannotCreateFile, "Fehler/Error", MBSTOP);
		return;
	}
	//! Wait-Mauszeiger setzen
	HCURSOR hOldCursor= SetDefaultCursor( IDC_WAIT );

	strcpy(buf, szMsgLine606);
	//! FileType mit eingefuehrt - TScanWindow.eSaveFormat= bkFile (beim Einlesen)
	strcat(buf, "FileType=Comparison\n");
	strcat(buf, "X >> Intensity >> Z\n[Data]\n");
	_lwrite(hFile, (LPSTR)buf, strlen(buf));

	//! an Anfang der Kurve gehen + Speicher der Punkte fixieren
	//! setzt ActualIdx auf 0 oder auf MaxPointIdx, wenn pos>MaxPointidx
	//! hier ActualIdx=0
	SecondCurve->SetPP();

	//! folgendes wird mit FastOpen aktualisiert (betrifft Klasse Curve...)
	//! BOOL* lpPV; // Zeiger auf Datenfeld von Validate
	//07.05.2004 SecondCurve->FastOpen();

	//! FastPGet(x,y,z) holt Werte der Kurve und setzt ActualIdx++ (m_curve.cpp)
	//! Kurve Tripel fr Tripel auslesen und zeilenweise in der Datei ablegen
	while ( SecondCurve->PGet(x, y, z) ) //07.05.2004 FastP...
	{   //! FIX Fehler 2 Diff/Refl
		sprintf(buf, "%.3f %.3f %.3f", x, y, z);
		strcat(buf, EOL);
		_lwrite(hFile, (LPSTR)buf, strlen(buf));
	}
	//! Schlieen der Datei + Fixieren des Speichers der Kurve aufheben
	//! Wait-Mauszeiger ausschalten
	_lclose(hFile);
	//! GlobalUnlock(hMemX);GlobalUnlock(hMemY);GlobalUnlock(hMemZ);
	//! GlobalUnlock(hMemV);bStreamOpen= FALSE; - Fkt. FastClose();
	// 07.05.2004 SecondCurve->FastClose();
	SetCursor(hOldCursor);
	return;
};