#ifndef __AUTOJUST_H
#define __AUTOJUST_H

// Struktur, die die Positionen der Motoren der
// bestimmten Antriebsachsen aufnimmt (OrigDF/TL/CC)
// zustzlich werden die Intervallschranken, innerhalb
// derer die Optimierung durchgefhrt wird, gespeichert
// (Min/MaxDF/TL/CC)
struct TMotorPositionsWerte
{
	double MinDF;
	double MaxDF;
	double OrigDF;
	double MinTL;
	double MaxTL;
	double OrigTL;
	double MinCC;
	double MaxCC;
	double OrigCC;
};

// Struktur, in der die Intensitt (Intensity)
// an einer bestimmten Position (DF,TL,CC) gespeichert wird
struct TIntensityPosition
{
	double DF;
	double TL;
	double CC;
	float Intensity;
};

// mgliche Arbeitsschritte whrend der automatischen Justage
enum EJustagePoints { jpSetParameters, jpInitialize, jpChooseAxis, jpOptimizingCC, jpOptimizingTL, jpOptimizingDF, jpCalculate, jpPostProcessing };

class TransformationClass;

// Dialogklasse der Automatischen Justage
class TAutomaticAngleControlDlg : public TModalDlg, public ITimer
{
public:
	TAutomaticAngleControlDlg();
	virtual ~TAutomaticAngleControlDlg();

// berschriebenes aus der (Fenster-)Basiaklasse
public:
	virtual BOOL Dlg_OnInit ( HWND, HWND, LPARAM );
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );
	virtual void LeaveDialog(void);

	// sorgt fr eine sequenzielle Abarbeitung der Arbeitsschritte (dazwischen Systemzeit freigeben)
	// nchster Schritt in <m_NextStep>
	virtual void OnTimer ( TBasicTimer *const );

private:
	void NextStep(EJustagePoints aNext, BOOL= FALSE);
	TIntList m_NextStep; // Liste von EJustagePoints
	TInterfaceTimer *m_lnkTimer;

	// Tranformationsobjekt als Member definiert
	TransformationClass *Transform;

	TDetector *Sensor;


	// Temporre Buffer zur Stringzusammensetzung
	char *buf, *buf1;

	// Inhalt des Statusfeldes
	char *status;

	// maximale Groesse fuer das Status-Textfeld
	unsigned nMaxString;

	//Aktivierung der Abbruchkriteriums Intensittsdifferenz
	bool bIntensityDiffActive;

	// Laufvariablen
	int durchlauf, step, step1, count;

	// Anzahl der Zeilen im Status-Textfeld
	DWORD dwStatusZeilen;

	// Zahl der durchzufhrenden Messungen pro Position
	unsigned nMeasureCount;

	// Positionsvariablen, Intensittsvariablen
	double fPosition, intervall_li, intervall_re, x, y, z, x2, y2, z2;
	double MaxTL, MaxDF, MaxCC;
	float fIntenseDifferenz;

	// Variablen zur Ausgabe der Justagedauer
	SYSTEMTIME datum;
	SYSTEMTIME zeit;
	int justageanfang, justageende, justagezeit;

	// Variablen zur Speicherung der Wertebereiche & Intensittspositionen
	TMotorPositionsWerte Wertebereich, aktWertebereich;
	TIntensityPosition MaxIntensity, LastIntensity, ActIntensity;
};

#endif
