//##############################################################################
//                                                                            //
// A_INTERN.H                                                                 //
//                                                                            //
// Subsystem: Diffraktometrie/ Reflektometrie                                 //
//            intern genutzte Fenster fr AreaScan                            //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//                                                                            //
//##############################################################################

#ifndef _A_INTERN_H
#define _A_INTERN_H

#include "utils\u_utils.h" // TModalDlg, TModelessDlg
#include "datavisa\datavisa.h" // fr TCurve

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

class TAreaScanWindow; // forward

//##############################################################################
// TSetupAreaScanDlg
//##############################################################################
// Dialog zum Einstellen der AreaScan-Parameter 

class TSetupAreaScanDlg : public TModalDlg
{
public:
	TSetupAreaScanDlg( TAreaScanWindow* );

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );
	virtual BOOL CanClose( void );

// weitere Ereignisse
private:
	void OnParamSet(HWND);
	void OnInquireRelevantData();
	void OnChooseDetector(HWND, HWND, UINT);
	void OnFixThetaAxis();
	void OnSetOffset(HWND);
	void OnChooseMonitor(UINT);

// ATTRIBUT(E)
private:
	//! zugehoeriges AreaScanfenster
	TAreaScanWindow *Areascan; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden

	//! enthaelt Nummer des Omega- sowie Thetamotors in der Motorenliste
	int nOmega, nTheta;

	//! scheinbar ueberfluessig
	//08.04.2004 int nOldMotor;

	//! Handle der Detektorauswahlbox
	HWND hDetectorList;

	//! beinhaltet den Wert des Feldes Minumum im Gruppenfeld 2Theta...
	double d2ThetaStart;

	//! beinhaltet o-t-offset
	double dOmegaOffset;
	double dThetaOffset;
};

//##############################################################################
// TAquisitionDlg
//##############################################################################

class TAquisitionDlg : public TModalDlg
{
public:
	TAquisitionDlg( TAreaScanWindow* );

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual BOOL CanClose( void );

// ATTRIBUT(E)
private:
	//! zugehoeriges AreaScanfenster
	TAreaScanWindow *Areascan; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden

	//08.04.2004 int nNumberPoints;
	//08.04.2004 int nRange;
	//08.04.2004 char FileName[ 20 ];
	//08.04.2004 HWND hRangeList;
};

//##############################################################################
// TChooseScanDlg
//##############################################################################
// Dialog zum Auswhlen von Scan's

class TChooseScanDlg : public TModelessDlg
{
public:
	TChooseScanDlg( TAreaScanWindow*, TModelessDlg** );

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );
	virtual void Dlg_OnVScrollBar( HWND, HWND, UINT, int );
	virtual void Dlg_OnLButtonUp( HWND, int, int, UINT );
	virtual BOOL CanClose( void );

// ATTRIBUT(E)
private:
	//! zugehoeriges AreaScanfenster
	TAreaScanWindow *Areascan; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden

	//! Pointer auf akt. angezeigte Kurve
	// 21.04.2004 TCurve* Curve;

	//! Pointer auf Scanreportkurve der datenbasis
	TCurve* Report; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden
	//! Pointer auf 2.Scanreportkurve (zusatzinfos)
	TCurve* Report2; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden

	//! Handle des Scrollbars
	HWND BarHandle;

	//! Handle der Info-Combobox
	HWND hInfoList;

	//! enthaelt Nr. des letzten Scans in Datenbasis
	int nMaxScan;

	//! beinhaltet zuordnung (Nr. des Comboxeintrages zu zusatzinfo)
	TIntList InfoField;

	//! enthaelt Nr. der aktuell ausgewaehlten zusatzinfo
	//! 0-Messdetektor / 1-integr.Int. / 2-absorber / 3-Monitorint / 4-messzeit
	int nInfoNumber;
};

//##############################################################################
// TComposeDBDlg
//##############################################################################
//! Dialog zum Zusammensetzen der Datenbasis

class TComposeDBDlg : public TModalDlg
{
public:
	TComposeDBDlg( TAreaScanWindow* );

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );
	virtual BOOL CanClose( void );

// ATTRIBUT(E)
private:
	//! zugehoeriges AreaScanfenster
	TAreaScanWindow *Areascan; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden

	char oldFirstFile[ _MAX_PATH+1 ], oldLastFile[ _MAX_PATH+1 ];
	int oldMaxFiles;
};

//##############################################################################
// TDismantleDBDlg
//##############################################################################
//! Dialog zum Zerlegen der Datenbasis

class TDismantleDBDlg : public TModalDlg
{
public:
	TDismantleDBDlg( TAreaScanWindow* );

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual BOOL CanClose( void );

// ATTRIBUT(E)
private:
	//! zugehoeriges AreaScanfenster
	TAreaScanWindow *Areascan; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden
};

#endif // _A_INTERN_H

//#############################################################################
// LastLine
//#############################################################################