//##############################################################################
//                                                                            //
// D_INTERN.H                                                                 //
//                                                                            //
// Subsystem: Diffraktometrie/ Reflektometrie                                 //
//            allgemeine, intern genutzte Fenster                             //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//                                                                            //
//##############################################################################

#ifndef _D_INTERN_H
#define _D_INTERN_H

#include "utils\u_utils.h" // TModalDlg
#include "MOTRSTRG\M_MOTCOM.H" // _MAXLENUINT

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

const UINT stdx= 500; //! Standardgroesse Koordinatensystemaussschnitt x
const UINT stdy= 500; //! Standardgroesse Koordinatensystemaussschnitt y

//! Bildschirm hat x und y Ausdehnung
struct TScreen 
{
	int x0, x1, dx, 
		y0, y1, dy;
};

struct TDisplay 
{
	double dThetaWindow, dMoveRelation, dOmegaMin, dOmegaMax;
	double dThetaMinFirst, dThetaMaxFirst, dThetaMinLast, dThetaMaxLast;
	float fAngleRange, fAngleStep;
	BOOL bPsd;
};

const UINT _COLORS= 256; //! Anzahl benutzter Farben (256)

class TColorTable 
{
public:
	TColorTable() : ColorMap(_COLORS, 255) {
		place.left= place.top= place.right= place.bottom= 0;
		zMin= zMax= 0;
		strcpy(zUnit, "");
	};

	RECT place;
	float zMin, zMax;
	char zUnit[ _MAXLENUNIT+1 ];
	TFloatList ColorMap;
};

//! Koordinatensystem mit x,y,z
//! und jeweils einer Angabe 1 Pixel in x,y,z Richtung
//! entspricht *SF Koordinatensystemeinheit
//! und eingestellter Ausgabetyp
struct TCoorSystem 
{
	double xMin, xMax, xSF;
	double yMin, yMax, ySF;
	double zMin, zMax, zSF;
	EScale xScal, yScal, zScal;
};

//! Aufzaezhlung Ausgabetypen/Darstellungsformen
enum EOutputType 
{
	otReciprokeLatticeBitmap= 1, otMatrixBitmap= 2, otCurve= 3 // die zugeordneten nummerischen Werte mssen so belassen werden, weil sie zum Zugriff auf die Elemente in einer ComboBox benutzt werden
}; //! weil sowieso nicht implementiert, Number, BarGraph, Histogram

//! Umrechnung Alt-/Neugrad...
#define DegToRad(val) ((val) * 0.01745329251994)
#define RadToDeg(val) ((val) / 0.01745329251994)

//! Umrechnung in Reziproke Einheiten
#define ArToRLx(_ome, _the) ((cos((_the)-(_ome))- cos(_ome))) 
#define ArToRLy(_ome, _the) ((sin((_the)-(_ome))+ sin(_ome))) 
/*
17.07.2003 gendert, weil Umrechnung so korrekt und Darstellung nicht gespiegelt...
#define ArToRLx(_ome, _the) ((cos(_ome) - cos((_the) - (_ome))))
#define ArToRLy(_ome, _the) ((sin(_ome) + sin((_the) - (_ome))))
*/

class TPlotWindow; // forward

//##############################################################################
// TCurveFreeScalDlg
//##############################################################################
//! neuer Dialog fr freie Intensitaetskalierung

class TCurveFreeScalDlg : public TModalDlg
{
	friend class TCurveFreeScalColorWindow;
public:
	TCurveFreeScalDlg( TPlotWindow* );

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );
	virtual BOOL CanClose( void );
	virtual void LeaveDialog( void );

// Verwaltung der Steuerelemente in den 8 Bereichen
private:
	int GetFarbeId(UINT);
	int GetBereichId(UINT);

// ATTRIBUT(E)
private:
	//! Plotdaten werden gebraucht
	TPlotWindow *PlotWindow; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden
};

//##############################################################################
// TCurveShowParamDlg
//##############################################################################
//! Dialog zum Einstellen der PlotFenster-Parameter

class TCurveShowParamDlg : public TModalDlg
{
public:
	TCurveShowParamDlg( TPlotWindow* );

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual BOOL Dlg_OnInit( HWND, HWND, LPARAM );
	virtual void Dlg_OnCommand( HWND, int, HWND, UINT );
	virtual void Dlg_OnHScrollBar(HWND , HWND , UINT , int );
	virtual BOOL CanClose( void );
	virtual void LeaveDialog( void );

// ATTRIBUT(E)
private:
	//! fast alle Plotdaten werden gebraucht
	TPlotWindow *PlotWindow; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden

	//! Groesse der x und y-Achse ueber Scrollbalken beeinflussbar
	int scrolldxpos, scrolldypos;

	//! nicht mehr benoetigt
	// TOutputType bOutputToShow;

	//! einige Handles behalten...
	HWND hWndOutputT, hWndScaleT, hWndScrolldx, hWndScrolldy;

	// Kanal- oder Winkeldarstellung (zum Wiederherstellen der letzten Einstellung, wenn RL-Bitmap aktiv)
	BOOL bSavedSecondaryCoor;
};

//##############################################################################
// TBitmapSource
//##############################################################################
//! Klasse um Bitmaps aus Kurvendaten zu errechnen und darzustellen

class TBMContens 
{
public:
	TBMContens() { omega= theta= intens= 0; };

	float omega, theta, intens;
};

const RLAreaPkts= 41;

class TBitmapSource
{
	friend class TPlotWindow;
	friend class TCurveShowParamDlg;
	friend class TCurveFreeScalDlg;
	friend class TCurveFreeScalColorWindow;

public:
	//! Konstruktor
	TBitmapSource( TPlotWindow* );

	//! Destruktor
	virtual  ~TBitmapSource();

public:
	//! wird aufgerufen, wenn ein Bitmap nachgeladen werden soll (m_arscan.cpp)
	// derzeit nicht verwendet 17.05.2004 BOOL ProcessBitmapFile( HFILE );

	//! generiert den Header fuer ein Bitmap, setzt Koordinatensystem und Screen
	void FormatDBaseToBitmapSource( void );

	//! ermglicht ein Update, derzeit leer implementiert
	void UpdateBitmapSource( void ) {};

// Zugriff auf m_Daten
public:
	bool IsDatenInited() { return m_DatenInited; };

	//*** zudem wird in der DEBUG-Version eine TraceErr-Fehlermeldung ausgegeben
	// bei zu groen/ kleinen Zeilen/ Spalten wird der Wert aus der letzten Zeile/ Spalte zurckgegeben***
	float GetOmega(UINT aRow, UINT aCol);
	float GetTheta(UINT aRow, UINT aCol);
	float GetIntens(UINT aRow, UINT aCol);

	// bei zu groen/ kleinen Zeilen/ Spalten wird nichts gesetzt und false zurckgegeben***
	bool SetOmega(UINT aRow, UINT aCol, float aValue);
	bool SetTheta(UINT aRow, UINT aCol, float aValue);
	bool SetIntens(UINT aRow, UINT aCol, float aValue);

private:
	void CreateDaten(UINT aRows, UINT aCols);
	bool CheckKoord(UINT &aRow, UINT &aCol); // korrigiert aRow und aCol bei berschreitung und gibt false zurck; ***
	void FreeDaten();

	void New( void );

	//! laedt Bitmap Ressource - benoetigt?
	void LoadBitmap( HDC );

	//! berechnet Bytes pro Zeile*Hoehe des Bitmaps - Gesamtgroesse im Speicher
	DWORD GetImageSize( void );

	//! ruft bei RL_Bitmap DrawMeasurementArea()
	//! berechnet Position der Anordnung der Farblegende
	//! setzt Min,MaxIntensitaeten
	//! ruft DrawColorTable()
	void DrawBitmapFrame( HDC, TScreen& );

	//! setzt Screen-Variablen, entsprechend der Bildschirmgroesse
	//! setzt Units, Orientation, Scaling
	void SetScreen( HDC, TScreen& );

	//! berechnet zu den Intensitten entsprechende Farbwerte und zeichnet die Farb-Legende
	void DrawColorTable( HDC, TColorTable& );

	//! falls RL-Bitmapdarstellung, berechnet und zeichnet
	//! das untersuchte Messgebiet im den reziproken Raum
	void DrawMeasurementArea( HDC );

	//! fuellt den Header eines Bitmaps mit Farbwerten der Palette
	void FillBMInfoFromPalette( LOGPALETTE* );

	//! Bitblock-Transfer von hMemDC (dem gegebenen Quell-DC)
	//! zu diesem HDC (dem Ziel-DC) - entspricht der Darstellung eines Bitmaps
	void DrawBitmap( HDC, int, int, TScreen& );

	//! function creates a device-dependent bitmap (DDB) from a device-independent
	//! bitmap (DIB) and, optionally, sets the bitmap bits
	HBITMAP CreateGDIObject( HDC );

	//! Zwischenablageoperation, wenn Typ CF_DIB
	HGLOBAL RenderDIB( void );

	//! ruft CreatePaletteFromDIB(), ansonsten CreateDefaultPalette()
	LOGPALETTE *GetPalette( void );

	//! generiert Palette von DIB (Device independend Bitmap)
	LOGPALETTE *CreatePaletteFromDIB( void );

	//! generiert neue Palette (Graustufen oder Bunt)
	LOGPALETTE *CreateDefaultPalette( void );

	//! gibt Darstellungtyp zurueck
	//! wird nicht benutzt
	// TOutputType GetType( void );

	//! um Intensitaetsabstufungen im User Falle zu generieren
	//! ansonsten werden Defaultwerte gesetzt
	//! fuellt ColorTable.ColorMap[cnt] mit Werten
	void ColorDef(void);

	//! setzt Variablenfelder farbe[] und ber[]
	//! so dass ein linearer Verlauf abhaengig von Min-,MaxIntensitaet
	void BereichDef(void);

	//! gibt zu uebergebener Intensitaet entsprechende Intensitaetsfarbe zurueck
	int GetColor( float );

	//! generiere MatrixBitmap
	void GenerateAngleSpaceBitmap(void);

	//! genriere RL-Bitmap
	void GenerateRLBitmap( void );

// ATTRIBUT(E)
private:
	// ehemalig global definiert
	double clogval;
	double clinval;
	HBITMAP hBitmap;
	POINT RLArea[RLAreaPkts];

private:
	//! Zeiger auf zugehoeriges TPlotWindow-Objekt
	TPlotWindow *PlotWindow; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden

	//! eigenes Koordinatensystem fuer Bitmapberechnung
	TCoorSystem *BMCoorSys; // nicht im Destruktor freigeben, wird von DataWnd bernommen

	//! Zeiger auf zugehoerigen TCurveFreeScalColorWindow-Dialog
	TCurveFreeScalColorWindow *UserFarb; // wird im Destruktor freigegeben


	//! Zeiger auf "Daten"feld
	//! hier wird zu jeder [x,y]-Pixelkoordinate der zugehoerigen
	//! Theta-/OmegaWert gespeichert (nur RL-Bitmap)
	TBMContens **m_Daten; // wird im Destruktor freigegeben

	// Dimensionen des <m_Daten>-Feldes
	UINT m_DatenRows, m_DatenCols;

	//! (bei RL-Bitmap wird zu jedem Bildpunkt Theta/Omega gespeichert)
	// gibt an, ob <m_Daten> schon initialisiert wurde
	bool m_DatenInited;


	//! False - keine Graustufendarstellung, True - Graustufendarstellung in Bitmaps
	//! wird im ini-File mit "swPalette" gesetzt
	BOOL bGrayScalePalette;	

	//! Zeiger auf ColorTable-Objekt
	TColorTable ColorTable;

	//! wichtig fuer Bitmap-Generierung
	HGLOBAL hDIBInfo;

	HGLOBAL hDIBData;

	HGLOBAL hMemPal;

	HPALETTE hPalette, hOldPalette;

	//! nicht mehr benoetigt, da GenerateAngleSpaceBitmap() geaendert
	//! int nEnlargedLines;

	//! int nAddedPoints;

	//! Feld mit Intensitaetsabstufungen im Falle von "User Skalierung"
	TFloatList ber;

	//! Feld mit Farbabstufungen im Falle von "User Skalierung"
	TIntList farbe;

	//! Speicherung des alten Filenamens
	//! zwecks Vergleich, wenn neues "Daten"feld angelegt werden muss
	char Fileold[MaxString];
};

//##############################################################################
// TCurveFreeScalColorWindow
//##############################################################################

class TCurveFreeScalColorWindow : public TMDIWindow
{
public:
	//! Konstruktor
	TCurveFreeScalColorWindow(HINSTANCE, TPlotWindow*);

	//! Destruktor
	virtual  ~TCurveFreeScalColorWindow(void);

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual void OnCreate();
	virtual BOOL SetTitle();
	virtual void DoPaint(HDC, PAINTSTRUCT*);

// Sonstiges
private:
	//! zeichne Farblegende
	void DrawColorTable(HDC);

// ATTRIBUT(E)
protected:
	//! Zeiger auf TPlotWindow-Objekt
	TPlotWindow *PlotWindow; // nicht im Destruktor freigeben, wird im Konstruktor bergeben und muss vom Aufrufer verwaltet werden
};

#endif // _D_INTERN_H

//#############################################################################
// LastLine
//#############################################################################