//##############################################################################
//                                                                            //
// S_EXTERN.H                                                                 //
//                                                                            //
// Subsystem: Diffraktometrie/ Reflektometrie                                 //
//            DAS MDI-Fenster fr Continious-, Step- und DynamicStepScan      //
// Benutzung durch andere Subsysteme erforderlich: JA                         //
//                                                                            //
//##############################################################################

#ifndef _S_EXTERN_H
#define _S_EXTERN_H

#include "difrkmty\d_extern.h" // TPlotWindow, TSaveFormat

#include "detecuse\detecuse.h" // TDetector

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//! Anzahl der max. unterschiedlichen Schrittweiten (dyn. Schrittweite)
const int nMaxScaleIdx= 10;

//##############################################################################
// TScanParameters
//##############################################################################
//! Datenhaltung fr Continious-, Step- und DynamicStepScan

class TScanParameters
{
	//! einlesen der Ini-Daten zu [Scan] und setzen von Defaultwerten fr StepScan
public:
	TScanParameters( void );

//ATTRIBUT(E)
public:
	//! aktueller Detektor zum Messen
	TDetector *Detector; // nicht im Destruktor freigeben, wird durch Detektorsubsystem verwaltet
	//! Detektor fuer Monitor
	TDetector *Monitor; // nicht im Destruktor freigeben, wird durch Detektorsubsystem verwaltet

	//! beinhaltet Nummer der aktuellen Achse
	int nMotor;

	//! Speicherformat fuer Kurven
	EFileFormat eSaveFormat;

	//! Art des aktuellen Scan(StandardScan, Omegea2Theta o. ContinuousScan)
	ExScanType eScanType;

	//! Speichern nach Beendigung der Messung
	BOOL bSaveOnReady;

	//! Anzahl der Messpunkte(nur bei Cont.Scan)
	int nCollectionAmount;

// Omega
	//! Startwinkel des Scans
	double dArgumentMin, dArgumentMax, dArgumentWidth;
	//! Geschwindigkeit des aktuellen Motors
	double dSpeed;

	//! gibt 2ThetaStartposition fuer einen zu fahrenden Stepscan an beim O2T-Scan an
	double d2ThetaStart;

	WORD nNumberCycle; // Anzahl der Mess-Zyklen
	float fMaxTime; // Maximale Messzeit
	DWORD dwMaxCounts; // Maximale Anzahl Impulse

	//! Speichern waehrend der Messung
	BOOL bSaveContinuous;

	//! gibt an ob dynamische Schrittweitensteuerung aktiv
	BOOL bDynamicStepWidth;
	//! Anzahl der verschied. Schrittweiten
	int nStepScaleLevels;

	//! Liste enthaelt in den ersten 10 werten die intensitaeten und in den
	//! zweiten 10 werten den zughoer. Schrittweitenfaktor
	TFloatList Scaler;
};

//##############################################################################
// TScanWindow
//##############################################################################
// DAS MDI-Fenster fr Continious-, Step- und DynamicStepScan

class TScanWindow : public TScanParameters, public TPlotWindow
{
	//Kullmann+Reinecker: zahlreiche unntige friend-Deklarationen entfernt
	friend class TScanCmd;
	friend class TSaveDataCmd;

public:
	TScanWindow( HINSTANCE );
	virtual  ~TScanWindow();

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual void OnCreate();
	virtual LRESULT OnCommand(WPARAM wParam, LPARAM lParam);

	virtual void OnPopupMenuInit ( WPARAM, LPARAM );

	virtual BOOL SetTitle( void );

	virtual BOOL CanNew()
	{ return !bLineScanStarted || bInterrupted; /*Messung beendet*/ }
	virtual BOOL CanOpenFile()
	{ return !bLineScanStarted || bInterrupted; /*Messung beendet*/ }
	virtual BOOL CanSave()
	{ return !bLineScanStarted || bInterrupted; /*Messung beendet*/ } 
	virtual BOOL IsScanning(BOOL &PsdContinious) 
	{ PsdContinious= FALSE; return bLineScanStarted; }

	virtual BOOL New( void );
	virtual BOOL LoadOldData();
	//! speichern einer .crv Datei
	//! ask== NoAskForSave(0x9900) => speichert unter Namen aus FileName
	//! ohne Abfrage
	//! == SaveWithNewName(0x9901) => speichert unter Namen, der in der
	//! Dialogbox erst noch enzugeben ist
	//! Rueckkehrcode False wenn Fehler bei anlegen der Datei oder bei
	//! speichern der Headerinfo. sonst true
	virtual BOOL SaveFile( EAskType );
	//! haengt aktuelles tripel der Messkurve an Datei mit FileName an
	//! aufruf nur bei kontinuierlichem Speichern
	//! Rueckkehrcode False wenn fehler bei oeffnen der Datei sonst true
	virtual BOOL UpdateFile();

	//! Ablaufsteuerung
	virtual void SteeringReady( LPARAM ); // Ende
	virtual void SteeringReset( LPARAM ); // Abbruch	
	virtual void Interrupt(); //! unterbricht Scanvorgang

	//! erhaelt Messpunkt, stellt ihn dar, speichert ihn ggf.
	virtual void CounterSetRequest( LPARAM );

	//! passt x+y-Scale des Koord.sys an neue Werte an
	virtual void SetRanges( void );

	//! liefert Klassennamen
	virtual LPCSTR ClassName()
	{
		return "Scan";
	};

	//! liefert Charakteristik
	virtual LPCSTR GetCharacteristic()
	{
		return "Curve";
	};

// berschriebenes aus TPlotWindow
private:
	// gibt die Beschriftung der X- und Y-Achse zurck
	virtual void GetTitleX(int, int, char *, bool);
	virtual void GetTitleY(int, int, char *, bool);

	//! sichert Koordinatensystem in AltCoorsys
	//! m_main.cpp im Falle von cm_MoveScanReady
	virtual void SetKSProperties( TKSProperties* );

// Laden+Speichern
private:
	//! laden der Messungsdaten aus einer .crv bzw. .bk datei
	//! wird von TScanWindow::LoadOldData() in m_scan.cpp aufgerufen
	//! hFile ist Filehandle der Messdatei
	//! Rueckkehrcode False wenn Datei zu kurz oder [HEADER] nicht vorhanden
	//! sonst true
	BOOL LoadMeasurementInfo( TTxtRead& );
	//BOOL LoadMeasurementInfo( HFILE );

	//! Speichern bzw. Aktualisieren der Headerinformation
	//! Par.==0 => neu erstellen
	//! ==1 => aktualisieren
	//! Rueckkehrcode False wenn Fehler bei speichern sonst true
	BOOL SaveMeasurementInfo( BOOL );

// Protokollbuch
private:
	//! klier 01.03.2003 Protokollbuch
	//! Initialisiert Speicherung im Protokollbuch
	void InitializeMeasurementProtocol( void );

	//! klier 01.03.2003 Protokollbuch
	//! Speichert Informationen im Protokollbuch
	void SetMeasurementProtocolParameter( void );

// Sonstiges
private:
	// Methoden fuer die Initialisierung verschiedener Scan-Tasks
	void InitializeStandardScanTask();
	void InitializeOmega2ThetaScanTask();
	void InitializeContinuousScanTask();

	// 21.04.2004 BOOL SaveDataBase( void );
	// 21.04.2004 void StopMeasurement();
	// 21.04.2004 void ExecuteNextCmd( void );

//ATTRIBUT(E)
private:
	// wie bMeasurementActive, aber bereits beim Anfahren der Startposition == TRUE	
	BOOL bLineScanStarted;

	//! soll Scanstart vor Aufruf einer Einstellungsbox verhindern
	//! (Fehler 2)
	BOOL bScanSetupOk;

	TDetector *oldDetector; // nicht im Destruktor freigegeben, wird durch das Detektorsubsystem verwaltet
};

#endif // _S_EXTERN_H

//#############################################################################
// LastLine
//#############################################################################