//#############################################################################
//                                                                           //
// MJ_GUI.H                                                                  //
//                                                                           //
// Subsystem : neue Manuelle Justage / Oberflche                            //
// Benutung durch andere Subsysteme erforderlich: JA                         //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand  : 21.04.2002                                                       //
//                                                                           //
//#############################################################################

#ifndef __MJ_GUI_H
#define __MJ_GUI_H

#include "manjust\mj_funk.h" // die Funktionalitt

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// globale Typdeklarationen
//#############################################################################

enum ETB { tb1= 1, tb2= 2, tb3= 3 }; // 0 fr "unbekannter Teilbereich"
enum ERessourceId{ riMotorList,   //Kombinationsfeld: Antriebsauswahl
				   riStart, riSetRelativeZero, riResetRelativeZero, riOffset,   //Schaltflche: Start/ Stop; Relative Null setzen, aufheben; "Offset..."
				   riDirectMode, riDriveMode, riStepMode,   //Optionsfelder: Betriebsart
				   riAngleDest, riSpeed, riStepWidth,   //Eingabefelder: Bewegungsparameter
				   riAngle, riOffsetState,   //Eingabefelder: Istposition, Offset
				   riRelativePos,   //Bildlaufleiste: relative Positionierung
				   riRelativePosMin, riRelativePosMax,   //Textfelder: Min-, Maxposition
				   riAngleDestUnit, riSpeedUnit, riStepWidthUnit, riAngleUnit, riRelativePosUnit,   //Textfelder: Einheit des Antriebs
				   riRefPoint //Textfeld: fehlender Referenzpunkt
				 };

//#############################################################################
// TManJustageDlg
//#############################################################################

class TManJustageDlg : public TModalDlg, public ITimer
{ // Hauptdialogfenster Manuelle Justage

  //*****************************************************************************
	// KONSTRUKTOR-DESTRUKTOR
public:

	// Konstruktor/Destruktor
	TManJustageDlg ( void );
	virtual  ~TManJustageDlg ( void );

	//*****************************************************************************
	// EREIGNISSE DER BASISKLASSE
protected:

	// stellt die Oberflche wieder her, wie sie verlassen wurde (nach Aufruf des Dialogfensters aufgerufen)
	virtual BOOL Dlg_OnInit ( HWND, HWND, LPARAM );

	// zum Verarbeiten von benutzerdefinierten Botschaften
	virtual LRESULT OnEvent ( HWND, UINT, WPARAM, LPARAM );

	// hier kommen die Windows-Botschaften an; die fr die Teilereiche werden an CheckLeft, CheckMiddle, CheckRight weitergeleitet
	virtual void Dlg_OnCommand ( HWND, int, HWND, UINT );

	// hier kommen die Bildlaufleistenbotschaften an (nur Fahr-/ Schrittbetrieb)
	virtual void Dlg_OnHScrollBar ( HWND, HWND, UINT, int );

	// darf der Dialog beendet werden
	virtual void LeaveDialog ( void );

	// POLLING-Timer Ereignis das die Antriebsbewegung und Makroverarbeitung berwacht und den Fortschritt darstellt
	// ruft dazu die weiter unten genannten Ereignisse
	virtual void OnTimer ( TBasicTimer *const );

private:

	// die Bewegung des angegebenen Antriebs hat begonnen
	void OnMotionStarts ( const UINT );

	// die Bewegung ist im vollen Gange
	void OnMotionProgress ( const UINT );

	// die Bewegung hatgestoppt
	void OnMotionStops ( const UINT );


	// die Makroverarbeitung wurde gestartet
	void OnMeasureStarts ( void );

	// ... ist in vollem Gange
	void OnMeasureProgress ( void );

	// ... ist beendet
	void OnMeasureStops ( void );

	//*****************************************************************************
	// Verhalten aller Steuerelemente auerhalb der Teilbereiche
private:

	// Psd-Offset definieren
	void OnPsdOffset ( void );

	// Teilbereich wechseln (Tastenkombination Ctrl+Tab)
	void OnChooseTB ( void );

	// Meldung ausgeben, wenn die gewnschte Aktion nicht ausgefhrt werden kann
	void OnNoAction ( void );

	//*****************************************************************************
	// Verhalten aller Steuerelemente innerhalb der Teilbereiche
private:

	// ==> TRUE <--> Botschaft durch eines der hier behandelten Steuerelemente verarbeitet
	BOOL TB_CheckLeft ( const ETB, const int ); // Auswahlliste Antrieb, Betriebsarten
	BOOL TB_CheckMiddle ( const ETB, const int ); // Start/ Stop und Betriebsarten, wenn [ENTER] gedrckt wurde
	BOOL TB_CheckRight ( const ETB, const int ); // Relative Null setzen, aufheben; Offset setzen
	BOOL TB_CheckExit ( const ETB, const int ); // Verhalten fr's Verlassen von Sollposition, Bewegungsgeschwindigkeit und Schrittweite

	//*****************************************************************************
	// Verhalten fr ausgewhlter Steuerelemente innerhalb der Teilbereiche
private:

	// Start bzw. Stop
	void TB_StartStopClicked ( const ETB );

	// der Parameter ist der Index des Antriebs, dessen Parameter gesetzt werden sollen
	void  TB_AngleDestEntered ( const ETB ); // Zielposition             wird in die Funktionalitt bernommen      
	void  TB_SpeedEntered     ( const ETB ); // Bewegungsgeschwindigkeit      
	void  TB_StepWidthEntered ( const ETB ); // Schrittweite

	// <Antrieb auswhlen> wurde benutzt
	void TB_MotorChanged ( const ETB );

	//*****************************************************************************
	// quasiparallele Aktualisierung (Inhalt und Zustand) aller Steuerelemente
private:

	// sendet eine "USR_"-Botschaft an den Dialog, dementsprechend wird dort eine der folgenden Methoden aufgerufen,
	// so dass die quasiparallel verarbeitet werden knnen
	void Notice( const int, const ETB, const BOOL= FALSE );

	// Inhalt aller Steuerelemente innerhalb des angegebenen Teilbereichs aktualisieren
	// Wenn der zweite Parameter mit TRUE ist, wird auch der Zustand der Steuerelemente aktualisiert.
	void OnTB_Controls ( const ETB, const BOOL ); // alle Steuerelemente
	void OnTB_KindOfMove ( const ETB, const BOOL ); // Betriebsart
	void OnTB_StartStop ( const ETB, const BOOL ); // Start bzw. Stop
	void OnTB_MotionParam ( const ETB, const BOOL ); // Bewegungsparameter
	void OnTB_Positions ( const ETB, const BOOL ); // Min-, Ist-, Soll-, Maximalposition und Bildlaufleiste
	void OnTB_Offset ( const ETB, const BOOL ); // Offset, Relative Null setzen und aufheben
	void OnTB_Units ( const ETB );             // Einheiten

	// Zustand aller Steuerelemente innerhalb des Teilbereichs aktualisieren
	void OnTB_ControlsEnable ( const ETB ); // alle Steuerelemente
	void OnTB_MotorListEnable ( const ETB ) const; // Antriebsauswahlliste
	void OnTB_KindOfMoveEnable ( const ETB ) const; // Betriebsart
	void OnTB_StartStopEnable ( const ETB ) const; // Start bzw. Stop
	void OnTB_MotionParamEnable ( const ETB ) const; // Bewegungsparameter
	void OnTB_PositionsEnable ( const ETB ) const; // Sollposition und Bildlaufleiste (Min-, Ist-, Maximalposition sind und bleiben gesperrt und ausgegraut)
	void OnTB_OffsetEnable ( const ETB ) const; // Offset, Relative Null setzen und aufheben

	// aktualisiert (Inhalt und Zustand) alle Steuerelemente auerhalb der Teilbereiche
	// "Psd-Offset" und "Halbwertsbreite messen"
	void OtherControls ( void ) const;

	//*****************************************************************************
	// Verwaltung der Teilbereiche
private:
	// ==> der Antrieb, der im angegebenen Teilbereich angezeigt wird; ggf. GetMotorCount() (d.h. "kein Antrieb")
	UINT TB2Selection ( const ETB ) const;

	// ==> Teilbereich, in dem der angebene Antrieb ausgewhlt wird; ggf. (ETB)0
	ETB Selection2TB ( const UINT ) const;

	// ==> SetSelection-Aufruf zulssig
	BOOL CanSetSelection ( const ETB, const UINT ) const;

	// whlt den angegebenen Antrieb im angegebenen Teilbereich aus und alle Steuerelemente im Teilbereich werden aktualisiert (Inhalt und Zustand)
	BOOL SetSelection ( const ETB, const UINT );

	// ==> Fensterhandle einer Ressource im Teilbereich; ggf. 0
	HWND GetCtrl ( const ETB, const ERessourceId ) const;

	// ==> Ressourcen-ID einer Ressource im Teilbereich; ggf. 0
	static int GetResId ( const ETB, const ERessourceId );

	//*****************************************************************************
	// zur Verwaltung der Teilbereiche
private:
	// der im ersten Parameter angegebenen Teilbereich wird um eins erhht und zurckgegeben
	// ist der erste Parameter "(TAxisType)0", wird der erste Teilbereich zurckgegeben
	// ==> FALSE <--> der bergebene Parameter kann nicht weiter inkrementiert werden (ist bereits der letzte Teilbereich)
	static BOOL TB_Next ( ETB& );

	// ==> Anzahl der Teilbereiche
	static UINT TB_Count ( void );

	// ==> Nummer des angegebenen Teilbereichs, beginnend bei 0
	static UINT TB_GetId ( const ETB );

	// gibt "Teilbereich<Nummer des Teilbereichs, beginnend bei 1>" in <aReturn> zurck;
	// Dort sollten mind. 13 Zeichen Platz reserviert sein!
	static void TB_GetName ( const ETB aTB, LPSTR aReturn );

	//*****************************************************************************
	// Winkelwerte in Position auf der Bildlaufleiste umrechnen
private:
	// Winkelwerte des Antriebs ins Intervall [0..32.000] der Bildlaufleiste einpassen und
	// die umgerechneten Werte in den letzten drei Parametern zurckgeben
	static void ScrollbarCalculate ( const TAngle, const TAngle, const TAngle,
									 int&, int&, int& );

	//*****************************************************************************
	// ATTRIBUTE
private:

	TManJustage *m_lnkManJustage; // die Funktionalitt
	TInterfaceTimer *m_lnkTimer; // POLLING-Timer
	UINT *m_Selection; // enthlt fr jeden Teilbereich, welcher Index zuletzt ausgewhlt war

	// Einstellungen zur Verbesserung des Reaktionsverhalens auf langsamen Rechnersystemen
	BOOL m_Parallel; // quasiparallele Nachrichtenverarbeitung?
	BOOL m_StateRefresh; // Steuerelementsperrung zulassen?
	BOOL m_DisplayNoActionMsg; // Meldung ausgeben, wenn die gewnschte Aktion nicht ausgefhrt werden kann
};

//##############################################################################
// TMotorOffsetDlg
//##############################################################################

class TMotorOffsetDlg : public TModalDlg
{ // Dialogfenster 'Offset fr <Antrieb>', Teil der Manuellen Justage

	//*****************************************************************************
	// Konstruktor-Destruktor
public:

	TMotorOffsetDlg ( const UINT, TManJustage* );

	//------------------------------------------------------------------------------
	// Ereignisse der Basisklasse
protected:

	BOOL Dlg_OnInit ( HWND, HWND, LPARAM );    // Initialisierungen
	void Dlg_OnCommand ( HWND, int, HWND, UINT ); // Eingabeverarbeitung

	//*****************************************************************************
	// Attribute
private:

	TManJustage *m_lnkManJustage;   // die Funktionalitt
	UINT m_Idx;             // Index des Antriebs, dessen Offset gesetzt werden soll
	UINT m_Digits;          // Nachkommastellen

	//*****************************************************************************
	// Methoden
private:

	void ChooseAngleMode ( BOOL );  // Parameter gibt an ob nun Winkel- (Parameter==TRUE) oder Offset-Eingabe (FALSE) erlaubt sind
	BOOL CalcValue ( BOOL );        // lokale Berechnung von Offset oder Winkel, bernahmen wenn Parameter==TRUE
	BOOL IsAngleMode ( void );      // welche Eingabemglichkeit besteht; TRUE <--> Winkeleingabe
};

//#############################################################################
//TPsdOffsetDlg
//#############################################################################

class TPsdOffsetDlg : public TModalDlg
{ // Dialogfenster 'Offset fr PSD', Teil der Manuellen Justage

	//*****************************************************************************
	// KONSTRUKTOR-DESTRUKTOR
public:

	TPsdOffsetDlg ( const UINT, TManJustage* );

  //*****************************************************************************
	// EREIGNISSE DER BASISKLASSE
protected:

	// Ereignisse der Basisklasse
	BOOL Dlg_OnInit ( HWND, HWND, LPARAM );
	void Dlg_OnCommand ( HWND, int, HWND, UINT );

	//*****************************************************************************
	// ATTRIBUTE
private:

	TManJustage *m_lnkManJustage; // die Funktionalitt
	UINT m_Idx;            // Index des Antriebs, dessen Offset gesetzt werden soll
	UINT m_Digits;         // Nachkommastellen
	UINT m_PSDDigits;      // Kanalbreite Nachkommastellen

	//*****************************************************************************
	// Methoden
private:

	BOOL CalcValue( BOOL );
};

#endif //__MJ_GUI.H

//#############################################################################
// LastLine
//#############################################################################