#ifndef __M_LAYER_H
#define __M_LAYER_H

#include "motrstrg\m_motcom.h"
#include "utils\u_utils.h"
#include "hardware\hwio.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

/*
 * High Level layer der Motoransteuerung.
 * Funktionen von ausserhalb sollten diese benutzen und nicht die
 * aus motors.h
 *
 * mFunctionName = funktion benutzt den aktivierten Motor
 * mlFunctionName= funktion benutzt den Motor mit der uebergebenen
 *                  MotorID (mid)
 * Achse= ein Motor, der eine bestimmte Achse bewegt
 *
 * Viele hier benutzte Typen werden in comhead.h definiert, auf
 * jeden Fall einmal anschauen.
 * In m_motcom.h stehen einige Kurzroutinen!!
 *
 * Kommentare: Jens-Uwe Rumstich, 18. Februar 1999
 *             Stefan Ltzkendorf, Februar 2001
 */

#if defined (Build_Motors)
#define _MOTORCLASS __declspec(dllexport)
#elif defined(Use_Motors)
#define _MOTORCLASS __declspec(dllimport)
#else
#define _MOTORCLASS
#endif

#ifdef __cplusplus
extern "C"
{
#endif
	// prototypes for export functions
	/*******************************************************************************
	 ******************** MotorListen-Funktionen ***********************************
	 ******************************************************************************/

	/* wurde Motors.DLL geladen und initialisiert?
	 * TRUE = ja,
	 * FALSE= no
	 */ 
	BOOL _MOTORCLASS WINAPI mlIsServerOK( void );

	/*******************************************************************************
	 ************************ Motor-Funktionen *************************************
	 ******************************************************************************/

	BOOL _MOTORCLASS WINAPI mSetLine( int, BOOL );

	/*******************************************************************************
	 ****** "MoveScan"-Funktionen **************************************************
	 *******************************************************************************
	 *
	 * Ein MoveScan setzt den Motor mit den aktuellen Parametern
	 * in Bewegung und tastet die Position in regelmigen Abstnden
	 * ab. Der Entstehende Vektor von Positionsangaben steht danach zur
	 * Verfgung.
	 *
	 */

	/* startet einen MoveScan fr den aktuellen Motor
	 */
	void _MOTORCLASS WINAPI mStartMoveScan( int, int );

	/*******************************************************************************
	 ************************ Motor-Simulation *************************************
	 ******************************************************************************/ 
	// bit 0 - hardware
	// bit 1 - simulation
	enum TSimulationType {
		no_simulation= 1,
		simulation_only= 2,
		test_simulation= 3
	};

	/* legt Simulationstyp fest
	 *
	 * t == no_simulation
	 *      keine Simulation, evtl. registierte callbacks werden 
	 *      ignoriert, es wird nur die Hardware angesteuert
	 * t == simulation_only
	 *      keine Ansteuerung der Hardware, es werden fr Schreib- und Lese-
	 *      Operationen die registirerten callbacks verwendet
	 * t == test_simulation
	 *      es wird erst die Hardware angesteuert und dann die ber die 
	 *      registierten callbacks die Simulation, Lese-Operationen liefern
	 *      zustzlich die Hardwarewerte fr den Vergleich mit der Simulation
	 */
	void _MOTORCLASS WINAPI msSetSimulationType(TSimulationType t);

	/* Typen fr callback-Funktionen zur Kommunikation mit einer
	 * Motornensimulation
	 * 
	 * GET-Funktionen sind Lese-Operationen aus Sicht der motors.dll
	 * PUT-Funktionen sind Schreib-Operationen 
	 *
	 * die Parameter der GET-funktionen sind
	 *   - die Quelle (addresse oder port)
	 *   - der Hardware-Vergleichs-Wert falls der 
	 *     Simulationstype test_simulation ist
	 * die GET-Funktionen liefern den von der Simulation 
	 * generierten Wert
	 *
	 * die Parameter der PUT-Funktionen sind
	 *   - Ziel der Schreiboperation (addresse oder port)
	 *   - der zu schreibende Wert
	 */
	typedef char (WINAPI *T812ISA_GET_CALLBACK) (char* addr, char hw_value);
	typedef void (WINAPI *T812ISA_PUT_CALLBACK) (char* addr, char put);
	typedef int (WINAPI *T832_GET_CALLBACK) (unsigned port, int hw_value);
	typedef void (WINAPI *T832_PUT_CALLBACK) (unsigned port, int put);

	/* Funktionen zum Registrieren oben beschriebener callback-Funktionen
	 */
	void _MOTORCLASS WINAPI msRegister_C812ISA_Get(T812ISA_GET_CALLBACK);
	void _MOTORCLASS WINAPI msRegister_C812ISA_Put(T812ISA_PUT_CALLBACK);
	void _MOTORCLASS WINAPI msRegister_C832_Get(T832_GET_CALLBACK);
	void _MOTORCLASS WINAPI msRegister_C832_Put(T832_PUT_CALLBACK);


#ifdef __cplusplus

}
#endif
 
/* verschiedene Type-Definitionen (Function-Casts)
 */
typedef BOOL (*TInitializeMotorsDLL)( void );
typedef BOOL (*TSetAxis)( int );
typedef int (*TGetAxis)( void );
typedef BOOL (*TSetLine)( int, BOOL );
typedef BOOL (*TIsMoveFinish)( void );
typedef BOOL (*TGetDistance)( double& );
typedef BOOL (*TIsServerOK)( void );
typedef BOOL (*TMoveToDistance)( double );
typedef BOOL (*TMoveByDistance)( double );
typedef BOOL (*TSetAxisWithName)( EAxisType );
typedef int (*TGetScanSize)( void );
typedef LPLONG (*TGetMoveScan)( void );
typedef void (*TStartMoveScan)( int, int );
typedef int (*TGetMoveFinishIdx)( void );
typedef void (*TSavePosition)( UINT, UINT, DWORD, DWORD, DWORD );

/* table with several functions
 */ 
struct TMFunctionTable {
public:
	TInitializeMotorsDLL InitializeMotorsDLL;
	TSetAxis SetAxis;
	TSetAxisWithName SetAxisWithName;
	TGetAxis GetAxis;
	TGetScanSize GetScanSize;
	TGetMoveScan GetMoveScan;
	TStartMoveScan StartMoveScan;
	TMoveToDistance MoveToDistance;
	TMoveByDistance MoveByDistance;
	TSetLine SetLine;
	TIsMoveFinish IsMoveFinish;
	TGetDistance GetDistance;
	TIsServerOK IsServerOK;
	TGetMoveFinishIdx GetMoveFinishIdx;
	TSavePosition SavePosition;
};


//liefert den der spezifizierten id entsprechenden Controller aus der spezifizierten
//Controllerliste -> instantiiert ggf. einen neuen Controller
Controller* GetController(EDeviceType DeviceID, LPTSTR HardwareID,
									 ControllerList* Controllers, DeviceList* Drivers, DWORD* index= 0);



#endif 
// __LastLine__


