//******************************************************************************
//Datei     : TProtocol.h
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Interface der Klasse TProtocol
//******************************************************************************
//!neu klier Protokollbuch
#ifndef _TPROTOCOL_H
#define _TPROTOCOL_H

#define MaxLine 1024
#define MaxLfdNr 15
#define MaxDate  15
#define MaxParameterString 161

class TProtocol
{
public:
	TProtocol( int );
	~TProtocol( void );

	// Initialisiert das Protokollbuch
	BOOL Initialize( void );

	// Initialisiert das Protokollbuch neu
	BOOL ReloadProtocol( void );

	// ist TRUE wenn eine Messung aufgezeichnet wird
	void SetProtocolOn( BOOL= TRUE );

	// liefert den Status ob eine Messung aufgezeichnet wird
	BOOL IsProtocolOn( void );

	// ndert den Status eine Messung die aufgezeichnet wird
	void ChangeProtocolOnStatus( void );

	// liefert den Status eine Messung die aufgezeichnet wird
	BOOL IsProtocolOnBegin( void );

	// Methoden zur Bearbeitung der Protokollparameter ----------------------------

	// Setzt den aktuellen Index auf NULL
	void ResetIndex( void );

	BOOL InitializeSearchProtocolString( LPCSTR );

	// sucht einen Protokollstring anhand der Lfd.-Nr. im Protokollindex
	// wird die Lfd.-Nr. gefunden liefert die Methode TRUE zurueck
	BOOL SearchProtocolString( LPCSTR );

	// loescht alle Felder des Parameterfeldes
	void ClearProtocolString( void );

	// loescht alle Felder von a bis b des Parameterfeldes
	void ClearProtocolString( int , int );

	// loescht alle Felder des Parameterfeldes und die Skizze
	void NewProtocolString( void );

	// loescht einige Felder des Parameterfeldes
	void CopyProtocolString( LPCSTR= NULL );

	// laed einen Protokollstring aus der Protokolldatei und fuelt das Parameterfeld
	// mit den entspraechenden Parametern
	BOOL LoadProtocolString( long );
	int LoadProtocolString( LPCSTR );
	int ReloadProtocolString( LPCSTR );

	// laed den naechsen Protokollstring aus der Protokolldatei und fuelt das
	// Parameterfeld mit den entspraechenden Parametern
	int LoadNextProtocolString( void );

	// laed den vorherigen Protokollstring aus der Protokolldatei und fuelt das
	// Parameterfeld mit den entspraechenden Parametern
	int LoadLastProtocolString( void );

	// speichert einen Protokollstring in der Protokolldatei
	BOOL SaveProtocolString( void );

	// speichert einen Protokollstring in der Protokolldatei
	BOOL ImportProtocolFile( void );

	// macht aus dem Systemdatum einen String
	LPSTR GetSysDate( LPSTR );

	// Prueft eine Zeichenkette ob sie ein gueltiges Datum enthaelt
	BOOL TestDate( LPCSTR );

	// der Wert eines Protokollparameters wird in das Protokollparameterfeld eingetragen
	LPSTR GetProtocolValue( int, LPSTR );

	// der Wert eines Protokollparameters wird aus dem Protokollparameterfeld ausgelesen
	BOOL SetProtocolValue( int, LPCSTR );

	// ist TRUE wenn die Protokolldatei leer ist
	BOOL IsEmptyProtocolFile( void );

	// ist TRUE wenn der aktuelle Protokollstring der Erste in der Protokolldatei ist
	BOOL IsFirstProtocolString( void );

	BOOL IsLastProtocolString( void );

	BOOL IsNewProtocolString( void );

	// die Variable fr die aktuelle Lfd.Nr. wird gesetzt
	void SetLfdNr( LPCSTR );

	// die Variable fr die aktuelle Lfd.Nr. wird ausgelesen
	LPSTR GetLfdNr( LPSTR );

	// speichert den aktuellen ProtokollDialog in einer Variable
	void SetCurrentDlg( int );

	// gibt den aktuellen Protokolldialog aus einer Variable zurck
	int GetCurrentDlg( void );

	// liefert den Namen eines Protokollparamerters zurck
	LPSTR GetParameterNamen( int, LPSTR );

	// liefert die maximale Lnge eines Protokollparamerters zurck
	int GetParameterMaxLen( int );

	// liefert den Status eines Protokollparamerters
	BOOL IsParameterReadOnly( int );

	// setzt den Status eines Protokollparamerters
	void SetParameterReadOnly( int );

	// setzt den Status eines Protokollparamerters
	void SetParameterReadWrite( int );

	// liefert den Status des aktuellen Protokollstrings
	BOOL IsProtocolReadOnly( void );

	// setzt den Status des aktuellen Protokollstrings readonly
	void SetProtocolReadOnly( void );

	// liefert den Status des aktuellen Protokollstrings
	BOOL IsProtocolDelete( void );

	// setzt den Status des aktuellen Protokollstrings delete
	void SetProtocolDelete( void );

	void SetAllParameterReadWrite( void );

	void SetAllParameterReadOnly( void );

	LPCSTR GetProtocolSection( void );
	BOOL IsProtocolSortDate( void );
	void SetProtocolSortDate( BOOL );
	BOOL IsProtocolNoSortNew( void );
	void SetProtocolNoSortNew( BOOL );

	// virtuelle Methode die erst in der Abgeleiteten Klasse definiert wird -------

	virtual BOOL IsMaxLfdNr( void )= 0;
	virtual BOOL TestLfdNr( LPCSTR )= 0;
	virtual void SaveLfdNr( LPCSTR )= 0;
	virtual LPSTR GetParameterLfdNr( LPSTR )= 0;
	virtual LPSTR GetParameterDate( LPSTR )= 0;
	virtual void SetParameterDate( LPCSTR )= 0;
	virtual LPSTR GetNewLfdNr( LPSTR )= 0;
	virtual void SetOnDefaultParameterReadOnly( void )= 0;
	virtual void SetDefaultParameterReadOnly( void )= 0;
	virtual void InitializeDefaultParameter( void )= 0;
	virtual void SetDefaultParameter( void )= 0;
	virtual void ClearOldParameter( void )= 0;

	virtual LPCSTR GetPBackupDirectory( void )= 0;
	virtual BOOL SetPBackupDirectory( LPCSTR )= 0;
	virtual LPCSTR GetPFile( void )= 0;
	virtual BOOL SetPFile( LPCSTR )= 0;
	virtual LPCSTR GetPName( void )= 0;
	virtual LPCSTR GetPDirectory( void )= 0;

	//Methoden zur Bearbeitung der Bilddateidaten ---------------------------------

	// der dynamische Speicher fr die Bildpunkte einer Skizze wird freigegeben
	void freePic();

	// die Variable fr den aktuellen Bildpunkt einer Skizze wird auf den ersten Bildpunkt gesetzt
	void ResetXY();

	// liefert einen Bildpunkt einer Skizze zurck
	// ist der letzte Bildpunkt erreicht wird der Rckgabewert der Funktion FALSE
	BOOL GetXY( char &c, int &x, int &y, LPSTR t );

	// belegt neuen dynamischen Speicher fr einen Bildpunkt einer Skizze und
	// initialisiert ihn mit den Wert eines Bildpunktes
	void SetXY( char, int, int, LPCSTR= "" );

	// loescht die letzten Bildpunkt einer Skizze
	BOOL UndoXY( void );

	// liest alle Bildpunkte einer Skizze aus einer Datei in den dynamischen Speicher
	// die Bildpunkte werden in einer verketteten Liste gespeichert
	BOOL LoadPicFile( LPCSTR );

	// speichert alle Bildpunkte einer Skizze in einer Datei ab
	BOOL SavePicFile( void );

	// lscht eine Bilddatei und gibt den Speicher fr die Bildpunkte frei
	void DeletePicFile( LPCSTR );

protected:
	// Methoden zur Bearbeitung der Protokollparameter ----------------------------

	// Zerlegt den eingelesenen Protokollstring und traegt die Parameter in das
	// Parameterfeld ein (Hochkomma am Anfang und am Ende eines Wertes werden entfernt)
	BOOL StrSplit( LPCSTR );

	// Erzeugt aus dem Parameterfeld einen Protokollstring
	// Enthaelt ein Parameter das Trennzeichen wird er in Hochkomma eingeschlossen
	LPSTR StrMerge( LPSTR );

	// Erzeugt aus einem Datum einen speziellen String fuer die Suche in der Protokolldatei
	// 20.4.1999 -->  19990420
	LPSTR MakeDateString( LPCSTR, LPSTR );

	// Erzeugt eine Backupkopie einer Datei
	BOOL BackupFile( LPCSTR, LPCSTR );

	// Sortiert die Protokolldatei, wenn sie nicht sortiert ist (bIsSort),
	// liefert TRUE wenn die Datei sortiert ist oder werden konnte
	// bIsSort wird von LoadProtocolFileIndex() oder AppendProtocolFileIndex() gesetzt
	BOOL SortProtocolFile( void );

	// Erzeugt aus der Protokolldatei einen Index im Speicher
	// Die Lfd.-Nr. und die Position des Datensatzes innerhalb der Datei werden
	// als Index im Speicher abgelegt.
	// setzt die Variable bIsSort auf FALSE, wenn die Datei nicht sortiert ist
	BOOL LoadProtocolFileIndex( LPCSTR= "" );
	BOOL AppendProtocolFileIndex( void );

	// Erzeugt aus einer Lfd.Nr. einen Namen fr eine Bilddatei (PATH + Dateiname)
	LPSTR GetSkizzeFile( LPCSTR );

	// Erzeugt aus einer Lfd.Nr. einen Namen fr eine Bildbackupdatei (PATH + Dateiname)
	LPSTR GetSkizzeBackupFile( LPCSTR );

	// Erzeugt einen Protokolldateinamen (PATH + Dateiname) aus mehreren Namensbestandteilen
	LPSTR GetProtocolFile( LPCSTR, LPCSTR= "", LPCSTR= "" );

	// Erzeugt einen Protokollbackupdateinamen (PATH + Dateiname) aus mehreren Namensbestandteilen
	LPSTR GetProtocolBackupFile( LPCSTR, LPCSTR= "", LPCSTR= "" );

	// Loescht den Index und gibt den Speicher frei
	void freeINDEX( LPCSTR= "" );

	// Variablen und Typen --------------------------------------------------------

	struct TParameter
	{
		char Wert[MaxParameterString];   // Wert des Parameters
		char Name[30];                   // Name des Parameters
		int MaxLen;                     // Maximale Lnge des Parameters
		int Status;                     // Status des Parameters (ReadWrite oder ReadOnly)
	}
	*Parameter;

	int Felder;                         //Felder entspricht der Anzahl der Parameter eines Protokollbucheintrages
	char CurrentLfdNr[MaxLfdNr];         // aktuelle Lfd.-Nr.
	char CopyLfdNr[MaxLfdNr];            // letzte kopierte Lfd.-Nr.
	int CurrentDlg;                     // aktueller Dialog
	char szProtocolFile[_MAX_PATH];        // Protokolldateiname
	char szProtocolSection[MaxString];   // Sektionsname in der INI-Datei

	struct TFileIndex
	{
		int Nr;                  // Nummer des Protokollbucheintrages
		long Pos;                 // Absolute Position innerhalb der Protokolldatei
		char LfdNr[MaxLfdNr];     // Lfd.-Nr. des Protokollbucheintrages
		char Date[MaxDate];       // Datum des Protokollbucheintrages
		int Status;              // Status des Protokollbucheintrages (ReadWrite oder ReadOnly)
		struct TFileIndex *last;  // Zeiger auf den vorherigen Protokollbucheintrag
		struct TFileIndex *next;  // Zeiger auf den nchsten Protokollbucheintrag
	}
	*firstINDEX, *lastINDEX, *currentINDEX;

	struct TMFPoint
	{
		char c;                   // Symbol fr eine Zeichenfunktion (M, L oder T)
		int x, y;                 // Koordinate fr eine Zeichenfunktion
		char t[MaxString / 4];      // Text der ausgegeben werden soll
		struct TMFPoint *last;    // Zeiger auf die vorherige Zeichenfunktion
		struct TMFPoint *next;    // Zeiger auf die nchste Zeichenfunktion
	}
	*firstPOINT, *lastPOINT, *currentPOINT;

	BOOL bActiv, bIsSortLfdNr, bIsSortDate, bNoSortNew, bSortDate, bDebug;
	int iSearch, iProtocolOnStatus;

	// Sortiert den Protokolldateiindex
	void QuickSortFileIndex( TFileIndex *, TFileIndex *, int );

	enum TStatus {ReadWrite= 0, ReadOnly, Deleted};
	enum TOnStatus {Off= 0, On, Begin= 1};
};

#endif //ndef _TPROTOCOL_H

// LastLine

