//#############################################################################
//                                                                           //
// U_TIMER.H                                                                 //
// Subsystem: Utilities - Timer                                              //
// Benutzung durch andere Subsysteme erforderlich: JA                         //
// Implementation: U_TIMER.CPP                                               //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   01.12.2002                                                       //
//                                                                           //
//#############################################################################

#ifndef __U_TIMER_H
#define __U_TIMER_H

#include "utils\u_values.h"
#include "utils\u_lists.h"
#include <mmreg.h> // <msacm.h>
#include <msacm.h> // MMRESULT
//ph

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// TBasicTimer
//##############################################################################

class _CURVECLASS TBasicTimer
{ // abstrakte Basisklasse fr Timer, das systemabhngige Verhalten wird in TTimer16 und TTimerMM erweitert
	//-----------------------------------------------------------------------------
	// Konstruktor-Destruktor
private:
	TBasicTimer ( TBasicTimer& ) {} // kein Copy-Konstruktor: leer implementiert

public:
	// initialisiert den Timer und setzt <aDelay> als Intervall [in Millisek.] in der das Event aufgerufen werden soll
	// der Timer wird noch NICHT gestartet
	TBasicTimer ( const UINT= 1000 );

	//-----------------------------------------------------------------------------
	// get/ set
public:
	// ==> Intervall [in Millisek.] in der das Event aufgerufen wird
	UINT GetDelay ( void );

	// setzt <aDelay> als Intervall [in Millisek.] in der das Event aufgerufen werden soll
	// wird erst beim nchsten StartTimer-Aufruf bernommen (oder wenn das Event das nchste Mal aufgerufen wird)
	// ==> <m_Delay> ODER 0
	UINT SetDelay ( const UINT= 1000 );

	// ==> TRUE <--> Timer erfolgreich gestartet
	virtual BOOL IsStarted ( void )= 0;

	//-----------------------------------------------------------------------------
	// start/ stop
public:
	// started den Timer
	// ==> TRUE <--> Timer erfolgreich gestartet
	BOOL StartTimer ( void );

	// started den Timer und setzt <aDelay> als Intervall [in Millisek.] in der das Event aufgrufen wird
	// ==> TRUE <--> Timer erfolgreich gestartet
	BOOL StartTimer ( const UINT ); // kein Defaultparameter da sonst in Konflikt zu StartTimer(), wo <m_Delay> verwendet wird

	// stoppt den Timer
	void StopTimer ( void );

	//-----------------------------------------------------------------------------
	// start/ stop ( + Ereignis sofort auslsen)
public:
	// siehe StartTimer ( void )
	BOOL StartTimerIm ( void );

	// siehe StartTimer ( const UINT )
	BOOL StartTimerIm ( const UINT ); // kein Defaultparameter da sonst in Konflikt zu StartTimer(), wo <m_Delay> verwendet wird

	// siehe StopTimer ( void )
	void StopTimerIm ( void );

	// von Hand auslsen
	void Immediately ( void );

	//-----------------------------------------------------------------------------
	// event
protected:
	// Event zum berschreiben
	virtual void OnTimer ( TBasicTimer *const )= 0;

	//-----------------------------------------------------------------------------
	// private
protected:
	UINT m_Delay;      // Intervall [in Millisek.] in der das Event aufgerufen wird ODER 0
	TIMECAPS m_Tc;     // Intervallgrenzen fr gltige Werte von <m_Delay>
	BOOL m_WantTimer;  // haben Anwender ODER TimedEvent den Timer gestoppt?
	//------------------------------------ start/ stop -----------------------------
private:
	virtual BOOL Start ( void )= 0;
	virtual void Stop ( void )= 0;
};

//##############################################################################
// TTimer16
//##############################################################################

// Kommentare siehe TBasicTimer
class _CURVECLASS TTimer16 : public TBasicTimer
{ // Win16 Anwendungen und Win3.11, 9x, Me
	//-----------------------------------------------------------------------------
	// Konstruktor-Destruktor
public:
	TTimer16 ( const UINT= 1000 );
	virtual  ~TTimer16 ( void );

	//-----------------------------------------------------------------------------
	// get/ set
public:
	virtual BOOL IsStarted ( void );

	//-----------------------------------------------------------------------------
	// start/ stop
private:
	virtual BOOL Start ( void );
	virtual void Stop ( void );

	//-----------------------------------------------------------------------------
	// ATTRIBUTE
protected:
	UINT m_TimerID;    // Identifikation des Timers

	//-----------------------------------------------------------------------------
	// STATIC
protected:
	static TIndList s_TimerList; // Liste der gestarteten Timer

private:
	// das Event das alle <m_Delay> Millisek. aufgerufen wird; muss static sein!!!
	static void CALLBACK TimedEvent( HWND, UINT, UINT, DWORD );
};

//##############################################################################
// TTimerMM
//##############################################################################

// Kommentare siehe TBasicTimer
class _CURVECLASS TTimerMM : public TBasicTimer
{ // Win32 Anwendungen und WinNT, 2k, XP (als allein verwendetes Betriebssystem!)
	//-----------------------------------------------------------------------------
	// Konstruktor-Destruktor
public:
	TTimerMM ( const UINT= 1000 );
	virtual  ~TTimerMM ( void );

	//-----------------------------------------------------------------------------
	// get/ set
public:
	virtual BOOL IsStarted ( void );

	//-----------------------------------------------------------------------------
	// start/ stop
private:
	virtual BOOL Start ( void );
	virtual void Stop ( void );

	//-----------------------------------------------------------------------------
	// ATTIBUTE
protected:
	MMRESULT m_TimerID; //Identifikation des Timers

	//-----------------------------------------------------------------------------
	// STATIC
private:
	//das Event das alle <m_Delay> Millisek. aufgerufen wird; muss static sein!!!
	static void CALLBACK TimedEvent( UINT, UINT, DWORD, DWORD, DWORD );
};

//##############################################################################
// TTimer
//##############################################################################

typedef TTimer16 TTimer; // Win32-Anwendungen oder Win98, WinMe, WinNT, Win2k, WinXP als Betriebsystem (unter Win3.1* und Win95: TTimerMM)

//##############################################################################
// ITimer
//##############################################################################

class TInterfaceTimer; // forward deklariert

class _CURVECLASS ITimer
{ //abstrakte Klasse bzw. Interface
public:
	virtual void OnTimer ( TBasicTimer *const )= 0;
};

//##############################################################################
// TInterfaceTimer
//##############################################################################

class _CURVECLASS TInterfaceTimer : public TTimer
{
	//-----------------------------------------------------------------------------
	// Konstruktor-Destruktor
public:
	TInterfaceTimer ( ITimer*, const UINT= 1000 );

	//-----------------------------------------------------------------------------
	// EREIGNIS(SE)
protected:
	virtual void OnTimer ( TBasicTimer *const );

	//-----------------------------------------------------------------------------
	// ATTRIBUTE
private:
	ITimer *m_lnkInterface;
};

//##############################################################################
// TWindowTimer
//##############################################################################

class _CURVECLASS TWindowTimer : public TTimer
{
	//-----------------------------------------------------------------------------
	// Konstruktor-Destruktor
public:
	TWindowTimer ( HWND, const UINT= 1000 );

	//-----------------------------------------------------------------------------
	// EREIGNIS(SE)
protected:
	virtual void OnTimer ( TBasicTimer *const );

	//-----------------------------------------------------------------------------
	// ATTRIBUTE
private:
	HWND m_lnkWnd;
};

#endif //__U_TIMER_H

//#############################################################################
// LASTLINE
//#############################################################################
