#include <fstream.h>
#include "motrstrg\msimstat.h"

#include "motrstrg\m_layer.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

static HINSTANCE theSimulationDLL;
static ofstream* theLogFile;

TMSimStatusWindow::TMSimStatusWindow(HINSTANCE aInstance) : TMDIWindow(aInstance)
{
	strcpy(m_Title, "MSIM-Status");
}

TMSimStatusWindow::~TMSimStatusWindow()
{}

void TMSimStatusWindow::OnCreate()
{
	// set a proper height and width
	SetWindowPos( GetHandle(), (HWND)0, 0, 0, 250, 400, 0 );
}

LRESULT TMSimStatusWindow::OnCommand(WPARAM wParam, LPARAM lParam )
{
	if ( (wParam == 1) && (lParam == 1) )
		InvalidateRect( GetHandle(), 0, TRUE); // gesendet von der MSIM.DLL (CVS Modul MSIM; Methode updateSimulationStatusWindow)
	return TMDIWindow::OnCommand( wParam, lParam );
}

void TMSimStatusWindow::DoPaint( HDC hdc, PAINTSTRUCT*)
{
	char* (WINAPI* getsimulationstatus)(void)= (char * (WINAPI*)(void)) GetProcAddress(theSimulationDLL, "GETSIMULATIONSTATUS");
	if ( !getsimulationstatus )
		return;

	char *buffer= getsimulationstatus();
	if ( !buffer )
		return;

	TEXTMETRIC tm;
	GetTextMetrics(hdc, &tm);
	UINT y= 0, w= 0;
	char *token= strtok( buffer, "\r\n");
	while (token)
	{
		int x= 0;
		TextOut(hdc, x, y, token, strlen(token));
		w= max(w, strlen(token) * tm.tmMaxCharWidth);
		token= strtok( NULL, "\r\n");
		y += tm.tmHeight;
	}
}


void InitializeMotorsSimulation(const char* ini_file)
{
	char buffer[100];
	TSimulationType sim_type;
	int logLevel;
	//geaendert klier
	//------------------------------------------------------
	OFSTRUCT of;
	char szFileName[_MAX_PATH];
	//------------------------------------------------------

	// check whether we need to initialize the simulation

	GetPrivateProfileString("MOTORSIM", "SimulationType", "no_simulation",
							buffer, 100, ini_file);
	if (stricmp(buffer, "no_simulation") == 0)
	{
		sim_type= no_simulation;
	}
	else if (stricmp(buffer, "test_simulation") == 0)
	{
		sim_type= test_simulation;
	}
	else if (stricmp(buffer, "simulation_only") == 0)
	{
		sim_type= simulation_only;
	}

	//geaendert klier
	//------------------------------------------------------
	HINSTANCE hLib= 0;

	// load simulation dll
	GetPrivateProfileString("MOTORSIM", "dll", "msim.dll",
							buffer, 100, ini_file);

	strcpy(szFileName, GetDirectory());
	strcat(szFileName, buffer);

	if (OpenFile(szFileName, &of, OF_EXIST) != HFILE_ERROR)
		hLib= LoadLibrary(buffer);
	//------------------------------------------------------

	if (hLib == 0)
	{
		// library not found, reset no_simulation
		sim_type= no_simulation;
		return;
	}

	msSetSimulationType(sim_type);

	// open logstream if the name is given in ini-file
	GetPrivateProfileString("MOTORSIM", "LogFile", "msim.log",
							buffer, 100, ini_file);
	logLevel= GetPrivateProfileInt("MOTORSIM", "LogLevel", 0, ini_file);
	if ( (strlen(buffer) > 0) && (logLevel > 0) )
	{
		theLogFile= new ofstream(buffer);

		void (WINAPI* setlogstream)(ostream*)= (void (WINAPI*)(ostream*))
												GetProcAddress(hLib, "SETLOGSTREAM");
		if (setlogstream)
			setlogstream(theLogFile);
	}


	// create the status window if required
	if ( sim_type != no_simulation &&
			GetPrivateProfileInt("MOTORSIM", "StatusWindow", 0, ini_file))
	{
		TMSimStatusWindow* stat_wnd= new TMSimStatusWindow(GetMainInstance());
		stat_wnd->Show();
		void (WINAPI* wndf)(HWND)= (void (WINAPI*)(HWND))
									GetProcAddress(hLib, "SETSIMULATIONSTATUSWINDOW");
		if (wndf)
			wndf(stat_wnd -> GetHandle()); // Bildschirmaktualisierung direkt an das Simulationsfenster senden
	}


	// register the callbacks provided by msim.dll
	if ( logLevel > 0)
	{
		switch (sim_type)
		{
			case no_simulation:
				msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
										GetProcAddress(hLib, "VERBOSE_C812ISA_GET_CALLBACK_NS") );
				msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
										GetProcAddress(hLib, "VERBOSE_C812ISA_PUT_CALLBACK") );
				msRegister_C832_Get( (T832_GET_CALLBACK)
									 GetProcAddress(hLib, "VERBOSE_C832_GET_CALLBACK_NS") );
				msRegister_C832_Put( (T832_PUT_CALLBACK)
									 GetProcAddress(hLib, "VERBOSE_C832_PUT_CALLBACK") );
				break;

			case test_simulation:
				msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
										GetProcAddress(hLib, "VERBOSE_C812ISA_GET_CALLBACK_TS") );
				msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
										GetProcAddress(hLib, "VERBOSE_C812ISA_PUT_CALLBACK") );
				msRegister_C832_Get( (T832_GET_CALLBACK)
									 GetProcAddress(hLib, "VERBOSE_C832_GET_CALLBACK_TS") );
				msRegister_C832_Put( (T832_PUT_CALLBACK)
									 GetProcAddress(hLib, "VERBOSE_C832_PUT_CALLBACK") );
				break;

			case simulation_only:
				msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
										GetProcAddress(hLib, "VERBOSE_C812ISA_GET_CALLBACK_SO") );
				msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
										GetProcAddress(hLib, "VERBOSE_C812ISA_PUT_CALLBACK") );
				msRegister_C832_Get( (T832_GET_CALLBACK)
									 GetProcAddress(hLib, "VERBOSE_C832_GET_CALLBACK_SO") );
				msRegister_C832_Put( (T832_PUT_CALLBACK)
									 GetProcAddress(hLib, "VERBOSE_C832_PUT_CALLBACK") );
				break;
		}
	}
	else
	{
		// no log requested, so we use silent callbacks, but only if we simulate
		if ( sim_type != no_simulation )
		{
			msRegister_C812ISA_Get( (T812ISA_GET_CALLBACK)
									GetProcAddress(hLib, "SILENT_C812ISA_GET_CALLBACK") );
			msRegister_C812ISA_Put( (T812ISA_PUT_CALLBACK)
									GetProcAddress(hLib, "SILENT_C812ISA_PUT_CALLBACK") );
			msRegister_C832_Get( (T832_GET_CALLBACK)
								 GetProcAddress(hLib, "SILENT_C832_GET_CALLBACK") );
			msRegister_C832_Put( (T832_PUT_CALLBACK)
								 GetProcAddress(hLib, "SILENT_C832_PUT_CALLBACK") );
		}
	}


	if ( sim_type != no_simulation )
	{
		void (WINAPI* initf)(const char*)= (void (WINAPI*)(const char*))
											GetProcAddress(hLib, "INITCONTROLLERFROMINIFILE");
		if (initf)
			initf(ini_file);
	}

	theSimulationDLL= hLib;
}


void UnInitializeMotorsSimulation()
{
	if (theSimulationDLL)
	{
		FreeLibrary(theSimulationDLL);
		theSimulationDLL= 0;
	}

	if (theLogFile)
	{
		theLogFile->close();
		_FREEOBJ(theLogFile);
	}
}
