//******************************************************************************
//Datei     : protocol.cpp
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Implementation der Ein- und Austrittspunkte der protocol.dll und
//            die zu exportierenden Variablen und Methoden
//           (Protokollbuchinterface fr das Messprogramm)
//******************************************************************************
//!neu klier Protokollbuch
#include "internls\evrythng.h"
#include "winresrc\rc_prtow.h" // Ressourcen-IDs
#include "help\help_def.h"     // Help-IDs

#include "utils\CheckMemory.h"	// CHECKMEM

// Bibliotheken zum Zugriff auf das Dateisystem (16/32bit)
// damit koennen die Dateiattribute der Protokolldatei
// geaendert werden
#include <winbase.h>

//! neu: klier Allgemeine Einstellungen
#include "mespara\mespara.h"

#include "protocol\TProtocolLanguage.h"
#include "protocol\TProtocol.h"
#include "protocol\protocol.h"
#include "protocol\TProtocolTopography.h"
#include "protocol\TProtocolDiffractometry.h"
#include "protocol\TProtocolDlg.h"
#include "protocol\TProtocolTopographyDlg.h"
#include "protocol\TProtocolDiffractometryDlg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

HINSTANCE hModuleInstance;

// Zeiger auf die Instanz der Allgemeine Einstellungen wird in m_mail.cpp
// initialisiert
TMeasurementParameter *MeasurementParameter;

// Fensterhandle des Programmfensters wird in m_mail.cpp initialisiert und
// fuer die Anzeige des Dialogfensters benoetigt
HWND MainHWND;


// Diese Klassenobjekte verwalten alle im jeweiligen Protokollbuch verwendeten
// Parameter einer Messung
TProtocolLanguage *ProtocolLanguage= NULL;
TProtocolTopography *ProtocolTopography= NULL;
TProtocolDiffractometry *ProtocolDiffractometry= NULL;


BOOL SetProtocolValue( TProtocol *Protocol, int Parameter, LPCSTR Value )
{
	if ( Protocol != NULL )
		if ( Protocol->IsProtocolOn() )
		{
			if (Protocol->SetProtocolValue(Parameter, Value))
			{
				Protocol->SetParameterReadOnly(Parameter);
				return TRUE;
			}
			Protocol->SetParameterReadWrite(Parameter);
		}
	return FALSE;
};

BOOL SetProtocolValue( TProtocol *Protocol, int Parameter, LPCSTR Format, float fValue )
{
	char buf[MaxString];

	if ( Protocol != NULL )
	{
		if ( Protocol->IsProtocolOn() )
		{
			sprintf(buf, Format, fValue);
			return SetProtocolValue(Protocol, Parameter, buf);
		}
	}
	return FALSE;
};

// Initialisiert die vom Protokollbuch benoetigten Variablen
void _PROTOCOLCLASS WINAPI InitializeProtocol( HWND hwnd, TMeasurementParameter *MeasPara )
{
	MainHWND= hwnd;
	MeasurementParameter= MeasPara;

	// Schreibschutzattribut der Protokolldatei wird gesetzt
	SetFileAttributes(GetPTopoFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);
	SetFileAttributes(GetPDiffFile(), FILE_ATTRIBUTE_ARCHIVE + FILE_ATTRIBUTE_READONLY);
};

// Funktion erzeugt einen ProtokollTopographieDialog und zeigt ihn an,
// beim ersten Aufruf wird das ProtocolTopography Objekt initialisiert
BOOL _PROTOCOLCLASS WINAPI ViewOnProtocolTopographyDlg()
{
	TModalDlg *dlg= 0;

	if ( ProtocolTopography != NULL )
	{
		dlg= (TProtocolTopographyDlg *)new TProtocolTopographyDlg("ProtocolTopographyDlg");
		if ( dlg ) dlg->ExecuteDialog(MainHWND);
		_FREEOBJ(dlg);
		return TRUE;
	}
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI ViewProtocolTopographyDlg()
{
	if ( !IsProtocolTopographyOn() && !IsProtocolDiffractometryOn() )
		return ViewOnProtocolTopographyDlg();
	return FALSE;
};

// Funktion erzeugt einen ProtokollDiffractometryDialog und zeigt ihn an,
// beim ersten Aufruf wird das ProtocolDiffractometry Objekt initialisiert
BOOL _PROTOCOLCLASS WINAPI ViewOnProtocolDiffractometryDlg()
{
	TModalDlg *dlg= 0;

	if ( ProtocolDiffractometry != NULL )
	{
		dlg= (TProtocolDiffractometryDlg *)new TProtocolDiffractometryDlg("ProtocolDiffractometryDlg");
		if ( dlg ) dlg->ExecuteDialog(MainHWND);
		_FREEOBJ(dlg);
		return TRUE;
	}
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI ViewProtocolDiffractometryDlg()
{
	if ( !IsProtocolTopographyOn() && !IsProtocolDiffractometryOn() )
		return ViewOnProtocolDiffractometryDlg();
	return FALSE;
};

// Diese Methoden werden von m_topo.cpp aufgerufen
// und setzen einen ProtokollTopographyParameter
//-------------------------------------------------------------

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyOn( void )
{
	if ( ProtocolTopography != NULL )
	{
		ProtocolTopography->SetProtocolOn();
		return TRUE;
	}
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyOff( void )
{
	if ( ProtocolTopography != NULL )
	{
		ProtocolTopography->SetProtocolOn(FALSE);
		return TRUE;
	}
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI IsProtocolTopographyOn( void )
{
	if ( ProtocolTopography != NULL )
		return ProtocolTopography->IsProtocolOn();
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyKKruemmung( float fValue ) //Maximale Anzahl Impulse
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->KKruemmung, "%.2f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyImpMax( float fValue ) //Maximale Anzahl Impulse
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->ImpMax, "%.0f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyImpsvor( float fValue )
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->Impsvor, "%.0f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyImpsnach( float fValue )
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->Impsnach, "%.0f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographySTE()
{
	//    return SetProtocolValue(ProtocolTopography, ProtocolTopography->STE, "?");
	return TRUE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyCONLIMIT()
{
	//    return SetProtocolValue(ProtocolTopography, ProtocolTopography->CONLIMIT, "?");
	return TRUE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyCRT( float fValue )
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->CRT, "%.0f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyHWB( float fValue )
{
	if (fValue != 0)
		return SetProtocolValue(ProtocolTopography, ProtocolTopography->HWB, "%.2f", fValue);
	else
		return TRUE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyINCDF( float fValue )
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->INCDF, "%.1f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyNBEXPOS( WORD nValue )
{
	char buf[MaxString];

	sprintf(buf, "%d", nValue);
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->NBEXPOS, buf);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyRSA( double dValue )
{
	char buf[MaxString];

	sprintf(buf, "%.1f", dValue);
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->RSA, buf);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyABE( float fValue )
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->ABE, "%.1f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyEXPT( long lValue ) //Belichtungszeit
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->EXPT, "%.1f", (lValue / 60l) / 60);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyCONPOINT( float fValue ) //Arbeitspunkt
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->CONPOINT, "%.0f", fValue*100);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyDFvor( float fValue )
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->DFvor, "%.2lf", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyDFnach( float fValue )
{
	return SetProtocolValue(ProtocolTopography, ProtocolTopography->DFnach, "%.2lf", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SaveProtocolTopographyParameter( void )
{
	if ( ProtocolTopography != NULL )
		if ( ProtocolTopography->IsProtocolOn() )
			return ProtocolTopography->SaveProtocolString();
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyReadOnly( void )
{
	if ( ProtocolTopography != NULL )
	{
		if ( ProtocolTopography->IsProtocolOn() && !ProtocolTopography->IsProtocolOnBegin() )
		{
			ProtocolTopography->SetProtocolReadOnly();
			return TRUE;
		}
	}
	return FALSE;
};
//-------------------------------------------------------------
// Diese Methoden werden von m_scan.cpp und m_arscan aufgerufen
// und setzen einen ProtokollDiffractometryParameter
//-------------------------------------------------------------
BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryOn( void )
{
	if ( ProtocolDiffractometry != NULL )
	{
		ProtocolDiffractometry->SetProtocolOn();
		return TRUE;
	}
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryOff( void )
{
	if ( ProtocolDiffractometry != NULL )
	{
		ProtocolDiffractometry->SetProtocolOn(FALSE);
		return TRUE;
	}
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI IsProtocolDiffractometryOn( void )
{
	if ( ProtocolDiffractometry != NULL )
		return ProtocolDiffractometry->IsProtocolOn();
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMessdatei( LPCSTR Value )
{
	char Drive[_MAX_DRIVE], Dir[_MAX_DIR], Name[_MAX_FNAME], Ext[_MAX_EXT], File[MaxString];

	_splitpath( Value, Drive, Dir, Name, Ext );
	sprintf( File, "%s%s", Name, Ext );
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Messdatei, File);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryNutzer( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Nutzer, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanart( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Scanart, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryOrientierung( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Orientierung, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryUReflex( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->UReflex, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySpannung( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Spannung, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryWellenlaenge( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Wellenlaenge, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScantyp( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Scantyp, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanmethode( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Scanmethode, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryStrom( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Strom, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanachse1( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Scanachse1, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySOffset1( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->SOffset1, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMinimum1( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Minimum1, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMaximum1( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Maximum1, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySchrittweite1( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Schrittweite1, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryGeschwindigkeit1( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Geschwindigkeit1, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanachse2( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Scanachse2, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySOffset2( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->SOffset2, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMinimum2( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Minimum2, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMaximum2( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Maximum2, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySchrittweite2( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Schrittweite2, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryGeschwindigkeit2( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Geschwindigkeit2, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryDetektor( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Detektor, Value);
};

/*
BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryDetektor( int iValue )
{
    if(iValue==1)
        return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Detektor, "PSD");
     else
        return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Detektor, "Counter");
};
*/

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMonitor( LPCSTR Value )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Monitor, Value);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMesszeit( float fValue )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Messzeit, "%.2f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryImpulse( DWORD dwValue )
{
	char buf[MaxString];

	sprintf( buf, "%lu", dwValue );
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Impulse, buf);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryWinkelbereich( float fValue )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Winkelbereich, "%.2f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryKanalabstand( float fValue )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Kanalabstand, "%.2f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMesskanal( float fValue )
{
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Messkanal, "%.0f", fValue);
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryAddiere( int iValue )
{
	char buf[MaxString];

	sprintf( buf, "%d", iValue );
	return SetProtocolValue(ProtocolDiffractometry, ProtocolDiffractometry->Addiere, buf);
};

BOOL _PROTOCOLCLASS WINAPI SaveProtocolDiffractometryParameter( void )
{
	if ( ProtocolDiffractometry != NULL )
		if ( ProtocolDiffractometry->IsProtocolOn() )
			return ProtocolDiffractometry->SaveProtocolString();
	return FALSE;
};

BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryReadOnly( void )
{
	if ( ProtocolDiffractometry != NULL )
	{
		if ( ProtocolDiffractometry->IsProtocolOn() && !ProtocolTopography->IsProtocolOnBegin() )
		{
			ProtocolDiffractometry->SetProtocolReadOnly();
			return TRUE;
		}
	}
	return FALSE;
};
//-------------------------------------------------------------


//Ein- und Austrittspunkte der protocol.dll
// Die Klassen ProtocolTopography und ProtocolDiffractometry werden erzeugt!

BOOL WINAPI DllMain(HINSTANCE hModule, DWORD dwFunction, LPVOID)
{
	switch (dwFunction)
	{
		case DLL_PROCESS_ATTACH:
			hModuleInstance= hModule;
			// Initialize function array.
			if ( ProtocolLanguage == NULL )
				ProtocolLanguage= new TProtocolLanguage();

			if ( ProtocolLanguage != NULL && ProtocolTopography == NULL )
				ProtocolTopography= new TProtocolTopography();

			if ( ProtocolLanguage != NULL && ProtocolDiffractometry == NULL )
				ProtocolDiffractometry= new TProtocolDiffractometry();
			break;

		case DLL_PROCESS_DETACH:
			_FREEOBJ(ProtocolTopography);
			_FREEOBJ(ProtocolDiffractometry);
			_FREEOBJ(ProtocolLanguage);
			//CHECKMEM("PROTOCOL.DLL");	//hp
			break;
	}
	return TRUE;
}

// __LastLine__


