//******************************************************************************
//Datei     : TProtocolDlg.cpp
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Implementation der Klasse TProtocolDlg
//******************************************************************************
//!neu klier Protokollbuch

#include "internls\evrythng.h"
#include "winresrc\rc_prtow.h" // Ressourcen-IDs
#include "help\help_def.h"     // Help-IDs
#include <commdlg.h>
#include <commctrl.h>

#include "protocol\TProtocolLanguage.h"
#include "protocol\TProtocol.h"
#include "protocol\TProtocolDlg.h"
#include "protocol\TProtocolManagePrintDlg.h"
#include "protocol\TProtocolPasswortDlg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

extern TProtocolLanguage *ProtocolLanguage;
TProtocolDlg *ProtocolDlg;
TProtocolManagePrintDlg *ProtocolManagePrintDlg;

//Datum: 27.07.2002 extern TModelessDlg* TheModeless;
extern TModelessDlg *ProtoPrintDlg; //! neu Kullmann+Reinecker

// Subclass procedure
LRESULT WINAPI SkizzeSubclassProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_PAINT)
	{
		ProtocolDlg->RefreshPic();
		ValidateRect(hwnd, NULL);
		return TRUE;
	}

	return CallWindowProc(ProtocolDlg->wpOrigProc, hwnd, uMsg, wParam, lParam);
}

TProtocolDlg::TProtocolDlg() : bOkStatus(ENABLE), bReadOnlyStatus(ENABLE)
{}


TProtocolDlg::~TProtocolDlg()
{
	Protocol->freePic();
	//! neu Kullmann+Reinecker: TheModeless durch ProtoPrintDlg ersetzt
	_FREEOBJ(ProtoPrintDlg);
}

// Aktivieren/Deaktivieren eines Dialogelementes
void TProtocolDlg::EnableDlgItem( int id , BOOL Status)
{
	EnableDlgItem(hwndDlg, id , Status);
};

// Aktivieren/Deaktivieren eines Dialogelementes
void TProtocolDlg::EnableDlgItem( HWND hwnd, int id , BOOL Status)
{
	EnableWindow( GetDlgItem(hwnd, id), Status);
};

// Sichtbarmachen/Verstecken eines Dialogelementes
void TProtocolDlg::ShowDlgItem( int id , int idShow)
{
	ShowDlgItem( hwndDlg, id , idShow);
};

// Sichtbarmachen/Verstecken eines Dialogelementes
void TProtocolDlg::ShowDlgItem( HWND hwnd, int id , int idShow)
{
	ShowWindow( GetDlgItem(hwnd, id), idShow );
};

// Ermittelt die Koordinaten des Rechteckes eines Dialogelementes
void TProtocolDlg::GetDlgItemRect( HWND hwnd, int id, LPRECT Rect )
{
	int dx, dy, dx1, dy1;
	RECT tmpRect;

	GetWindowRect(hwnd, &tmpRect);
	dx1= tmpRect.left;
	dy1= tmpRect.top;
	dx= tmpRect.right - tmpRect.left;
	dy= tmpRect.bottom - tmpRect.top;
	GetClientRect(hwnd, &tmpRect);
	dx= dx - tmpRect.right;
	dy= dy - tmpRect.bottom;
	dx= (dx / 2) + dx1;
	dy= (dy - (dx - dx1)) + dy1;
	GetWindowRect(GetDlgItem(hwnd, id), &tmpRect);
	Rect->left= tmpRect.left - dx;
	Rect->top= tmpRect.top - dy;
	Rect->right= tmpRect.right - dx;
	Rect->bottom= tmpRect.bottom - dy;
};

void TProtocolDlg::SaveDlg(HWND hwnd, int id)
{
	char buf[MaxString];

	if ( (id == ProbeSkizzeDlg) && (IsDlgButtonChecked(hwnd, id_ReadOnly) == 1) )
		Protocol->SetProtocolValue(GetStatusId(), itoa(1, buf , 10));
	ManageDlg(hwnd, id, Save);
};

void TProtocolDlg::RefreshDlg(HWND hwnd, int id)
{
	ManageDlg(hwnd, id, Refresh);
};

BOOL TProtocolDlg::TestDlg(HWND hwnd, int id)
{
	char buf[3*MaxParameterString];

	if (id == ProbeSkizzeDlg)
	{
		if (IsDlgButtonChecked(hwnd, id_ReadOnly) == 1)
		{
			sprintf(buf, ProtocolLanguage->GetMessage(20), "Passwortschutz",
					"Nicht Aktiv", "Aktiv");
			SetDlgItemText(hwnd, id_StatusLine, buf);
			MessageBox(GetFocus(), buf, ProtocolLanguage->GetWarning(), MBINFO);
			return FALSE;
		}
		if (bChangeSkizze)
		{
			SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(69));
			MessageBox(GetFocus(), ProtocolLanguage->GetMessage(69),
					   ProtocolLanguage->GetWarning(), MBINFO);
			return FALSE;
		}
	}
	return ManageDlg(hwnd, id, ChangeTest);
};

void TProtocolDlg::EnableDlg(HWND hwnd, int id, BOOL Status)
{
	if (Status == ENABLE)
		ManageDlg(hwnd, id, Enable);
	if (Status == DISABLE)
		ManageDlg(hwnd, id, Disable);
};

void TProtocolDlg::InitializeManagePrintDlg( TProtocolManagePrintDlg *ManagePrintDlg)
{
	ProtocolManagePrintDlg= ManagePrintDlg;
};

void TProtocolDlg::EnableDialog( BOOL Status)
{
	EnableDlg( hwndDlg, ProbeSkizzeDlg, Status );
	EnableDlg( hwndDlg, CurrentDlg, Status );
	EnableDlgItem( id_Next , Status );
	EnableDlgItem( id_Back , Status );
	EnableDlgItem( id_Copy , Status );
	EnableDlgItem( id_Edit, Status );
	if (Status == DISABLE)
	{
		EnableDlgItem( IDOK, Status );
		EnableDlgItem( id_ReadOnly, Status );
	}
	else
	{
		EnableDlgItem( IDOK, bOkStatus );
		EnableDlgItem( id_ReadOnly, bReadOnlyStatus );
	}
	EnableDlgItem( IDCANCEL, Status );
	EnableDlgItem( IDPRINT, Status );
	EnableDlgItem( id_Tab, Status );
	//! neu Kullmann+Reinecker: TheModeless durch ProtoPrintDlg ersetzt
	if ( ProtoPrintDlg )
		ProtocolManagePrintDlg->EnableButton( Status );
};

void TProtocolDlg::EnableOk( BOOL Status)
{
	bOkStatus= Status;
	EnableDlgItem( IDOK, Status );
};

void TProtocolDlg::EnableReadOnly( BOOL Status)
{
	bReadOnlyStatus= Status;
	EnableDlgItem( id_ReadOnly, Status );
};

BOOL TProtocolDlg::TestChangeDlg( )
{
	char lLfdNr[MaxString / 4], lLfdNr2[MaxString / 4];
	BOOL bTest;
	char buf[MaxString];

	bTest= FALSE;
	if (bDebug)
		if (strcmp(Protocol->GetLfdNr(lLfdNr), GetDlgItemText_LfdNr(hwndDlg, lLfdNr2)) != 0)
		{
			sprintf(buf, "currentLfdNr: %s != DlgLfdNr: %s", lLfdNr, lLfdNr2);
			MessageBox(GetFocus(), buf, "Debug Info", MBINFO);
		}
	if (!TestDlg(hwndDlg, ProbeSkizzeDlg) || !TestDlg(hwndDlg, CurrentDlg))
	{
		if (MessageBox(GetFocus(), ProtocolLanguage->GetMessage(18),
					   ProtocolLanguage->GetQuestion(),
					   MBASK) == IDYES)
		{
			bTest= TRUE;
			if (!TestPflichtDlg(hwndDlg))
				return bTest;
			if (Protocol->IsProtocolReadOnly())
				if (!GetPasswort(hwndDlg))
					return bTest;
			SaveDlg( hwndDlg, ProbeSkizzeDlg);
			SaveDlg( hwndDlg, CurrentDlg);
			Protocol->SavePicFile();
			Protocol->SaveProtocolString();
			LoadMeasurement(hwndDlg);
			SetDlgItemText(hwndDlg, id_StatusLine, ProtocolLanguage->GetMessage(62));
			if ( ProtoPrintDlg )
				ProtocolManagePrintDlg->SearchNew();
		}
	}
	return bTest;
};

BOOL TProtocolDlg::ManageDlgItem(HWND hwnd, int id, int Nr, TAction Action)
{
	char buf[MaxParameterString], buf2[MaxParameterString];
	char buf3[3*MaxParameterString], buf4[MaxString];
	HWND DlgHwnd;

	DlgHwnd= GetDlgItem( hwnd, id);
	switch (Action)
	{
		case DefaultTest:    // Prueft ob ein Pflichtfeld einen Wert enthaelt
			if ( IsWindowVisible(DlgHwnd) )
				GetDlgItemText(hwnd, id, buf, MaxParameterString);
			else
				Protocol->GetProtocolValue(Nr, buf);
			if ( strlen(buf) == 0 )
			{
				sprintf(buf, ProtocolLanguage->GetMessage(19),
						Protocol->GetParameterNamen(Nr, buf2));
				SetDlgItemText(hwnd, id_StatusLine, buf);
				MessageBox(GetFocus(), buf, ProtocolLanguage->GetFailure(), MBFAILURE);
				if ( !IsWindowVisible(DlgHwnd) )
				{
					SaveDlg( hwnd, CurrentDlg);
					ShowDialog( GetDlg(id), SHOW );
				}
				SetFocus(DlgHwnd);
				return FALSE;
			}
			if ((Nr == GetDatumId()) && !Protocol->TestDate(buf))
			{
				SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(58));
				MessageBox(GetFocus(), ProtocolLanguage->GetMessage(58),
						   ProtocolLanguage->GetFailure(), MBFAILURE);
				SetFocus(DlgHwnd);
				return FALSE;
			}
			break;

		case ChangeTest:    // Testet ein Feld ob es geaendert wurde
			GetDlgItemText(hwnd, id, buf, MaxParameterString);
			Protocol->GetProtocolValue( Nr, buf2);
			if ( strcmp(buf, buf2) != 0 )
			{
				sprintf(buf3, ProtocolLanguage->GetMessage(20),
						Protocol->GetParameterNamen(Nr, buf4),
						Protocol->GetProtocolValue(Nr, buf2), buf);
				SetDlgItemText(hwnd, id_StatusLine, buf3);
				MessageBox(GetFocus(), buf3, ProtocolLanguage->GetWarning(), MBINFO);
				return FALSE;
			}
			break;

		case Save:    // Speichert ein Feld und testet die zulaessige Parameterlaenge
			GetDlgItemText(hwnd, id, buf, MaxParameterString);
			if (!Protocol->SetProtocolValue(Nr, buf))
			{
				sprintf(buf3, ProtocolLanguage->GetMessage(21),
						Protocol->GetParameterNamen(Nr, buf2), buf,
						Protocol->GetParameterMaxLen(Nr));
				SetDlgItemText(hwnd, id_StatusLine, buf3);
				MessageBox(GetFocus(), buf3, ProtocolLanguage->GetFailure(), MBFAILURE);
			}
			break;

		case Refresh:    // Liest ein Feld aus dem Protokolldatensatz und zeigt es an
			EnableDlgItem(hwnd, id, ENABLE);
			Edit_LimitText(GetDlgItem(hwnd, id),
						   Protocol->GetParameterMaxLen(Nr));
			SetDlgItemText(hwnd, id, Protocol->GetProtocolValue( Nr, buf));
			if ( Protocol->IsParameterReadOnly(Nr) && ((strlen(buf) > 0) ||
					Protocol->IsProtocolOn()) || Protocol->IsProtocolDelete() )
				EnableDlgItem(hwnd, id, DISABLE);
			break;

		case Enable:    // Enabled ein Feld
			if ( !(Protocol->IsParameterReadOnly(Nr) && ((strlen(buf) > 0) ||
					Protocol->IsProtocolOn()) || Protocol->IsProtocolDelete()) )
				EnableDlgItem(hwnd, id, ENABLE);
			break;

		case Disable:    // Disabled ein Feld
			EnableDlgItem(hwnd, id, DISABLE);
			break;
	}
	return TRUE;
};

void TProtocolDlg::RefreshPic()
{
	char c, t[MaxString];
	int x, y;
	RECT Rect;

	SelectObject(hDC, hPenWhite);
	GetDlgItemRect( hwndDlg, id_Skizze, &Rect );
	for (int i= 0; i <= Rect.bottom - Rect.top; i++)
	{
		MoveToEx(hDC, 0, i, NULL);
		LineTo(hDC, Rect.right - Rect.left, i);
	}
	SelectObject(hDC, hPenBlack);

	Protocol->ResetXY();
	while (Protocol->GetXY( c, x, y, t ))
	{
		switch (c)
		{
			case 'M':
				MoveToEx(hDC, x, y, NULL);
				break;
			
			case 'L':
				LineTo(hDC, x, y);
				break;
			
			case 'T':
				TextOut(hDC, x, y, t, strlen(t));
				break;
			
			default:
				MessageBox(GetFocus(), ProtocolLanguage->GetMessage(23),
						   ProtocolLanguage->GetFailure(), MBFAILURE);
				Protocol->DeletePicFile(Protocol->GetLfdNr(t));
		}
	}
};


void TProtocolDlg::Dlg_OnInit(HWND hwnd, HWND hwndCtl, LPARAM lParam)
{
	HWND hwndSkizze;
	RECT Rect;
	char buf[MaxString];

	if (!Protocol->Initialize())
	{
		DestroyWindow(hwnd);
		return;
	}

	if ( GetPrivateProfileInt("Protocol", "Debug", 0, GetCFile()) == 1)
		bDebug= TRUE;
	else
		bDebug= FALSE;

	ProtocolDlg= this;

	ProtocolLanguage->InitializeDlgLanguage(hwnd);

	// Retrieve the handle of the edit control.
	hwndSkizze= GetDlgItem(hwnd, id_Skizze);
	// Subclass the edit control.
	wpOrigProc= (WNDPROC) SetWindowLong(hwndSkizze,
										 GWL_WNDPROC, (LONG) SkizzeSubclassProc );

	hwndDlg= hwnd;
	bLine= TRUE;
	LButtonDown= bEdit= FALSE;
	bLfdNrCorrect= TRUE;
	hDC= GetDC(GetDlgItem(hwndDlg, id_Skizze));
	hPenWhite= CreatePen(PS_SOLID, 1, RGB(255, 255, 255));
	hPenBlack= CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	SelectObject(hDC, hPenBlack);

	GetDlgItemRect( hwndDlg, id_EditText, &Rect );
	dxText= Rect.right - Rect.left;
	dyText= Rect.bottom - Rect.top;
	GetDlgItemRect( hwndDlg, id_Skizze, &Rect );
	MoveWindow(GetDlgItem( hwndDlg, id_Skizze ), Rect.left, Rect.top,
			   Rect.right - Rect.left, Rect.bottom - Rect.top, FALSE);

	hTab= GetDlgItem( hwnd, id_Tab );

	Protocol->InitializeDefaultParameter();
	InitializeDlgItems(hwnd);

	if ( Protocol->LoadProtocolString(Protocol->GetLfdNr(buf)) == 1 )
	{
		CurrentDlg= DefaultDlg;
		NewMeasurement(hwnd);
	}
	else
	{
		if (CurrentDlg == -1)
			CurrentDlg= Protocol->GetCurrentDlg();
		LoadMeasurement(hwnd);
	}
	ShowDialog(CurrentDlg, SHOW);
	SetFocus(GetDlgItem(hwnd, IDOK));
};

void TProtocolDlg::Dlg_OnDestroy( HWND hwnd )
{
	SetWindowLong(hwnd, GWL_WNDPROC, (LONG) wpOrigProc);
};

void TProtocolDlg::ShowLfdNr(LPCSTR lLfdNr)
{
	bLfdNrCorrect= TRUE;
	EnableDialog(ENABLE);
	if (Protocol->ReloadProtocolString(lLfdNr) != 0)
	{
		NewMeasurement(hwndDlg);
	}
	else
	{
		LoadMeasurement(hwndDlg);
	}
};

BOOL TProtocolDlg::GetPasswort(HWND hwnd)
{
	TModalDlg *dlg= 0;
	char buf[MaxString];

	GetPrivateProfileString( "Steuerprogramm", "PW", "", buf, MaxString, GetCFile( ) );
	strrev(buf);

	dlg= (TProtocolPasswortDlg *)new TProtocolPasswortDlg("ProtocolPasswortDlg", this);
	if ( dlg ) dlg->ExecuteDialog(hwnd);
	_FREEOBJ(dlg);

	if (strlen(buf) != 0)
		if (strcmp(buf, Passwort) == 0)
			return TRUE;
	SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(24));
	MessageBox(GetFocus(), ProtocolLanguage->GetMessage(24),
			   ProtocolLanguage->GetFailure(), MBFAILURE);
	return FALSE;
};

void TProtocolDlg::NewMeasurement( HWND hwnd )
{
	bChangeSkizze= FALSE;
	if (Protocol->IsEmptyProtocolFile())
		ShowDlgItem(id_Back, HIDE);
	else
		ShowDlgItem(id_Back, SHOW);
	ShowDlgItem(id_Next, HIDE);
	ShowDlgItem(id_Copy, HIDE);
	EnableOk(ENABLE);
	CheckDlgButton(hwnd, id_ReadOnly, 0);
	EnableReadOnly(DISABLE);
	SetDlgItemText_LfdNr(hwnd);
	Protocol->SetDefaultParameter();
	SetParameterStatus();
	RefreshDlg(hwnd, ProbeSkizzeDlg);
	RefreshPic();
	RefreshCombobox();
	RefreshDlg(hwnd, CurrentDlg);
	SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(63));
};

void TProtocolDlg::LoadMeasurement( HWND hwnd )
{
	char buf[MaxString];

	bChangeSkizze= FALSE;
	if ( Protocol->IsFirstProtocolString() )
		ShowDlgItem(id_Back, HIDE);
	else
		ShowDlgItem(id_Back, SHOW);
	if ( Protocol->IsLastProtocolString() )
		SetDlgItemText(hwnd, id_Next, "new");
	else
		SetDlgItemText(hwnd, id_Next, "-->");
	if (Protocol->IsProtocolOn() && !Protocol->IsProtocolOnBegin())
	{
		if (Protocol->IsLastProtocolString())
			ShowDlgItem(id_Next, HIDE);
		else
			ShowDlgItem(id_Next, SHOW);
		ShowDlgItem(id_Copy, HIDE);
	}
	else
	{
		ShowDlgItem(id_Next, SHOW);
		ShowDlgItem(id_Copy, SHOW);
	}
	if (Protocol->IsMaxLfdNr())
	{
		if (Protocol->IsLastProtocolString())
			ShowDlgItem(id_Next, HIDE);
		ShowDlgItem(id_Copy, HIDE);
	}
	SetDlgItemText_LfdNr(hwnd);
	Protocol->LoadPicFile(Protocol->GetLfdNr(buf));
	if (!Protocol->IsProtocolReadOnly())
		CheckDlgButton(hwnd, id_ReadOnly, 0);
	else
		CheckDlgButton(hwnd, id_ReadOnly, 3);
	if (Protocol->IsProtocolOn())
	{
		EnableReadOnly(DISABLE);
		if ( Protocol->IsLastProtocolString() && !Protocol->IsProtocolReadOnly() )
		{
			if ( Protocol->IsProtocolOnBegin() )
				Protocol->SetDefaultParameter();
			EnableOk(ENABLE);
		}
		else
			EnableOk(DISABLE);
	}
	else
		EnableReadOnly(ENABLE);
	SetParameterStatus();
	RefreshDlg(hwnd, ProbeSkizzeDlg);
	RefreshPic();
	RefreshCombobox();
	RefreshDlg(hwnd, CurrentDlg);
	SetDlgItemText(hwnd, id_StatusLine, "");
};

void TProtocolDlg::SetParameterStatus( )
{
	if (Protocol->IsProtocolOn())
		Protocol->SetOnDefaultParameterReadOnly();
	else if (Protocol->IsProtocolDelete())
		Protocol->SetAllParameterReadOnly();
	else if (Protocol->IsProtocolReadOnly())
		Protocol->SetDefaultParameterReadOnly();
	else
		Protocol->SetAllParameterReadWrite();
};

BOOL TProtocolDlg::Dlg_OnCommand(HWND hwnd, int id, UINT codeNotify)
{
	char buf[MaxString], buf2[MaxString];

	switch (id)
	{
		case id_Back:
			bLfdNrCorrect= TRUE;
			if (TestChangeDlg())
				break;
			Protocol->LoadLastProtocolString();
			LoadMeasurement(hwnd);
			break;
		
		case id_Next:
			bLfdNrCorrect= TRUE;
			if (TestChangeDlg())
				break;
			if (Protocol->LoadNextProtocolString() == 1)
				NewMeasurement(hwnd);
			else
				LoadMeasurement(hwnd);
			break;
		
		case id_Copy:
			bLfdNrCorrect= TRUE;
			if (TestChangeDlg())
				break;
			Protocol->CopyProtocolString();
			NewMeasurement(hwnd);
			break;
		
		case id_LfdNrBereich:
		case id_RTKNr:
		case id_LfdNr:
			if ( (codeNotify == EN_CHANGE) && IsDlgItemModify() )
			{
				if (bLfdNrCorrect)
				{
					bLfdNrCorrect= FALSE;
					if (id == id_LfdNr)
						if (TestChangeDlg())
						{
							SetDlgItemText_LfdNr(hwnd);
							RefreshDlg(hwnd, ProbeSkizzeDlg);
							RefreshPic();
							RefreshDlg(hwnd, CurrentDlg);
							break;
						}
				}
				EnableDialog( DISABLE );
				if ( TestDlgItemText_LfdNr(hwnd, buf) )
				{
					bLfdNrCorrect= TRUE;
					EnableDialog( ENABLE );
					if ( !Protocol->SearchProtocolString(buf) )
					{
						if (id == id_LfdNr)
						{
							Protocol->CopyProtocolString(buf);
							NewMeasurement(hwnd);
						}
					}
					else
					{
						Protocol->NewProtocolString();
						Protocol->LoadProtocolString(buf);
						LoadMeasurement(hwnd);
					}
				}
				else
				{
					SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(64));
				}
				SetFocus(GetDlgItem(hwndDlg, id));
			}
			break;
		
		case id_ReadOnly:
			if (Protocol->IsProtocolOn())
				break;
			switch (IsDlgButtonChecked(hwnd, id_ReadOnly))
			{
				case 0:
					if (Protocol->IsLastProtocolString())
					{
						if (MessageBox(GetFocus(), ProtocolLanguage->GetMessage(26),
									   ProtocolLanguage->GetQuestion(), MBASK) == IDNO)
							break;
					}
					else
					{
						if (MessageBox(GetFocus(), ProtocolLanguage->GetMessage(27),
									   ProtocolLanguage->GetQuestion(), MBASK) == IDNO)
							break;
					}
					CheckDlgButton(hwnd, id_ReadOnly, 1);
					break;
		
				case 1:
					CheckDlgButton(hwnd, id_ReadOnly, 0);
					break;
			}
			break;
		
		case id_OKEdit:
			GetDlgItemText(hwndDlg, id_EditText, buf, MaxString / 4);
			if (strlen(buf) > 0)
			{
			    bChangeSkizze= TRUE;
				Protocol->SetXY( 'T', xText, yText, buf );
			}
			ShowDlgItem( id_EditText, HIDE );
			ShowDlgItem( id_OKEdit, HIDE );
			ShowDlgItem( id_UnDo, SHOW );
			if (bLine)
			{
				ShowDlgItem( id_LineButton, HIDE );
				ShowDlgItem( id_LineText, SHOW );
				ShowDlgItem( id_PointText, HIDE );
				ShowDlgItem( id_PointButton, SHOW );
			}
			else
			{
				ShowDlgItem( id_PointButton, HIDE );
				ShowDlgItem( id_PointText, SHOW );
				ShowDlgItem( id_LineText, HIDE );
				ShowDlgItem( id_LineButton, SHOW );
			}
			ShowDlgItem( id_Loeschen, SHOW );
			ShowDlgItem( id_Cancel, SHOW );
			ShowDlgItem( id_Edit, SHOW );
			RefreshPic();
			break;
		
		case id_Loeschen:
			if (MessageBox(GetFocus(), ProtocolLanguage->GetMessage(25),
						   ProtocolLanguage->GetQuestion(), MBASK) == IDYES)
			{
				Protocol->freePic();
				RefreshPic();
				EnableDlgItem(id_UnDo, DISABLE);
			}
			break;
		
		case id_UnDo:
			bChangeSkizze= TRUE;
			if (!Protocol->UndoXY())
				EnableDlgItem(id_UnDo, DISABLE);
			RefreshPic();
			break;
		
		case id_PointButton:
			bLine= FALSE;
			ShowDlgItem( id_PointButton, HIDE );
			ShowDlgItem( id_PointText, SHOW );
			ShowDlgItem( id_LineText, HIDE );
			ShowDlgItem( id_LineButton, SHOW );
			SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(65));
			break;
		
		case id_LineButton:
			bLine= TRUE;
			ShowDlgItem( id_LineButton, HIDE );
			ShowDlgItem( id_LineText, SHOW );
			ShowDlgItem( id_PointText, HIDE );
			ShowDlgItem( id_PointButton, SHOW );
			SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(66));
			break;
		
		case id_Cancel:
			bEdit= FALSE;
			EnableDlgItem(id_UnDo, ENABLE);
			ShowDlgItem( id_UnDo, HIDE );
			ShowDlgItem( id_LineButton, HIDE );
			ShowDlgItem( id_PointText, HIDE );
			ShowDlgItem( id_PointButton, HIDE );
			ShowDlgItem( id_LineText, HIDE );
			ShowDlgItem( id_Loeschen, HIDE );
			ShowDlgItem( id_Cancel, HIDE );
			SetDlgItemText(hwnd, id_Edit, "Edit");
			Protocol->LoadPicFile(Protocol->GetLfdNr(buf));
			RefreshPic();
			EnableDialog( ENABLE );
			EnableDlgItem_LfdNr( ENABLE );
			SetDlgItemText(hwnd, id_StatusLine, "");
			break;
		
		case id_Edit:
			if (Protocol->IsProtocolDelete())
				break;
			if (bEdit)
			{
				bEdit= FALSE;
				EnableDlgItem(id_UnDo, ENABLE);
				ShowDlgItem( id_UnDo, HIDE );
				ShowDlgItem( id_LineButton, HIDE );
				ShowDlgItem( id_PointText, HIDE );
				ShowDlgItem( id_PointButton, HIDE );
				ShowDlgItem( id_LineText, HIDE );
				ShowDlgItem( id_Loeschen, HIDE );
				ShowDlgItem( id_Cancel, HIDE );
				SetDlgItemText(hwnd, id_Edit, "Edit");
				RefreshPic();
				EnableDialog( ENABLE );
				EnableDlgItem_LfdNr( ENABLE );
				SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(67));
			}
			else
			{
				bEdit= TRUE;
				SetDlgItemText(hwnd, id_Edit, "Ok");
				ShowDlgItem( id_UnDo, SHOW );
				if (bLine)
				{
					ShowDlgItem( id_LineButton, HIDE );
					ShowDlgItem( id_LineText, SHOW );
					ShowDlgItem( id_PointText, HIDE );
					ShowDlgItem( id_PointButton, SHOW );
				}
				else
				{
					ShowDlgItem( id_PointButton, HIDE );
					ShowDlgItem( id_PointText, SHOW );
					ShowDlgItem( id_LineText, HIDE );
					ShowDlgItem( id_LineButton, SHOW );
				}
				ShowDlgItem( id_Loeschen, SHOW );
				ShowDlgItem( id_Cancel, SHOW );
				RefreshPic();
				EnableDialog( DISABLE );
				EnableDlgItem_LfdNr( DISABLE );
				EnableDlgItem( id_Edit, ENABLE );
				SetDlgItemText(hwnd, id_StatusLine, ProtocolLanguage->GetMessage(68));
			}
			break;
		
		case id_Help:
			if (bEdit)
				WinHelp(hwnd, GetHelpFile(), HELP_CONTEXT, Help_ProtocolEditSkizze);
			else
				WinHelp(hwnd, GetHelpFile(), HELP_CONTEXT, Help_Protocol);
			break;
		
		case IDCANCEL:
			if (TestChangeDlg())
				break;
			if (Protocol->IsProtocolOn())
			{
				if (MessageBox(GetFocus(), ProtocolLanguage->GetMessage(22),
							   ProtocolLanguage->GetQuestion(), MBASK) == IDNO)
					break;
				else
					Protocol->SetProtocolOn(FALSE);
			}
			_FREEOBJ(ProtoPrintDlg);
			return FALSE;
		
		case IDOK:
			if ( bDebug )
			{
				if (strlen(Protocol->GetLfdNr(buf)) == 0)
					MessageBox(GetFocus(), "LfdNr leer bei Ok", "Debug Info", MBINFO);
				if (strcmp(GetDlgItemText_LfdNr(hwnd, buf2), buf) != 0)
					MessageBox(GetFocus(), "LfdNr != CurrentLfdNr bei Ok", "Debug Info", MBINFO);
			}
			if (!TestPflichtDlg(hwnd))
				break;
			if (Protocol->IsProtocolOn())
				Protocol->ChangeProtocolOnStatus();
			SaveDlgItems(hwnd);
			if (Protocol->IsProtocolReadOnly())
				if (!GetPasswort(hwnd))
					break;
			SaveDlg( hwnd, ProbeSkizzeDlg);
			SaveDlg( hwnd, CurrentDlg);
			Protocol->SavePicFile();
			Protocol->SaveProtocolString();
			SetDlgItemText(hwnd, id_StatusLine, "");
			SetStaticInfo("");
			_FREEOBJ(ProtoPrintDlg);
			return FALSE;
		
		default:
			return FALSE;
	}
	return TRUE;
};


void TProtocolDlg::Dlg_OnLButtonUp(HWND hwnd, int x, int y, UINT codeNotify)
{
	RECT Rect;

	if (bEdit)
	{
		if (LButtonDown)
		{
			LButtonDown= FALSE;
			if ( bLine )
			{
				GetDlgItemRect( hwndDlg, id_Skizze, &Rect );
				if ( (x > Rect.left) && (x < Rect.right) && (y > Rect.top) && (y < Rect.bottom) )
				{
					LineTo( hDC, x - Rect.left, y - Rect.top );
					Protocol->SetXY( 'L', x - Rect.left, y - Rect.top );
				}
			}
		}
	}
};


void TProtocolDlg::Dlg_OnLButtonDown(HWND hwnd, UINT codeNotify1, int x, int y, UINT codeNotify2)
{
	RECT Rect;

	if (bEdit)
	{
		if ( !LButtonDown )
		{
			GetDlgItemRect( hwndDlg, id_Skizze, &Rect );
			if ( (x > Rect.left) && (x < Rect.right) && (y > Rect.top) && (y < Rect.bottom) )
			{
				bChangeSkizze= TRUE;
				EnableDlgItem(id_UnDo, ENABLE);
				LButtonDown= TRUE;
				MoveToEx( hDC, x - Rect.left, y - Rect.top, NULL );
				Protocol->SetXY( 'M', x - Rect.left, y - Rect.top );
				if (bLine)
				{
					xMove= x - Rect.left;
					yMove= y - Rect.top;
				}
			}
		}
	}
};


void TProtocolDlg::Dlg_OnMouseMove(HWND hwnd, int x, int y, UINT codeNotify)
{
	RECT Rect;
	char buf[MaxString];

	if (bEdit)
	{
		GetDlgItemRect( hwndDlg, id_Skizze, &Rect );
		if ( (x > Rect.left) && (x < Rect.right) && (y > Rect.top) && (y < Rect.bottom) )
		{
			if ( LButtonDown )
				if ( bLine )
				{
					RefreshPic();
					MoveToEx( hDC, xMove, yMove, NULL );
					LineTo( hDC, x - Rect.left, y - Rect.top );
					sprintf(buf, " Linie: %d, %d --> %d, %d", xMove, yMove, x - Rect.left, y - Rect.top);
					SetDlgItemText(hwnd, id_StatusLine, buf);
				}
				else
				{
					LineTo( hDC, x - Rect.left, y - Rect.top );
					Protocol->SetXY( 'L', x - Rect.left, y - Rect.top );
					sprintf(buf, " Position: %d, %d", x - Rect.left, y - Rect.top);
					SetDlgItemText(hwnd, id_StatusLine, buf);
				}
		}
		else
			SetDlgItemText(hwnd, id_StatusLine, "");
	}
};


void TProtocolDlg::Dlg_OnRButtonDown(HWND hwnd, UINT codeNotify1, int x, int y, UINT codeNotify2)
{
	RECT Rect;
	int dx1, dx2;
	SIZE size;
	char buf[MaxString / 4];

	if ( bEdit )
	{
		if ( !LButtonDown )
		{
			GetDlgItemRect( hwndDlg, id_Skizze, &Rect );
			if ( (x > Rect.left) && (x < Rect.right) && (y > Rect.top) && (y < Rect.bottom) )
			{
				EnableDlgItem(id_UnDo, ENABLE);
				if ( (dyText + y) > Rect.bottom )
					y= Rect.bottom - dyText;
				xText= x - Rect.left;
				yText= y - Rect.top;
				dx1= dxText;
				if ( (dx1 + x) > Rect.right )
				{
					dx1= Rect.right - x;
					GetDlgItemText(hwnd, id_EditText, buf, MaxString / 4);
					GetTextExtentPoint(hDC, buf, strlen(buf), &size);
					while (size.cx > dx1)
					{
						buf[strlen(buf) - 1]= '\0';
						GetTextExtentPoint(hDC, buf, strlen(buf), &size);
					}
					SetDlgItemText(hwnd, id_EditText, buf);
				}
				MoveWindow(GetDlgItem( hwndDlg, id_EditText ), x, y, dx1, dyText, TRUE);
				GetDlgItemRect( hwndDlg, id_OKEdit, &Rect );
				dx2= Rect.right - Rect.left;
				MoveWindow(GetDlgItem( hwndDlg, id_OKEdit ), x + dx1, y, dx2, dyText, TRUE);
				ShowDlgItem( id_Loeschen, HIDE );
				ShowDlgItem( id_Edit, HIDE );
				ShowDlgItem( id_UnDo, HIDE );
				ShowDlgItem( id_LineButton, HIDE );
				ShowDlgItem( id_PointText, HIDE );
				ShowDlgItem( id_PointButton, HIDE );
				ShowDlgItem( id_LineText, HIDE );
				ShowDlgItem( id_Cancel, HIDE );
				ShowDlgItem( id_EditText, SHOW );
				ShowDlgItem( id_OKEdit, SHOW );
				SetFocus(GetDlgItem(hwnd, id_EditText));
			}
		}
	}
};


// LastLine

