// m_device.cpp
// (C) 1993,1994 by Heiko Damerow
// MPG AG "Rntgenbeugung an Schichtstrukturen"

#include "utils\u_utils.h"
#include "winresrc\rc_def.h" // Ressourcen-IDs
#pragma hdrstop

#include "swintrac\m_dlg.h" // fr TCounterWindow
#include "detecuse\detecuse.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

TCounterWindow *TCounterWindow::s_Singleton= 0;

//******************************************************************************
//******** Anwender-Schnittstelle **********************************************
//******** Methoden fr das Zhler-Fenster *************************************
TCounterWindow::TCounterWindow(HINSTANCE aInstance, TDetector *detectorParam) : TMDIWindow(aInstance), IField(0, 0.0) // 31.04.2004 MEMCORRECT (verwende IField), hMemG(0)
{
	if ( detectorParam == 0 )
		Sensor= TDetectorManager::DetectorManager().GetDetector();
	else
		Sensor= detectorParam;

	Sensor->SetCounterWnd(NULL);
	if ( s_Singleton ) _FREEOBJ(s_Singleton);
	s_Singleton= this;

	// Aussehen des Fensters festlegen
	bShowAsDigital= TRUE;
	bQuiteOpen= FALSE;
	eScaleType= sLinear;
	bAnalogSwitchOn= TRUE;
	bWindowWillBeOpen= TRUE;
	bWriteLogFile= FALSE;
	fData= 100.0;
	fThickness= 10.0;
	fHight= 200.0;
	// Gre der Historien-Schlange
	nBarCount= 40;
	nIdx= -1;
	fMaxIntensity= 14000.1f;
	hBrushRed= CreateSolidBrush(RGB(255, 0, 0));
	hBrushWhite= CreateSolidBrush(RGB(0, 0, 0));
	hBrushBlack= CreateSolidBrush(RGB(255, 255, 255));
	hBrushOC= CreateSolidBrush(RGB(0, 255, 127));
	hPen= CreatePen(PS_SOLID, 1, RGB(255, 255, 255));
	strcpy(FontName, "");

	//! erstellen der Meneintraege 
	hHoleMenu= LoadMenu(GetMainInstance(), MAKEINTRESOURCE(menu_Counter));
    // Fix fr PopupMens aus Ressourcen (Dummy Popup)
    TheMenu= GetSubMenu(hHoleMenu, 0); 
};

TCounterWindow::~TCounterWindow()
{	
	char buf[MaxString];
	RECT rc, rcm;

	if (bQuiteOpen)
	{
		Sensor->MeasureStop();
		Sensor->SetCounterWnd(NULL);
	}

	GetWindowRect(GetFrameHandle(), &rcm);
	GetWindowRect(GetHandle(), &rc);
	sprintf(buf, "%d", rc.left - rcm.left - 4);
	WritePrivateProfileString("Counter", "xo", buf, GetCFile());
	sprintf(buf, "%d", rc.top - rcm.top - 42);
	WritePrivateProfileString("Counter", "yo", buf, GetCFile());
	sprintf(buf, "%d", rc.right - rc.left);
	WritePrivateProfileString("Counter", "dx", buf, GetCFile());
	sprintf(buf, "%d", rc.bottom - rc.top);
	WritePrivateProfileString("Counter", "dy", buf, GetCFile());
	DeleteObject(hBrushWhite);
	DeleteObject(hBrushRed);
	DeleteObject(hBrushBlack);
	DeleteObject(hBrushOC);
	DeleteObject(hPen);

	s_Singleton= 0;

	/* 31.04.2004 MEMCORRECT (verwende IField) if ( hMemG ) HeapFree(GetProcessHeap(),0,hMemG);
	hMemG= 0;*/
};

BOOL TCounterWindow::CanOpen(void)
{
	char buf[MaxString];

	if (!TMDIWindow::CanOpen())
		return FALSE;

	if (Sensor->GetCounterWnd())
	{
		sprintf(buf, "Es ist bereits ein darstellendes Fenster geffnet!");
		MessageBox((LPSTR)buf, "Fehler", MBINFO);
		return FALSE;
	}

	return TRUE;
};

void TCounterWindow::OnCreate()
{
	int value;

	bQuiteOpen= TRUE;
	Sensor->MeasureStop();
	Sensor->SetCounterWnd( GetHandle() );
	Sensor->SetParameters();
	Sensor->MeasureStart();
	GetPrivateProfileString( "Counter", "font", "Times New Roman", FontName,
							 MaxString, GetCFile());
	nBarCount= GetPrivateProfileInt("Counter", "BarNumber", 40, GetCFile());

	WndRect.left= GetPrivateProfileInt("Counter", "xo", 500, GetCFile());
	WndRect.top= GetPrivateProfileInt("Counter", "yo", 400, GetCFile());
	value= GetPrivateProfileInt("Counter", "dx", 170, GetCFile());
	WndRect.right= WndRect.left + value;
	value= GetPrivateProfileInt("Counter", "dy", 50, GetCFile());
	WndRect.bottom= WndRect.top + value;


	// 31.04.2004 MEMCORRECT (verwende IField) hMemG= (HGLOBAL)HeapAlloc(GetProcessHeap(),0,nBarCount * sizeof(float));
	IField.SetCount( nBarCount );


	SetWindowPos( GetHandle(), NULL, WndRect.left, WndRect.top,
				  WndRect.right - WndRect.left, WndRect.bottom - WndRect.top, SWP_NOZORDER );
	SetTitle();
};

void TCounterWindow::OnSize(WPARAM, LPARAM)
{
	char buf[MaxString];

	if (bWindowWillBeOpen)
		bWindowWillBeOpen= FALSE;
	else
		GetWindowRect(GetHandle(), &WndRect);

	GetClientRect(GetHandle(), &ClientRect);

	sprintf(buf, "%d", nBarCount);

	WritePrivateProfileString("Counter", "BarNumber", buf, GetCFile());
};

BOOL TCounterWindow::SetTitle()
{
	strcpy(m_Title, "Zhler: ");
	strcat(m_Title, Sensor->GetCharacteristic());
	return TMDIWindow::SetTitle();
};

LRESULT TCounterWindow::OnCommand(WPARAM wParam, LPARAM lParam)
{
	TModalDlg *dlg;
	WPARAM Cmd;
	Cmd= GET_WM_COMMAND_ID(wParam, lParam);
	switch (Cmd)
	{
		case cm_ShowCounterParam:
			dlg= new TCounterShowParamDlg(this);
			if ( dlg ) dlg->ExecuteDialog(GetHandle());
			_FREEOBJ(dlg);
			return 0l;

		case cm_Logging:
			SetupLogging();
			return 0l;

		case cm_ShowDigital:
			bShowAsDigital= TRUE;
			return 0l;

		case cm_ShowAnalog:
			bShowAsDigital= FALSE;
			return 0l;
	}
	return TMDIWindow::OnCommand(wParam, lParam);
}

void TCounterWindow::CounterSetRequest(LPARAM)
{
	float temp= 0;
	int res= Sensor->GetIntegral(temp);
	if ( (res==R_OK) || (res==R_SoftOverflow) ) ShowCounterSetRequest(temp);
	else ShowCounterSetRequest(fData); // bei Hardoverflow
};

void TCounterWindow::ShowCounterSetRequest(float aIntegral)
{
	FILE *hFile;
	char buf[MaxString];
	fData= aIntegral;

	if (bWriteLogFile)
	{
		sprintf(buf, "device%d.log", Sensor->GetId());
		hFile= fopen(buf, "at");
		sprintf(buf, "%.3f\n", fData);
		fwrite(buf, 1, strlen(buf), hFile);
		fclose(hFile);
	}

	if (bShowAsDigital)
		UpdateWnd();
	else
		UpdateWnd( ptPaintPoint );

	if (!Sensor->GetControlWnd())
		Sensor->MeasureStart();
};

void TCounterWindow::SetupLogging(void)
{
	FILE *hFile;
	char buf[MaxString];

	SYSTEMTIME t;

	bWriteLogFile= !bWriteLogFile;

	if (bWriteLogFile)
	{
		GetLocalTime(&t);
		sprintf(buf, "device%d.log", Sensor->GetId());
		hFile= fopen(buf, "at");
		sprintf(buf, "Logging Detector%d starts at %02i.%02i.%i %2d:%02d\n",
				Sensor->GetId(), t.wDay, t.wMonth, t.wYear, t.wHour, t.wMinute);
		fwrite(buf, 1, strlen(buf), hFile);
		fclose(hFile);
	}
};

void TCounterWindow::OnPopupMenuInit(WPARAM, LPARAM lParam)
{
	TDetectorManager::DetectorManager().SetDetector(Sensor);
	CheckMenuItem(TheMenu, cm_ShowDigital, bShowAsDigital ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(TheMenu, cm_ShowAnalog, bShowAsDigital ? MF_UNCHECKED : MF_CHECKED);
	CheckMenuItem(TheMenu, cm_Logging, bWriteLogFile ? MF_CHECKED : MF_UNCHECKED);
};

void TCounterWindow::OnFocus(void)
{
	TDetectorManager::DetectorManager().SetDetector(Sensor);
};

void TCounterWindow::OnLButtonDown(WPARAM, LPARAM)
{};

void TCounterWindow::DoPaint(HDC hdc, PAINTSTRUCT *)
{
  char buf[MaxString];
  HDC hMemDC;
  HBITMAP hOldBitmap, hBitmap;
  HFONT hOldFont, hFont;
  int yp, xp;
  int xclient, yclient, i;
  LOGFONT lf;
  SIZE size;
  POINT point;

  if (!WholeWnd()) {
    if (bShowAsDigital) return;
    
    // Definition des logischen Zeichen-Fensters
    SetMapMode(hdc, MM_ANISOTROPIC);
    
    // Koordinatenursprung setzen
    SetWindowExtEx(hdc, ClientRect.right - ClientRect.left, ClientRect.bottom - ClientRect.top, &size);
    SetWindowOrgEx(hdc, 0, 0, &point);
    SetViewportExtEx(hdc, ClientRect.right - ClientRect.left, -(ClientRect.bottom - ClientRect.top), &size);
    SetViewportOrgEx(hdc, 0, ClientRect.bottom - ClientRect.top, &point);

    
    // 31.04.2004 MEMCORRECT (verwende IField) IField= (HPFLOAT)GlobalLock(hMemG);    
	
	SelectObject(hdc, hPen);
    SelectObject(hdc, hBrushRed);
    if (nIdx >= 0) Rectangle(hdc, nIdx*fThickness, IField[nIdx]*fHight, (nIdx + 1)*fThickness, 0);
    if (++nIdx >= nBarCount) nIdx= 0;
    
    IField[nIdx]= fData;
    
    // Zeichnen des letzten Wertes als Sule in schwarz
    SelectObject(hdc, hBrushBlack);
    
    Rectangle(hdc, nIdx*fThickness, ClientRect.bottom - ClientRect.top, (nIdx + 2)*fThickness, IField[nIdx]*fHight);
    SelectObject(hdc, hBrushWhite);
    Rectangle(hdc, nIdx*fThickness, IField[nIdx]*fHight, (nIdx + 2)*fThickness, 0);
    sprintf(buf, "Intensitt:  %.1f", IField[nIdx]);
    SetInfo(buf);


    // 31.04.2004 MEMCORRECT (verwende IField) GlobalUnlock(hMemG);


    return;
  }

  if (bShowAsDigital) {
    bAnalogSwitchOn= TRUE;
    xclient= ClientRect.right - ClientRect.left;
    yclient= ClientRect.bottom - ClientRect.top;
    SetMapMode(hdc, MM_TEXT);
    
    // MemDC erzeugen und auswhlen
    hMemDC= CreateCompatibleDC(hdc);
    hBitmap= CreateCompatibleBitmap(hdc, xclient, yclient);
    hOldBitmap= (HBITMAP)SelectObject(hMemDC, hBitmap);

    // auszugebende Zeichenkette erstellen
    if (fabs(fData) > 10000) sprintf(buf, "%.0f", fData);
    else if (fabs(fData) > 1000) sprintf(buf, "%.1f", fData);
    else sprintf(buf, "%.2f", fData);

    //Schriftskalierung berechnen (neu Kullmann+Reinecker)
    GetTextExtentPoint(hMemDC, buf, strlen(buf), &size);
    float scaleX= (float)xclient / max(1.0f, (float)size.cx); // DIV0 verhindern
    float scaleY= (float)yclient / max(1.0f, (float)size.cy); // DIV0 verhindern

    // Font erstellen und auswhlen
    memset(&lf, 0, sizeof(LOGFONT));
    lf.lfHeight= (int) ( size.cy*min(scaleX, scaleY) ); // neu Kullmann+Reinecker: yclient ersetzt durch Skalierung
    lf.lfWidth= 0;
    lf.lfWeight= 200;
    lf.lfUnderline= FALSE;
    strcpy(lf.lfFaceName, FontName);
    hFont= CreateFontIndirect(&lf);
    hOldFont= (HFONT)SelectObject(hMemDC, hFont);

    // Zeichnen der Daten
    GetTextExtentPoint(hMemDC, buf, strlen(buf), &size);
    xp= xclient - size.cx; // horizontal rechtsbndig
    yp= yclient / 2 - size.cy / 2; // vertikal zentiert
    TextOut(hMemDC, xp, yp, (LPSTR)buf, strlen(buf));    
    BitBlt(hdc, xp, yp, size.cx, size.cy, hMemDC, xp, yp, SRCCOPY); // PatBlt(hMemDC,0,0,xclient,yclient,WHITENESS);

    // DC wiederherstellen
    SelectObject(hMemDC, hOldFont);
    DeleteObject(hFont);
    SelectObject(hMemDC, hOldBitmap);
    DeleteObject(hBitmap);
    DeleteDC(hMemDC);
  } else { // !bShowAsDigital
    // 31.04.2004 MEMCORRECT (verwende IField) IField= (HPFLOAT)GlobalLock(hMemG);

	  
    if (bAnalogSwitchOn) {
      for (nIdx= 0;nIdx < nBarCount; nIdx++) IField[nIdx]= 0.0;      
      nIdx= -1;
      bAnalogSwitchOn= FALSE;
      fHight= (ClientRect.bottom - ClientRect.top) / fMaxIntensity * 0.9;
    };
    
    fThickness= (ClientRect.right - ClientRect.left) / nBarCount;
    SetMapMode(hdc, MM_ANISOTROPIC);
    SetWindowExtEx( hdc, ClientRect.right - ClientRect.left, ClientRect.bottom - ClientRect.top, &size);
    SetWindowOrgEx(hdc, 0, 0, &point);
    SetViewportExtEx( hdc, ClientRect.right - ClientRect.left, -(ClientRect.bottom - ClientRect.top), &size);
    SetViewportOrgEx(hdc, 0, ClientRect.bottom - ClientRect.top, &point);
    
    // Zeichnen der aktuellen Werte in Rot
    SelectObject(hdc, hPen);
    SelectObject(hdc, hBrushRed);
    for (i= 0;i < nBarCount;i++) Rectangle(hdc, i*fThickness, 0, (i + 1)*fThickness, IField[i]*fHight);

    
    // 31.04.2004 MEMCORRECT (verwende IField) GlobalUnlock(hMemG);
  }
};

//******************************************************************************
TCounterShowParamDlg::TCounterShowParamDlg(TCounterWindow *cwnd)
		: TModalDlg("CounterShowParam", GetMainInstance() )
{
	CounterWnd= cwnd;
};


BOOL TCounterShowParamDlg::Dlg_OnInit(HWND hwnd, HWND hwndCtl, LPARAM lParam)
{
	char buf[MaxString];

	SetFocus(GetDlgItem(GetHandle(), IDOK));
	CheckDlgButton(GetHandle(), id_ShowAsLog, (CounterWnd->eScaleType == sLogarithmic));
	sprintf(buf, "%.1f", CounterWnd->fMaxIntensity);
	SetDlgItemText(GetHandle(), id_MaxIntensity, (LPSTR)buf);
	sprintf(buf, "%u", CounterWnd->nBarCount);
	SetDlgItemText(GetHandle(), id_BarNumber, (LPSTR)buf);
	return TRUE;
};

BOOL TCounterShowParamDlg::CanClose(void)
{
	float value_f;
	short value_s;
	char buf[MaxString];

	if (!IsDlgButtonChecked(GetHandle(), id_ShowAsLog))
		CounterWnd->eScaleType= sLinear;
	else
		CounterWnd->eScaleType= sLogarithmic;

	GetDlgItemText(GetHandle(), id_MaxIntensity, (LPSTR)buf, 10);

	value_f= atof(buf);

	if ((value_f < 2e5) && (value_f > 0.0))
		CounterWnd->fMaxIntensity= value_f;
	else
	{
		CounterWnd->fMaxIntensity= 2.6e4;
		sprintf(buf, "%.1f", CounterWnd->fMaxIntensity);
		SetDlgItemText(GetHandle(), id_MaxIntensity, (LPSTR)buf);
		return FALSE;
	}

	GetDlgItemText(GetHandle(), id_BarNumber, (LPSTR)buf, 10);
	value_s= (short)atoi(buf);

	if (CounterWnd->nBarCount == value_s)
		return TRUE;

	if ((value_s < 140) && (value_s > 0))
	{
		CounterWnd->nBarCount= value_s;
		// nderung des belegten Speichers notwendig
		CounterWnd->Sensor->MeasureStop();


		// 31.04.2004 MEMCORRECT (verwende IField) HeapReAlloc(GetProcessHeap(),0, CounterWnd->hMemG, CounterWnd->nBarCount*sizeof(float));
		CounterWnd->IField.SetCount( CounterWnd->nBarCount );


		CounterWnd->nIdx= 0;
		CounterWnd->Sensor->MeasureStart();
		sprintf(buf, "%u", CounterWnd->nBarCount);
		SetDlgItemText(GetHandle(), id_BarNumber, (LPSTR)buf);
	}
	else
	{
		sprintf(buf, "%u", CounterWnd->nBarCount);
		SetDlgItemText(GetHandle(), id_BarNumber, (LPSTR)buf);
		return FALSE;
	}

	return TRUE;
};

// __LastLine__

