//##############################################################################
//                                                                            //
// TP_FUNK.CPP                                                                //
//                                                                            //
// Subsystem : Topographie / Funktionalitt                                   //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//----------------------------------------------------------------------------//
// Autoren:  Thomas Kullmann, Gnther Reinecker (2002)                        //
//                                                                            //
// Stand  : 05.12.2002                                                        //
//                                                                            //
//##############################################################################

#include <assert.h> // ::assert()

#include "topogrfy\tp_funk.h" // SCHNITTSTELLE fr diese Datei

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// externe Abhngigkeiten
//##############################################################################

__declspec(dllimport) LPMList lpMList; //Komposition aus TMotor-Objekten
extern TSteering Steering;
extern TMain Main; // in MAIN.CPP deklariertes Hauptprogramm

//##############################################################################
// TTopography
//##############################################################################

//------------------------------------------------------------------------------
// die einzige Instanz der Klasse

TTopography *TTopography::s_Instance= 0;
//------------------------------------------------------------------------------
// Erzeugung einer Singleton-Instanz

TTopography *TTopography::GetInstance( void )
{
	if ( s_Instance == 0 ) 
	{
		s_Instance= new TTopography;
		Main.AddToEventList(s_Instance); // neu! Kullmann+Reinecker (31.01.2004) Singleton-Objekt freigeben, wenn Hauptprogramm beendet wird		
	}
	return s_Instance;
};
//------------------------------------------------------------------------------
// Konstruktor

TTopography::TTopography( void ) : IMainEvents()  // neu! Kullmann+Reinecker (31.01.2004) Singleton-Objekt freigeben, wenn Hauptprogramm beendet wird
{
	char Ident[]= "Topography";

	m_fMoveStep= 0.3f;
	m_fMaxAngleEscape= 50;    // in Winkelsekunden
	m_lMeasurementTime= 600;   // in Sekunden
	m_nNumberCycle= 1;     // Anzahl der Mehrfachbelichtungen muss != 0
	// sein, da sonst Topographie Einstellungsfenster
	// nicht mit OK zu schliessen ist
	m_dStartAngle= -20.0f; // in Winkelsekunden
	m_fAngleBetweenShots= 3.0f;
	m_lnkDetector= 0;
	m_lnkMonitor= 0;
	m_Motor= -1;
	m_dwCurrentTime= m_dwStartTime= 0;

	m_bMultipleShot= FALSE;  // Mehrfachbelichtung
	m_bMonitorUsed= FALSE;
	m_bControlActive= FALSE;
	m_bTimeRunning= FALSE;
	m_bExceptionOccured= FALSE;
	m_bAdditionalTime= FALSE;

	float temp= (float)IniReadDouble(GetCFile(), Ident, "WorkPoint", -60, 0);
	m_fWorkPoint= fabsf(temp / 100); // anstatt 100.0 msste hier die Nachkommastellengenauigkeit des Antriebs bercksichtigt werden (100 == 2 Nachommastellen; 1000 == 3, usw.)
	m_bSmallAngleSide= (temp < 0);

	m_fControlRange= (float)IniReadDouble(GetCFile(), Ident, "ControlRange", 3.0, 0) / 100; // anstatt 100.0 msste hier die Nachkommastellengenauigkeit des Antriebs bercksichtigt werden (100 == 2 Nachommastellen; 1000 == 3, usw.)
	m_fControlStep= (float)IniReadDouble(GetCFile(), Ident, "ControlStep", 0.2, GetMotorDigits(eSF)); // Nachkommastellengenauigkeit fr Schrittweiten
	m_fMaxTime= (float)IniReadDouble(GetCFile(), Ident, "ExposureTime", 10.0, 10); // unbekannte Nachkommastellengeauigkeit
	m_dwMaxCounts= IniReadLong(GetCFile(), Ident, "ExposureCounts", 10000);
};
//------------------------------------------------------------------------------
// SET-Methoden
//------------------------------------------------------------------------------
// setzt den lokalen und den aktiven Motor

BOOL TTopography::SetMotor( EAxisType aAxis )
{
	for ( int i= 0; i < lpMList->GetAxisNumber(); i++ )
	{
		if ( aAxis == lpMList->ParsingAxis(lpMList->MP( i )->pCharacteristic()) )
		{
			if ( !lpMList->SetAxis( i ) )
				return FALSE; // setzt den aktiven Motor auf i
			m_Motor= i; // Motorenachse lokal setzen
			return TRUE;
		};
	};
	return FALSE;
};
//------------------------------------------------------------------------------
// setzt den lokalen und den aktiven Motor

BOOL TTopography::SetMotor( int aIndex )
{
	if ( !lpMList->SetAxis( aIndex ) )
		return FALSE;
	m_Motor= aIndex;
	return TRUE;
};
//------------------------------------------------------------------------------
// setzt die Schrittweite im aktiven Motor

BOOL TTopography::SetAngleWidth( float aWidth )
{
	if ( !lpMList->MP() )
		return FALSE;
	lpMList->MP()->SetAngleWidth( aWidth );
	return TRUE;
};
//------------------------------------------------------------------------------
// maximale Zaehlercounts

BOOL TTopography::SetMaxCounts( LONG aCount )
{
	if ( (aCount < 0) || (aCount > 999999l) )
		return FALSE; // Werte sind positiv, ganzahlig
	m_dwMaxCounts= aCount;
	return TRUE;
};
//------------------------------------------------------------------------------
// maximale Zaehlzeit des Detektors

BOOL TTopography::SetMaxTime( float aTime )
{
	// da Genauigkeit beim Lesen nur eine Nachkommastelle
	if ( (aTime < 0.1) || (aTime > 99999.0) )
		return FALSE;
	m_fMaxTime= aTime;
	return TRUE;
};
//------------------------------------------------------------------------------
// Lage des Arbeitspunktes relativ zur aktuellen Intensitaet

BOOL TTopography::SetWorkPoint( float aPoint )
{
	// da Genauigkeit beim Lesen nur eine Nachkommastelle
	if ( (aPoint < -9999.0) || ((aPoint < 0.1) && (aPoint > -0.1)) || (aPoint > 99999.0) )
		return FALSE;
	m_fWorkPoint= fabsf(aPoint / 100);
	return TRUE;
};
//------------------------------------------------------------------------------
// Anzahl der Belichtungszyklen, falls Mehrfachbelichtung angekreuzt ist

BOOL TTopography::SetNumberCycle( WORD aNumber )
{
	// Werte sind positiv, ganzahlig
	if ( (aNumber <= 0) || (aNumber > 65535l) )
		return FALSE;
	m_nNumberCycle= aNumber;
	return TRUE;
};
//------------------------------------------------------------------------------
// Belichtungszeit

BOOL TTopography::SetMeasurementTime( LONG aTime )
{
	// Werte sind positiv, ganzahlig
	if ( (aTime <= 0) || (aTime > 9999999l) )
		return FALSE;
	m_lMeasurementTime= aTime;
	return TRUE;
};
//------------------------------------------------------------------------------
// maximale Abweichung vom Arbeitspunkt

BOOL TTopography::SetMaxAngleEscape( float aAngle )
{
	// da Genauigkeit beim Lesen nur eine Nachkommastelle
	if ( (aAngle < 0.1) || (aAngle > 9999.0) )
		return FALSE;
	m_fMaxAngleEscape= aAngle;
	return TRUE;
};
//------------------------------------------------------------------------------
// Schrittweite zum Anfahren des Arbeitspunktes

BOOL TTopography::SetMoveStep( float aStep )
{
	// Anzahl der Nachkommastellen ist antriebsabhngig
	float min= 1 / (float)pow((float)10, (float)GetMotorDigits(eSF));
	if ( (aStep < min) || (aStep > 99999.0) )
		return FALSE;
	m_fMoveStep= aStep;
	return TRUE;
};
//------------------------------------------------------------------------------
// Regelschrittweite

BOOL TTopography::SetControlStep( float aStep )
{
	// Anzahl der Nachkommastellen ist antriebsabhngig
	float min= 1 / (float)pow((float)10, (float)GetMotorDigits(eSF));
	if ( (aStep < min) || (aStep > 9999999.0) )
		return FALSE;
	m_fControlStep= aStep;
	return TRUE;
};
//------------------------------------------------------------------------------
// Abstand bei Mehrfachbelichtungen

BOOL TTopography::SetAngleBetweenShots( float aAngle )
{
	// Anzahl der Nachkommastellen ist antriebsabhngig
	float min= 1 / (float)pow((float)10, (float)GetMotorDigits(eDF));
	if ( (aAngle < -9999.0) || ((aAngle < min) && (aAngle > (( -1)*min))) || (aAngle > 99999.0) )
		return FALSE;
	m_fAngleBetweenShots= aAngle;
	return TRUE;
};
//------------------------------------------------------------------------------
// ist bei Mehrfachbelichtung die relative Startposition zur aktuellen Motorposition

BOOL TTopography::SetStartAngle( double aAngle )
{
	if ( (aAngle < -9999.0) || (aAngle > 99999.0) )
		return FALSE;
	m_dStartAngle= aAngle;
	return TRUE;
};
//------------------------------------------------------------------------------
// Regelbereich nach Intensitaet in Prozent

BOOL TTopography::SetControlRange( float aRange )
{
	// da Genauigkeit beim Lesen nur zwei Nachkommastellen
	if ( (aRange < (float)0.01) || (aRange > 9999.0) )
		return FALSE;
	m_fControlRange= aRange / 100.0f;
	return TRUE;
};
//------------------------------------------------------------------------------
// GET-Methoden
//------------------------------------------------------------------------------
// gibt den Index des aktuellen Motors zurck

int TTopography::GetActMotor( void )
{
	int Index= lpMList->GetAxis(); // gibt den Index des aktiven Motor zurck
	if ( !lpMList->MP( Index ) )
		return -1; // bei Fehler ungltigen Index zurckgeben
	return Index;
};
//------------------------------------------------------------------------------
// gibt Zeiger auf den aktuellen Detektor zurck

TDetector *TTopography::GetActDetector( void )
{
	return TDetectorManager::DetectorManager().GetDetector();
};
//------------------------------------------------------------------------------
// Rckgabe der standardisierter Antriebsbezeichnung fr einen Motor

LPCSTR TTopography::GetMotorName( int aIndex )
{
	TMotor *Mtr= lpMList->MP( aIndex );
	if ( !Mtr )
		return ""; // Motorliste muss stets gltiges Motorobjekt zurckgeben
	return Mtr->pCharacteristic();
};
//------------------------------------------------------------------------------
// Rckgabe der Einheit des aktuellen Motors

LPCSTR TTopography::GetMotorUnit( void )
{
	return lpMList->MP()->GetUnitName();
};
//------------------------------------------------------------------------------
// gibt die Anzahl der Motoren zurck

int TTopography::GetMotorCount( void )
{
	return lpMList->GetAxisNumber();
};
//------------------------------------------------------------------------------
// gibt die Anzahl der Nachkommastellen des aktuellen Motors zurck

UINT TTopography::GetMotorDigits( EFormat aFormat )
{
	if ( aFormat == eSF )
		return lpMList->MP()->GetDigits() + 1;
	else
		return lpMList->MP()->GetDigits();
};
//------------------------------------------------------------------------------
// gibt die Winkelposition des aktuellen Motors zurck

double TTopography::GetAngle( void )
{
	if ( !lpMList->MP() )
		return 0.0;
	return lpMList->MP()->GetAngle();
};
//------------------------------------------------------------------------------
// Rckgabe eines Zahlenformats mit Digits oder Digits+1 Nachkommastellen des aktuellen Motors

LPCSTR TTopography::GetDigitFormat( EFormat aFormat)
{
	if ( aFormat == eSF )
		return lpMList->MP()->pSF( );
	return lpMList->MP()->pDF();
};
//------------------------------------------------------------------------------
// Lesen und Setzen von Attributen der Ablaufsteuerung
//------------------------------------------------------------------------------

void TTopography::SetStrngStartPoint ( double aPoint )
{
	Steering.dStartPoint= aPoint;
};
//------------------------------------------------------------------------------

double TTopography::GetStrngStartPoint( void )
{
	return Steering.dStartPoint;
};
//------------------------------------------------------------------------------

double TTopography::GetStrngDistance( void )
{
	return Steering.GetDistance();
};
//------------------------------------------------------------------------------

BOOL TTopography::DetectorRequest( void )
{
	return Steering.DetectorRequest();
};
//------------------------------------------------------------------------------

float TTopography::GetStrngPeakIntensity( void )
{
	return Steering.fPeakIntensity;
};
//------------------------------------------------------------------------------

float TTopography::GetStrngStartIntensity( void )
{
	return Steering.fStartIntensity;
};
//------------------------------------------------------------------------------

float TTopography::GetStrngIntensity( void )
{
	return Steering.GetIntensity();
};
//------------------------------------------------------------------------------

double TTopography::GetStrngHwb( void )
{
	return Steering.GetHwb();
};
//------------------------------------------------------------------------------
// DO-Methoden
//------------------------------------------------------------------------------
// Initialisierung von Detektor und Monitor

BOOL TTopography::DoInitDetectors( )
{
	if ( !m_lnkDetector )
	{
		SetDetector( GetActDetector() );
		SetMonitor( m_lnkDetector );
		SetMonitorUsed( FALSE );
	};
	if ( !m_lnkDetector )
		return FALSE;
	return TRUE;
};
//------------------------------------------------------------------------------
// Initialisierung des Motors (bevorzugt Omega)

BOOL TTopography::DoInitMotor( void )
{
	if ( (!DoInitMotor( Omega, TRUE )) && (!DoInitMotor( DF, TRUE )) )
	{
		DoInitMotor( (EAxisType)GetActMotor(), TRUE );
		return FALSE; // Omega ist nicht verfgbar
	}
	return TRUE;
};
//------------------------------------------------------------------------------
// Initialisierung des Motors

BOOL TTopography::DoInitMotor( EAxisType aType, BOOL aSetup )
{
	if ( m_Motor > -1 )
		return TRUE; // Motor bereits erfolgreich gesetzt

	if ( !SetMotor(aType) )
		return FALSE;
	if ( aSetup )
		DoSaveMotorSettings(); // Werte des aktuellen Motors in Settings Struktur eintragen
	return TRUE;
};
//------------------------------------------------------------------------------
// Ablaufsteuerung initialisieren

void TTopography::DoInitWorkPoint( void )
{
	Steering.Visualising( TRUE, FALSE ); // Reportausgabe fuer Steering= true
	m_lnkMWorkPoint= Steering.GetMacroByName("SetupTopography"); // Arbeitspunkteinstellung holen und anpassen
	if ( m_bSmallAngleSide )
		m_lnkMWorkPoint->CmdList[1].P1= SmallSide;
	else
		m_lnkMWorkPoint->CmdList[1].P1= LargeSide;
	sprintf(m_lnkMWorkPoint->CmdList[1].P3, GetDigitFormat(eDF), fabs(m_fWorkPoint));
};
//------------------------------------------------------------------------------
// Durchfhrung einer Messung mittels Kommandoverarbeitung

BOOL TTopography::DoStartMeasure ( ECmdId aCmdId, int p1, int p2, LPSTR aStr, HWND aControlWnd )
{
	Steering.Reset();
	Steering.StartUp(aControlWnd, m_Motor, m_lnkDetector);
	return Steering.StartCmdExecution( aCmdId, p1, p2, aStr, aControlWnd);
};
//------------------------------------------------------------------------------
// Durchfhrung einer Messung mittels Macroverarbeitung

BOOL TTopography::DoStartMeasure ( TMacroTag *aTag, HWND aControlWnd)
{
	Steering.Reset();
	Steering.StartUp(aControlWnd, m_Motor, m_lnkDetector);
	return Steering.StartMacroExecution( aTag, aControlWnd);
};
//------------------------------------------------------------------------------
// Unterbrechung einer Messung

BOOL TTopography::DoStopMeasure()
{
	return Steering.ToggleInterrupt();
};
//------------------------------------------------------------------------------

BOOL TTopography::DoSaveMonitorSignal()
{
	if ( !m_bMonitorUsed )
		return FALSE;

	float norm;
	m_lnkMonitor->GetIntegral(norm); // Messsignal normieren
	Steering.SetNorm(norm); // Messwerte speichern
	return TRUE;
};
//------------------------------------------------------------------------------
// Settings des aktuellen Motors sichern

BOOL TTopography::DoSaveMotorSettings ( void )
{
	return (lpMList->MP( GetActMotor() )->PushSettings());
};
//------------------------------------------------------------------------------
// Settings des aktuellen Motors zurckschreiben

BOOL TTopography::DoLoadMotorSettings( void )
{
	return (lpMList->MP( GetActMotor() )->PopSettings( ThisPosition ));
};
//------------------------------------------------------------------------------
// neu! Kullmann+Reinecker (31.01.2004) Singleton-Objekt freigeben, wenn Hauptprogramm beendet wird

void  TTopography::OnLeave ( void ) 
{
	if ( s_Instance ) {
		Main.RemoveFromEventList(s_Instance);
		_FREEOBJ(s_Instance);		
	}
};

//------------------------------------------------------------------------------
// setzt Parameter fr die Belichtung

void TTopography::DoSetDetectorParams( float aTime, DWORD aCount, BOOL Start )
{
	if ( (m_lnkDetector->IsActive()) || (Start) )
	{
		m_lnkDetector->MeasureStop();
		m_lnkDetector->SetExposureSettings( TExposureSettings( aTime, aCount ) );
		m_lnkDetector->MeasureStart();
	}
	else
		m_lnkDetector->SetExposureSettings( TExposureSettings( aTime, aCount ) );
};
//------------------------------------------------------------------------------
// lschen die im Detektor gespeicherten Daten zur Topographie

void TTopography::DoResetDetectorParams( void )
{
	GetDetector()->MeasureStop();
	GetDetector()->PopSettings();
	GetDetector()->SetControlWnd( 0 );
	GetDetector()->MeasureStart();
	if ( IsSetupOk() )
		SetDetector( 0 );
};
//------------------------------------------------------------------------------
// __LastLine__
