//#############################################################################
//                                                                           //
// U_TIMER.CPP                                                               //
// Subsystem: Utilities - Timer                                              //
// Deklaration: U_TIMER.H                                                    //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   01.12.2002                                                       //
//                                                                           //
//#############################################################################

#include <stdlib.h> // fr: min(), max()

#include "utils\u_timer.h" // SCHNITTSTELLE fr diese Datei

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
//                                     TBasicTimer                            //
//##############################################################################

TBasicTimer::TBasicTimer ( const UINT aDelay )
{
	//Intervallgrenzen ermitteln
	if (timeGetDevCaps( &m_Tc, sizeof( TIMECAPS ) ) != TIMERR_NOERROR)
	{
		m_Tc.wPeriodMin= 0;
		m_Tc.wPeriodMax= 0;
	}

	m_Delay= max( m_Tc.wPeriodMin, min( aDelay, m_Tc.wPeriodMax )); //ggf. 0 !!!
	m_WantTimer= FALSE;
};

//------------------------------------------------------------------------------

UINT TBasicTimer::GetDelay ( void )
{
	return m_Delay;
};
//------------------------------------------------------------------------------

UINT TBasicTimer::SetDelay ( const UINT aDelay )
{
	if ( m_Delay )
		return 0; //wenn Intervallgrenzen nicht ermittelt werden konnten

	m_Delay= max( m_Tc.wPeriodMin, min( aDelay, m_Tc.wPeriodMax ) ); //Sollintervall ins Intervall einpassen
	return m_Delay;
};
//******************************************************************************
// start/ stop
//******************************************************************************

BOOL TBasicTimer::StartTimer ( const UINT aDelay )
{
	SetDelay(aDelay);
	return StartTimer();
};
//------------------------------------------------------------------------------

BOOL TBasicTimer::StartTimer ( void )
{
	m_WantTimer= Start();
	return m_WantTimer;
};
//------------------------------------------------------------------------------

void TBasicTimer::StopTimer ( void )
{
	Stop();
	m_WantTimer= FALSE;
};

//******************************************************************************
// start/ stop ( + Ereignis sofort auslsen)
//******************************************************************************

BOOL TBasicTimer::StartTimerIm ( void )
{
	BOOL result= StartTimer();
	Immediately(); // auch aufrufen, wenn Timer nicht gestartet werden konnte
	return result;
};
//------------------------------------------------------------------------------

BOOL TBasicTimer::StartTimerIm ( const UINT aDelay)
{
	BOOL result= StartTimer(aDelay);
	Immediately(); // auch aufrufen, wenn Timer nicht gestartet werden konnte
	return result;
};
//------------------------------------------------------------------------------

void TBasicTimer::StopTimerIm ( void )
{
	m_WantTimer= FALSE; // bei Immediately nicht neu starten
	Immediately();
	StopTimer();
};
//------------------------------------------------------------------------------

void TBasicTimer::Immediately ( void )
{
	Stop();
	OnTimer(this);
	if ( m_WantTimer )
		Start(); //knnte in OnTimer FALSE gesetzt worden sein
};

//##############################################################################
//                                     TTimer16                               //
//##############################################################################

TIndList TTimer16::s_TimerList(true, true, false, true); 

//------------------------------------------------------------------------------

TTimer16::TTimer16 ( const UINT aDelay ) : TBasicTimer( aDelay )
{
	m_TimerID= 0; //bislang nicht gestartet
};
//------------------------------------------------------------------------------

TTimer16::~TTimer16 ( void )
{
	StopTimer(); //Timer stoppen
};
//------------------------------------------------------------------------------

BOOL TTimer16::IsStarted ( void )
{
	return (m_TimerID != 0);
};
//------------------------------------------------------------------------------

BOOL TTimer16::Start ( void )
{
	if ( m_Delay == 0 )
		return FALSE; //wenn Intervallgrenzen nicht ermittelt werden konnten

	Stop(); // alten Timer stoppen
	m_TimerID= SetTimer( NULL, 123, m_Delay, (TIMERPROC) & TTimer16::TimedEvent );
	if ( !IsStarted() )
		return FALSE;

	// zur Liste der gestarteten Timer hinzufgen
	s_TimerList.Append(this);
	return TRUE;
};
//------------------------------------------------------------------------------

void TTimer16::Stop ( void )
{
	if ( !IsStarted() )
		return;

	KillTimer( NULL, m_TimerID );
	m_TimerID= NULL; // Timer ist gestoppt
	// aus der Liste der gestarteten Timer entfernen
	s_TimerList.RemoveData(this);
};
//------------------------------------------------------------------------------

void CALLBACK TTimer16::TimedEvent ( HWND, UINT, UINT aTimerID, DWORD )
{
	// Objekt aus der Liste der gestarteten Timer ermitteln; damit endlich auch Zugriff auf Member
	TTimer16 *self= 0;
	for (TIndex i= 0; (i<s_TimerList.GetCount() && !self ); i++) {
		TTimer16 *act= (TTimer16 *)s_TimerList[i];
		if ( act && act->m_TimerID==aTimerID ) self= act; // gefunden :-)
	}
	if ( !self ) { // nicht in der Liste :-(
		const char prae[]= "Unbekannte TIMER-Botschaft fr ID= %d !";
		char *buf= new char[ strlen(prae) + _MAXLENDOUBLE + 1 ];
		sprintf(buf, prae, aTimerID);
		TraceErr(buf);
		_FREELIST(buf);
		return; 
	}		

	self->Stop();
	self->OnTimer( self );
	if ( self->m_WantTimer )
		self->Start(); // <m_WantTimer> knnte in OnTimer FALSE gesetzt worden sein
};

//##############################################################################
//                                     TTimerMM                               //
//##############################################################################

TTimerMM::TTimerMM ( const UINT aDelay ) : TBasicTimer( aDelay )
{
	m_TimerID= 0; //bislang nicht gestartet
};
//------------------------------------------------------------------------------

TTimerMM::~TTimerMM ( void )
{
	StopTimer(); //Timer stoppen
};
//------------------------------------------------------------------------------

BOOL TTimerMM::IsStarted ( void )
{
	return (m_TimerID != 0);
};
//------------------------------------------------------------------------------

BOOL TTimerMM::Start ( void )
{
	if ( m_Delay == 0 )
		return FALSE; //wenn Intervallgrenzen nicht ermittelt werden konnten

	StopTimer(); //Timer stoppen
	m_TimerID= timeSetEvent( m_Delay, m_Tc.wPeriodMin, &TTimerMM::TimedEvent, (DWORD)this, TIME_PERIODIC ); //gibt 0 zurck, falls Timer nict gestartet werden konnte
	return IsStarted();
};
//------------------------------------------------------------------------------

void TTimerMM::Stop ( void )
{
	if ( IsStarted() )
		timeKillEvent( m_TimerID );
	m_TimerID= 0; //Timer ist gestoppt
};
//------------------------------------------------------------------------------

void CALLBACK TTimerMM::TimedEvent ( UINT, UINT, DWORD dwUser, DWORD, DWORD )
{
	TTimerMM *self= (TTimerMM*)dwUser; //damit endlich auch Zugriff auf Member

	self->Stop();
	self->OnTimer( self ); //solange diese Methode rechnet, kann OnTimer nicht erneut aufgerufen werden: deshalb Timer stoppen und neu starten
	if ( self->m_WantTimer )
		self->Start(); //knnte in OnTimer FALSE gesetzt worden sein
};

//##############################################################################
// TInterfaceTimer
//##############################################################################

TInterfaceTimer::TInterfaceTimer ( ITimer *aInterface, const UINT aDelay ) : TTimer( aDelay )
{
	m_lnkInterface= aInterface;
};
//------------------------------------------------------------------------------

void TInterfaceTimer::OnTimer ( TBasicTimer *const aTimer )
{
	if ( m_lnkInterface )
		m_lnkInterface->OnTimer(aTimer);
};

//##############################################################################
// TWindowTimer
//##############################################################################

TWindowTimer::TWindowTimer ( HWND aWnd, const UINT aDelay ) : TTimer( aDelay )
{
	m_lnkWnd= aWnd;
};
//------------------------------------------------------------------------------

void TWindowTimer::OnTimer ( TBasicTimer *const )
{
	if ( m_lnkWnd )
		SendMessage( m_lnkWnd, WM_TIMER, 0, 0 ); //wartet solange bis die Botschaft verarbeitet wurde
};

//##############################################################################
// LASTLINE
//##############################################################################
